-- 收藏册Vo
checkRequire("Lua_CollectionBookConfigData");
Lua_CollectionBookVo = class("Lua_CollectionBookVo");
function Lua_CollectionBookVo:ctor(id, config)
	self:setData(id, config);
end


function Lua_CollectionBookVo:setData(id, config)
	self.id = tonumber(id);
	self.config = config;
	if self.config == nil then
		self.config = getConfig("collection_bag", id);
	end
	self:init_cards();
end

function Lua_CollectionBookVo:dispose()
	self._pushId = nil;
	self._rewardState = nil;
	self.cardIds = nil;
	self.cardDic = nil;
	self._show_detail_ids = nil;
	self._show_detail_Dic = nil;
	self._hasClassicCount = nil;
	self._hasCommonCount = nil;
	self._hasFlashCount = nil;

	self:clearOtherPlayerInfo();
end

function Lua_CollectionBookVo:fromServer(serverData)
	self._pushId = self:checkCardComplete(serverData.push_id);
	self._rewardState = serverData.reward_state;

	local card_infos = serverData.card_state;
	if card_infos and #card_infos > 0 then
		local info;
		for i = 1, #card_infos do
			info = card_infos[i];
			self:updateCard(info.key, info.value);
		end
		self:updateActiveCount();
	end
end

function Lua_CollectionBookVo:updateActiveCount()
	self._hasClassicCount = 0;
	self._hasCommonCount = 0;
	self._hasFlashCount = 0;
	if self.cardIds and #self.cardIds > 0 then
		local cardId;
		for i = 1, #self.cardIds do
			cardId = self.cardIds[i];
			if self.cardDic[cardId] > 1 then
				if cardId ~= self.classic then
					if cardId == self.common then
						self._hasCommonCount = self._hasCommonCount + 1;
					else
						self._hasFlashCount = self._hasFlashCount + 1;
					end
				else
					self._hasClassicCount = self._hasClassicCount + 1;
				end
			end
		end
	end
end

-- 激活数量
function Lua_CollectionBookVo.Get:hasClassicCount()
	return self._hasClassicCount or 0;
end

function Lua_CollectionBookVo.Get:hasCommonCount()
	return self._hasCommonCount or 0;
end

function Lua_CollectionBookVo.Get:hasFlashCount()
	return self._hasFlashCount or 0;
end

-- 总数量
function Lua_CollectionBookVo.Get:classicCount()
	return self._classicCount or 0;
end

function Lua_CollectionBookVo.Get:commonCount()
	return self._commonCount or 0;
end

function Lua_CollectionBookVo.Get:flashCount()
	return self._flashCount or 0;
end

-- 英雄闪卡总数
function Lua_CollectionBookVo.Get:heroFlashCount()
	return self._heroFlashCount or 0;
end

-- 皮肤闪卡总数
function Lua_CollectionBookVo.Get:skinFlashCount()
	return self._skinFlashCount or 0;
end

-- 封面卡牌id
function Lua_CollectionBookVo.Get:pushId()
	local id = self._pushId or 0;
	id = id ~= 0 and id or self.common;

	return id;
end

function Lua_CollectionBookVo.Get:rewardState()
	return self._rewardState or 0;
end

function Lua_CollectionBookVo:init_cards()
	if self.cardIds == nil then
		self.cardIds = {};
		self.cardDic = {};
		self._classicCount = 0;
		self._commonCount = 0;
		self._flashCount = 0;
		self._heroFlashCount = 0; -- 英雄闪卡总数
		self._skinFlashCount = 0; -- 皮肤闪卡总数

		self._show_detail_ids = {};
		self._show_detail_Dic = {};

		local count;
		-- 普卡
		local id = self.common;
		id = self:checkCardComplete(id);
		if id ~= 0 then
			self.cardIds[#self.cardIds + 1] = id;
			self.cardDic[id] = 0;
			self._commonCount = 1;

			count = #self._show_detail_ids + 1;
			self._show_detail_ids[count] = id;
			self._show_detail_Dic[id] = count;
		end

		-- 经典卡
		id = self.classic;
		id = self:checkCardComplete(id);
		if id ~= 0 then
			self.cardIds[#self.cardIds + 1] = id;
			self.cardDic[id] = 0;
			self._classicCount = 1;
		end

		-- 闪卡
		id = self.flash;
		id = self:checkCardComplete(id);
		if id ~= 0 then
			self.cardIds[#self.cardIds + 1] = id;
			self.cardDic[id] = 0;
			self._flashCount = self._flashCount + 1;
			self._heroFlashCount = self._heroFlashCount + 1;

			count = #self._show_detail_ids + 1;
			self._show_detail_ids[count] = id;
			self._show_detail_Dic[id] = count;
		end

		-- 皮肤闪卡
		local cardIds = self.cloth_card;
		if cardIds and #cardIds > 0 then
			for i = 1, #cardIds do
				id = self:checkCardComplete(cardIds[i]);
				if id ~= 0 then
					self.cardIds[#self.cardIds + 1] = id;
					self.cardDic[id] = 0;
					self._flashCount = self._flashCount + 1;
					self._skinFlashCount = self._skinFlashCount + 1;

					count = #self._show_detail_ids + 1;
					self._show_detail_ids[count] = id;
					self._show_detail_Dic[id] = count;
				end
			end
		end
	end
end

function Lua_CollectionBookVo:updateCard(cardId, time)
	cardId = tonumber(cardId);
	if self.cardDic[cardId] then
		self.cardDic[cardId] = tonumber(time);
	end
end

function Lua_CollectionBookVo:getCardTimeByCardId(cardId)
	cardId = tonumber(cardId);
	return self.cardDic[cardId];
end

function Lua_CollectionBookVo:getFlashSortWeightByCard(cardId)
	cardId = tonumber(cardId);
	local time = self:getCardTimeByCardId(cardId) or 0;
	local bPrior = DataCenter.collectionBookUtil:checkCardTypeById(cardId, Lua_CollectionBookUtil.FLASH_CARD);
	local weightSort = bPrior and time * 10 or time;
	return weightSort;
end

function Lua_CollectionBookVo:checkActiveByCardId(cardId)
	local time = self:getCardTimeByCardId(cardId);
	return time and time > 1;
end

function Lua_CollectionBookVo:canActiveByCardId(cardId)
	local time = self:getCardTimeByCardId(cardId);
	if time == nil or time > 1 then
		return false;
	end

	local cardCfg = Lua_CollectionBookConfigData.getConfig(cardId);
	if cardCfg then
		local exchanges = cardCfg.change_item;
		if exchanges and #exchanges > 0 then
			local itemId;
			local itemNum;
			local hasNum;
			local bCheck = true;
			for i = 1, #exchanges do
				itemId = exchanges[i][1];
				itemNum = exchanges[i][2];
				hasNum = DataCenter.packData:getItemCountById(itemId);
				if itemNum > hasNum then
					bCheck = false;
					break;
				end
			end
			return bCheck;
		end
	end

	return false;
end

function Lua_CollectionBookVo.Get:receive_reward_redpoint()
	return self.rewardState == 1;
end

function Lua_CollectionBookVo:show_card_redpoint(cardId)
	cardId = tonumber(cardId);

	if self.receive_reward_redpoint then
		return true;
	end

	return self:canActiveByCardId(cardId);
end

function Lua_CollectionBookVo.Get:card_type()
	if self._card_type == nil then
		if self.sub_type == Lua_CollectionBookUtil.MODEL_CARD or self.sub_type == Lua_CollectionBookUtil.ONLY_CARD then -- 神兵列传
			self._card_type = 1;
		elseif self.sub_type == Lua_CollectionBookUtil.FRAME_CARD then -- 名人堂
			self._card_type = 2;
		elseif self.sub_type == Lua_CollectionBookUtil.CONTACT_CARD then -- 联动卡
			self._card_type = 3;
		end
	end

	return self._card_type;
end

function Lua_CollectionBookVo:checkCardComplete(cardId)
	cardId = tonumber(cardId);
	local bCheck = DataCenter.collectionBookUtil:checkCompletedCard(cardId);
	cardId = bCheck and cardId or 0;
	return cardId;
end

-- function Lua_CollectionBookVo:




-- 其他玩家
function Lua_CollectionBookVo:fromServerOtherPlayer(serverData)
	local cardId = tonumber(serverData.push_id);
	self._otherPushId = cardId;
	self:updateOtherCard(cardId, serverData.unlock_tick);

	-- 经典卡
	cardId = self.classic;
	self:updateOtherCard(cardId, serverData.classic_tick);
	-- 普卡
	cardId = self.common;
	self:updateOtherCard(cardId, serverData.common_tick);

	self._otherFlashCount = serverData.flash_amount;
end

function Lua_CollectionBookVo:clearOtherPlayerInfo() 
	self._otherPushId = nil;
	self.otherCardDic = nil;
	self._otherFlashCount = nil;
end

function Lua_CollectionBookVo:updateOtherCard(cardId, time)
	cardId = tonumber(cardId);
	if self.otherCardDic == nil then
		self.otherCardDic = {};
	end
	
	if self:getCardTimeByCardId(cardId) then
		self.otherCardDic[cardId] = tonumber(time);
	end
end

function Lua_CollectionBookVo:getOtherCardTimeByCardId(cardId)
	cardId = tonumber(cardId);
	if self.otherCardDic == nil then
		return;
	end

	return self.otherCardDic[cardId];
end

function Lua_CollectionBookVo:checkOtherActiveByCardId(cardId)
	local time = self:getOtherCardTimeByCardId(cardId);
	return time and time > 1;
end

function Lua_CollectionBookVo.Get:otherPushId()
	local id = self._otherPushId or 0;
	id = id ~= 0 and id or self.common;

	return id;
end

-- 激活数量
function Lua_CollectionBookVo.Get:hasOtherClassicCount()
	local cardId = self.classic;
	local time = self:getOtherCardTimeByCardId(cardId);
	if time and time > 1 then
		return 1;
	else
		return 0;
	end
end

function Lua_CollectionBookVo.Get:hasOtherCommonCount()
	local cardId = self.common;
	local time = self:getOtherCardTimeByCardId(cardId);
	if time and time > 1 then
		return 1;
	else
		return 0;
	end
end

function Lua_CollectionBookVo.Get:hasOtherFlashCount()
	return self._otherFlashCount or 0;
end

function Lua_CollectionBookVo.Get:show_detail_ids()
	return self._show_detail_ids;
end

function Lua_CollectionBookVo:getShowDetailIndexByCardId(cardId)
	cardId = tonumber(cardId);
	return self._show_detail_Dic[cardId];
end



-- 新版：检测卡牌是否显示红点
function Lua_CollectionBookVo:checkCardItemRedPoint(cardId)
	cardId = tonumber(cardId);
	if self:getCardTimeByCardId(cardId) == nil then
		return false;
	end

	local bCheck = false;
	if cardId == self.common then -- 普卡
		if self.receive_reward_redpoint then
			bCheck = true;
		else
			bCheck = self:canActiveByCardId(cardId) == true;
		end
	elseif cardId == self.classic then -- 经典卡
		bCheck = false;
	else
		local bActive = self:checkActiveByCardId(cardId);
		if bActive then
			bCheck = DataCenter.collectionBookData:checkFlashCardProp(cardId);
		else
			bCheck = self:canActiveByCardId(cardId) == true;
		end

	end
	
	return bCheck;
end












-------------------------配置表数据---------------------------------
-- 类型
function Lua_CollectionBookVo.Get:sub_type()
	if self.config then
		return self.config.sub_type;
	end
end

-- 普卡
function Lua_CollectionBookVo.Get:common()
	if self.config then
		return tonumber(self.config.common);
	end
end

-- 经典
function Lua_CollectionBookVo.Get:classic()
	if self.config then
		return tonumber(self.config.classic);
	end
end

-- 闪卡
function Lua_CollectionBookVo.Get:flash()
	if self.config then
		return tonumber(self.config.flash);
	end
end

-- 拥有皮肤闪卡
function Lua_CollectionBookVo.Get:cloth_card()
	if self.config then
		return self.config.cloth_card;
	end
end

-- 奖励道具
function Lua_CollectionBookVo.Get:reward()
	if self.config then
		return self.config.reward;
	end
end

-- 排序
function Lua_CollectionBookVo.Get:sort()
	if self.config then
		return self.config.sort;
	end
end