Lua_CollectionBookUtil = class("Lua_CollectionBookUtil");
-- 普卡类型
Lua_CollectionBookUtil.MODEL_CARD = 1; -- 模型卡
Lua_CollectionBookUtil.ONLY_CARD = 2; -- 仅有卡
Lua_CollectionBookUtil.FRAME_CARD = 3; -- 名人堂
Lua_CollectionBookUtil.CONTACT_CARD = 4; --联动卡

-- 卡类型
Lua_CollectionBookUtil.COMMON_CARD = 1; -- 普卡
Lua_CollectionBookUtil.CLASSIC_CARD = 2; -- 经典卡
Lua_CollectionBookUtil.FLASH_CARD = 3; -- 闪卡
Lua_CollectionBookUtil.CLOTH_CARD = 4; -- 皮肤闪卡
function Lua_CollectionBookUtil:ctor()
end

function Lua_CollectionBookUtil:dispose()
	self._allBagIds = nil;
	self._commonTCount = nil; --普卡总数
	self._flashTCount = nil; -- 闪卡总数
	self._heroFlashTCount = nil; -- 英雄闪卡总数
	self._skinFlashTCount = nil; -- 皮肤闪卡总数
	self._classicTCount = nil;-- 经典总数

	self._weaponCTCount = nil; --神兵列传普卡数量
	self._weaponFTCount = nil; --神兵列传闪卡数量
	self._weaponBagIds = nil; --神兵列传卡包id

	self._frameCTCount = nil;  -- 名人堂普卡数量
	self._frameFTCount = nil;  -- 名人堂闪卡数量
	self._frameBagIds = nil;  --名人堂卡包id

	self._contactCTCount = nil; -- 联动卡普卡数量
	self._contactFTCount = nil; -- 联动卡闪卡数量
	self._contactBagIds = nil;  -- 联动卡卡包id
end

-- 判断该卡牌是否为已制作完成卡牌
function Lua_CollectionBookUtil:checkCompletedCard(cardId)
	local bCheck;
	local cardCfg = Lua_CollectionBookConfigData.getConfig(cardId);
	if cardCfg and cardCfg.bCompletion then
		bCheck = true;
	end

	return bCheck;
end

function Lua_CollectionBookUtil:initCardBagInfo()
	if self._allBagIds == nil then
		self._allBagIds = {};
		self._commonTCount = 0; --普卡总数
		self._flashTCount = 0; -- 闪卡总数

		-- 收藏册旧排版
		self._heroFlashTCount = 0; -- 英雄闪卡总数
		self._skinFlashTCount = 0; -- 皮肤闪卡总数
		self._classicTCount = 0;-- 经典总数

		self._weaponCTCount = 0; --神兵列传普卡数量
		self._weaponFTCount = 0; --神兵列传闪卡数量
		self._weaponBagIds = {}; --神兵列传卡包ids

		self._classicBagIds = {}; -- 经典卡

		self._frameCTCount = 0;  -- 名人堂普卡数量
		self._frameFTCount = 0;  -- 名人堂闪卡数量
		self._frameBagIds = {};  --名人堂卡包ids

		self._contactCTCount = 0; -- 联动卡普卡数量
		self._contactFTCount = 0; -- 联动卡闪卡数量
		self._contactBagIds = {};  -- 联动卡卡包ids


		-- 收藏册新排版
		self._weaponCardIds = {}; --神兵列传卡牌ids
		self._flashCardIds = {}; -- 闪卡ids
		self._classicCardIds = {}; -- 经典卡ids
		self._frameCardIds = {};  --名人堂卡牌ids
		self._contactCardIds = {};  -- 联动卡卡牌ids


		local config = getConfig("collection_bag");
		for k, v in pairs(config) do
			if self:bShieldCardBag(k) ~= true then
				self._allBagIds[#self._allBagIds + 1] = k;

				local vo = DataCenter.collectionBookData:getCollectionBookVoById(k);
				local card_type = vo.card_type;
				local commonNum = vo.commonCount;
				local flashNum = vo.flashCount;

				------------------------------------
				local classicNum = vo.classicCount;
				local heroFlashNum = vo.heroFlashCount;
				local skinFlashNum = vo.skinFlashCount;
				self._classicTCount = self._classicTCount + classicNum;
				self._heroFlashTCount = self._heroFlashTCount + heroFlashNum;
				self._skinFlashTCount = self._skinFlashTCount + skinFlashNum;
				------------------------------------


				self._commonTCount = self._commonTCount + commonNum;
				self._flashTCount = self._flashTCount + flashNum;

				if card_type == 1 then -- 神兵列传
					self._weaponCTCount = self._weaponCTCount + commonNum;
					self._weaponFTCount = self._weaponFTCount + flashNum;
					self._weaponBagIds[#self._weaponBagIds + 1] = k;
					self._weaponCardIds[#self._weaponCardIds + 1] = vo.common;
				elseif card_type == 2 then -- 名人堂
					self._frameCTCount = self._frameCTCount + commonNum;
					self._frameFTCount = self._frameFTCount + flashNum;
					self._frameBagIds[#self._frameBagIds + 1] = k;
					self._frameCardIds[#self._frameCardIds + 1] = vo.common;
				elseif card_type == 3 then -- 联动卡
					self._contactCTCount = self._contactCTCount + commonNum;
					self._contactFTCount = self._contactFTCount + flashNum;
					self._contactBagIds[#self._contactBagIds + 1] = k;
					self._contactCardIds[#self._contactCardIds + 1] = vo.common;
				end

				if vo.card_type == 1 and vo.classic and Lua_CollectionBookConfigData.getConfig(vo.classic) then
					self._classicBagIds[#self._classicBagIds + 1] = k;
					self._classicCardIds[#self._classicCardIds + 1] = vo.classic;
				end

				if vo.flash and Lua_CollectionBookConfigData.getConfig(vo.flash) then
					self._flashCardIds[#self._flashCardIds + 1] = vo.flash;
				end

				if vo.cloth_card and #vo.cloth_card > 0 then
					local cardId;
					for i = 1, #vo.cloth_card do
						cardId = vo.cloth_card[i];
						if Lua_CollectionBookConfigData.getConfig(cardId) then
							self._flashCardIds[#self._flashCardIds + 1] = cardId;
						end
					end
				end
			end
		end

		self._allBagIds = self:sortBagIds(self._allBagIds);
		self._weaponBagIds = self:sortBagIds(self._weaponBagIds);
		self._classicBagIds = self:sortBagIds(self._classicBagIds);
		self._frameBagIds = self:sortBagIds(self._frameBagIds);
		self._contactBagIds = self:sortBagIds(self._contactBagIds);

		self._weaponCardIds = self:sortCardIds(self._weaponCardIds);
		self._flashCardIds = self:sortCardIds(self._flashCardIds);
		self._classicCardIds = self:sortCardIds(self._classicCardIds);
		self._frameCardIds = self:sortCardIds(self._frameCardIds);
		self._contactCardIds = self:sortCardIds(self._contactCardIds);
	end
end

function Lua_CollectionBookUtil:sortBagIds(ids)
	if ids and #ids > 1 then
		table.sort( ids, function(a, b)
				local AVo = DataCenter.collectionBookData:getCollectionBookVoById(a);
				local BVo = DataCenter.collectionBookData:getCollectionBookVoById(b);
				local ASort = AVo and AVo.sort or 0; 
				local BSort = BVo and BVo.sort or 0;
				return ASort < BSort;
			end
			 );
	end
	return ids;
end

function Lua_CollectionBookUtil:sortCardIds(ids)
	if ids and #ids > 1 then
		table.sort( ids, function(a, b)
				local aBagId = self:getCardBagIdByCBId(a);
				local bBagId = self:getCardBagIdByCBId(b);

				local AVo = DataCenter.collectionBookData:getCollectionBookVoById(aBagId);
				local BVo = DataCenter.collectionBookData:getCollectionBookVoById(bBagId);
				local ASort = AVo and AVo.sort or 0; 
				local BSort = BVo and BVo.sort or 0;
				if ASort == BSort then
					return a < b;
				else
					return ASort < BSort;
				end
			end
			 );
	end
	return ids;
end

function Lua_CollectionBookUtil.Get:allBagIds()
	if self._allBagIds == nil then
		self:initCardBagInfo();
	end
	return self._allBagIds;
end

--普卡总数
function Lua_CollectionBookUtil.Get:commonTCount()
	if self._commonTCount == nil then
		self:initCardBagInfo();
	end
	return self._commonTCount;
end

-- 闪卡总数
function Lua_CollectionBookUtil.Get:flashTCount()
	if self._flashTCount == nil then
		self:initCardBagInfo();
	end
	return self._flashTCount;
end






----------------------------- 收藏册旧排版------------------------
-- 英雄闪卡总数
function Lua_CollectionBookUtil.Get:heroFlashTCount()
	if self._heroFlashTCount == nil then
		self:initCardBagInfo();
	end
	return self._heroFlashTCount;
end

-- 皮肤闪卡总数
function Lua_CollectionBookUtil.Get:skinFlashTCount()
	if self._skinFlashTCount == nil then
		self:initCardBagInfo();
	end
	return self._skinFlashTCount;
end

-- 经典总数
function Lua_CollectionBookUtil.Get:classicTCount()
	if self._classicTCount == nil then
		self:initCardBagInfo();
	end
	return self._classicTCount;
end

-- 神兵列传卡包ids
function Lua_CollectionBookUtil.Get:weaponBagIds()
	if self._weaponBagIds == nil then
		self:initCardBagInfo();
	end
	return self._weaponBagIds;
end

-- 神兵列传普卡数量
function Lua_CollectionBookUtil.Get:weaponCTCount()
	if self._weaponCTCount == nil then
		self:initCardBagInfo();
	end
	return self._weaponCTCount;
end

-- 神兵列传闪卡数量
function Lua_CollectionBookUtil.Get:weaponFTCount()
	if self._weaponFTCount == nil then
		self:initCardBagInfo();
	end
	return self._weaponFTCount;
end

-- 经典卡ids
function Lua_CollectionBookUtil.Get:classicBagIds()
	if self._classicBagIds == nil then
		self:initCardBagInfo();
	end
	return self._classicBagIds;
end

-- 名人堂卡包ids
function Lua_CollectionBookUtil.Get:frameBagIds()
	if self._frameBagIds == nil then
		self:initCardBagInfo();
	end
	return self._frameBagIds;
end

-- 名人堂普卡数量
function Lua_CollectionBookUtil.Get:frameCTCount()
	if self._frameCTCount == nil then
		self:initCardBagInfo();
	end
	return self._frameCTCount;
end

 -- 名人堂闪卡数量
function Lua_CollectionBookUtil.Get:frameFTCount()
	if self._frameFTCount == nil then
		self:initCardBagInfo();
	end
	return self._frameFTCount;
end

-- 联动卡卡包ids
function Lua_CollectionBookUtil.Get:contactBagIds()
	if self._contactBagIds == nil then
		self:initCardBagInfo();
	end
	return self._contactBagIds;
end

-- 联动卡普卡数量
function Lua_CollectionBookUtil.Get:contactCTCount()
	if self._contactCTCount == nil then
		self:initCardBagInfo();
	end
	return self._contactCTCount;
end

-- 联动卡闪卡数量
function Lua_CollectionBookUtil.Get:contactFTCount()
	if self._contactFTCount == nil then
		self:initCardBagInfo();
	end
	return self._contactFTCount;
end







----------------------------- 收藏册新排版------------------------
-- 神兵列传卡牌ids
function Lua_CollectionBookUtil.Get:weaponCardIds()
	if self._weaponCardIds == nil then
		self:initCardBagInfo();
	end
	return self._weaponCardIds;
end

-- 闪卡ids
function Lua_CollectionBookUtil.Get:flashCardIds()
	if self._flashCardIds == nil then
		self:initCardBagInfo();
	end
	return self._flashCardIds;
end

-- 经典卡ids
function Lua_CollectionBookUtil.Get:classicCardIds()
	if self._classicCardIds == nil then
		self:initCardBagInfo();
	end
	return self._classicCardIds;
end

-- 名人堂卡牌ids
function Lua_CollectionBookUtil.Get:frameCardIds()
	if self._frameCardIds == nil then
		self:initCardBagInfo();
	end
	return self._frameCardIds;
end

-- 联动卡卡牌ids
function Lua_CollectionBookUtil.Get:contactCardIds()
	if self._contactCardIds == nil then
		self:initCardBagInfo();
	end
	return self._contactCardIds;
end













-- 屏蔽卡包
function Lua_CollectionBookUtil:bShieldCardBag(bagId)
	local bShield = false;
	local openState, openDay = DataCenter.heroUtil:getNewHeroFuncOpenState(bagId, Lua_NewHeroFuncOpen.collection_book);
	if not openState then
		bShield = true;
	end
	return bShield;
end

-- 根据卡牌id,通过公式计算出是否为该type类型
function Lua_CollectionBookUtil:checkCardTypeById(cardId, type)
	cardId = tonumber(cardId);
	if getConfig("collection_book", cardId) == nil then
		return false;
	end

	if cardId and type then
		local residue = Mathf.Floor( (cardId % 1000) / 100 );
		if type == Lua_CollectionBookUtil.COMMON_CARD then -- 普卡
			return residue == 1;
		elseif type == Lua_CollectionBookUtil.CLASSIC_CARD then -- 经典 
			return residue == 2;
		elseif type == Lua_CollectionBookUtil.FLASH_CARD then -- 闪卡
			return residue == 3;
		elseif type == Lua_CollectionBookUtil.CLOTH_CARD then -- 皮肤闪卡
			return residue == 4;
		end
	end
end

-- 根据闪卡id（hero_des-闪卡表）计算出卡包id
function Lua_CollectionBookUtil:getCardBagIdByFlashId(flashId)
	if flashId ~= nil then
		return Mathf.Floor(flashId / 100); -- 闪卡id = 卡包id * 100 + 类型
	end

	return nil;
end

-- 根据收藏册卡牌id（collection_book-收藏册表）计算出卡包id
function Lua_CollectionBookUtil:getCardBagIdByCBId(cardId)
	if cardId ~= nil then
		return Mathf.Floor(cardId / 1000); 
	end

	return nil;
end

-- 显示闪卡特效
function Lua_CollectionBookUtil:checkShowEffectByCardId(cardId)
	local isShowEffect = self:checkCardTypeById(cardId, Lua_CollectionBookUtil.FLASH_CARD) or 
						 self:checkCardTypeById(cardId, Lua_CollectionBookUtil.CLOTH_CARD);
	return isShowEffect;
end

-- 封面最多展示个数
function Lua_CollectionBookUtil.Get:CBMaxCount()
	return getConfig("collection_book_const", "const", "max_count");
end


-- 是否有联动卡
function Lua_CollectionBookUtil.Get:hasContactCard()
	return #self.contactBagIds > 0;
end

-- 获取英雄获取途径
function Lua_CollectionBookUtil:getCollectionHeroPathId()
	return getConfig("collection_book_const", "const", "hero_collect_id");
end

-- 获取闪卡获取途径
function Lua_CollectionBookUtil:getCollectionFlashPathId()
	return getConfig("collection_book_const", "const", "flash_collect_id");
end

-- 获取皮肤获取途径
function Lua_CollectionBookUtil:getCollectionSkinPathId()
	return getConfig("collection_book_const", "const", "skin_collect_id");
end

-- 获取所有需要激活的卡牌且是已制作完成的
function Lua_CollectionBookUtil:getAllNeedActiveCards()
	if self.needActiveCards == nil then
		self.needActiveCards = {};
		local bagId;
		local bShield;-- 屏蔽英雄
		local cardCfg;
		local cBookVo;
		local config = getConfig("collection_book");

		for k, v in pairs(config) do
			cardCfg = Lua_CollectionBookConfigData.getConfig(k);
			bagId = cardCfg.hero_id;
			bShield = self:bShieldCardBag(bagId);
			if bShield ~= true then
				cBookVo = DataCenter.collectionBookData:getCollectionBookVoById(bagId);
				if cBookVo and cardCfg.bCompletion and cardCfg.change_item then
					self.needActiveCards[#self.needActiveCards + 1] = k;
				end
			end
		end
	end
	return self.needActiveCards;
end

-- 根据英雄id判断出普卡（经典卡）是否制作完成
-- 普卡、经典卡同时制作完成
function Lua_CollectionBookUtil:checkCompletedCardByHeroId(heroId)
	local cBookVo = DataCenter.collectionBookData:getCollectionBookVoById(heroId);
	if cBookVo then
		local cardId = cBookVo.common;
		return self:checkCompletedCard(cardId);
	end
	return false;
end

-- 排行榜上榜限制
function Lua_CollectionBookUtil.Get:showRankLimit()
	return getConfig("collection_book_const", "const", "show_limit");
end










------------------------------羁绊--------------------------------------------
function Lua_CollectionBookUtil.Get:allFetterId()
	if self._allFetterId == nil then
		self._allFetterId = {};

		local cfg = getConfig("hero_fetter");
		for k, v in pairs(cfg) do
			self._allFetterId[#self._allFetterId + 1] = k;
		end

		table.sort( self._allFetterId, function(a, b) 
				local Asort = getConfig("hero_fetter", a, "sort"); 
				local Bsort = getConfig("hero_fetter", b, "sort");
				return Asort < Bsort;
			end 
		);
	end

	return self._allFetterId;
end