Lua_DivineDomainTrialData = class("Lua_DivineDomainTrialData")
function Lua_DivineDomainTrialData:ctor()
end

function Lua_DivineDomainTrialData:dispose()
	self.divineDomainTrialDic = nil; --
	self.divineDomainTrialOpenedDic = nil;
	self.divineDomainTrialCountDic = nil;
	self.divineDomainTrialBuffDic = nil;
	self.divineDomainTrialSpecialDic = nil;
	self.divineDomainTrialSelectBuffDic = nil; -- 增益选择数据
	self.divineDomainTrialSBuffTipType = nil; -- 首次进入特殊副本时时空乱流的类型
	self.divineDomainTrialBShieldBuffSelect = nil; -- 是否屏蔽buff提示
	self.divineDomainTrialBuffRefleshCount = nil; -- buff刷新次数
	self.curSelectingBuffId = nil;
	self.rewardDic = nil;
	self.rewardIds = nil;
	self._bBuffSelectMode = nil;
	self.commonTrialMaxLeveNum = nil; -- 普通系列副本中最大的关卡
	self.bWipeOut = nil; -- 是否扫荡刷新关卡数据

	-- self.divineDomainTrialLevelDic = nil;
end

--请求神域试炼界面(返回)
function Lua_DivineDomainTrialData:fromServer50210001(msg)
	if msg == nil then
		return;
	end

	local _trialId = tonumber(msg.type);
	local _residueTime = tonumber(msg.close_tick);
	local _bOpened = DataCenter.divineDomainTrialUtil:checkOpened(_residueTime);
	local _maxFightingLevel = msg.pass_stage;
	self:updateDivineDomainTrial(_trialId, _bOpened, _residueTime, _maxFightingLevel, msg.pass_state);

	if _bOpened then
		local key = DataCenter.divineDomainTrialUtil:getBuffTypeByTrialId(_trialId);
		self:setBuffDicByTrialId(key, tonumber(msg.buff)); -- 已选择的buff
		self:setCountDicByTrialId(_trialId, msg.challenge_times);
	end

	if DataCenter.divineDomainTrialUtil:checkSpecialTrial(_trialId) then -- 特殊关卡
		self:updateSpecialTrialData(msg.boss_buff, _maxFightingLevel, tonumber(msg.boss_health));

		-- 通关后buff需清空,且打开增益选择界面
		self:cleanSelectBuffData(_trialId);
	end

	-- self:updateRedPointByTrialId(_trialId);
	self:updateRedPoint()

	self.bWipeOut = msg.is_repeat == 1;
end

-- 通知神域试炼每日刷新
function Lua_DivineDomainTrialData:fromServer80210001(msg)
	if msg == nil then
		return;
	end

	self.resetFlag = msg.flag;--通知更新标志

end

-- 请求增益界面(返回)
function Lua_DivineDomainTrialData:fromServer50210002(msg)
	if msg == nil then
		return;
	end

	self:setSelectBuffData(msg.buff_list, msg.refresh_times, msg.type);
end

-- 请求选择增益(返回)
function Lua_DivineDomainTrialData:fromServer50210003(msg)
	if msg == nil then
		return;
	end

	self:setBuffDicByTrialId(msg.type, tonumber(msg.buff_id));
end

-- 请求刷新增益界面(返回)
function Lua_DivineDomainTrialData:fromServer50210006(msg)
	if msg == nil then
		return;
	end

	self:setSelectBuffData(msg.buff_list, msg.refresh_times, msg.type);
	self:setBuffDicByTrialId(msg.type, tonumber(msg.buff));
end

-- 请求神域试炼成就奖励界面(返回)
function Lua_DivineDomainTrialData:fromServer50210007(msg)
	if msg == nil then
		return;
	end

	if self.rewardDic == nil then
		self.rewardDic = {};
	end

	local reward_type = msg.reward_type; --  1成就奖励 2 挑战奖励
	if self.rewardDic[reward_type] == nil then
		self.rewardDic[reward_type] = {};
	end
	self:clearRewardIdByType(reward_type);
	local id;
	local value;
	local obj = msg.reward_maps;
	for i = 1, #obj do
		id = tonumber(obj[i].key);
		value = tonumber(obj[i].value);
		self.rewardDic[reward_type][id] = value; -- 0不可领取 1可领取 2已领取
		self:setRewardIdList(reward_type, id);
	end

	self:updateRedPoint();
end

-- 通知服务端特殊关卡重置时boss特性变更时窗口已打开
function Lua_DivineDomainTrialData:fromServer50210011(msg)
	if msg == nil then
		return;
	end

	self.bShowNoticeBossSkillChange = msg.flag; -- 0未弹出 1已弹出
	self.showNoticeBossDiff = msg.difficulty - msg.last_difficulty;
end

function Lua_DivineDomainTrialData:setRewardIdList(type, id)
	if self.rewardIds == nil then
		self.rewardIds = {};
	end

	local arr = self.rewardIds[type];
	if arr == nil then
		arr = Array();
		self.rewardIds[type] = arr;
	end

	if arr:indexOf(id) == -1 then
		arr:add(id)
	end
end

function Lua_DivineDomainTrialData:clearRewardIdByType(type)
	if self.rewardIds == nil or self.rewardIds[type] == nil then
		return
	end

	self.rewardIds[type]:dispose();
	self.rewardIds[type]:clear();
	self.rewardIds[type] = nil;
end

function Lua_DivineDomainTrialData:getRewardIdListByType(type)
	if self.rewardIds == nil or self.rewardIds[type] == nil then
		return;
	end
	return self.rewardIds[type];
end

function Lua_DivineDomainTrialData:getRewardStateById(reward_type, id)
	if self.rewardDic == nil or self.rewardDic[reward_type] == nil or self.rewardDic[reward_type][id] == nil then
		return;
	end
	return self.rewardDic[reward_type][id];
end

function Lua_DivineDomainTrialData:updateDivineDomainTrial(_trialId, _bOpened, _clockTime, _maxFightingLevel, _passState)
	if _trialId == nil then
		return;
	end

	local divineDomainTrialId = _trialId;
	if self.divineDomainTrialDic == nil then
		self.divineDomainTrialDic = {};
	end

	local data = self:getDivineDomainTrialData(divineDomainTrialId);
	if data == nil then
		data = {
			bOpened = _bOpened,
			clockTime = _clockTime,
			maxFightingLevel = _maxFightingLevel,
			passState = _passState
		};
		self.divineDomainTrialDic[divineDomainTrialId] = data;
	else
		data.bOpened = _bOpened;
		data.clockTime = _clockTime;
		data.maxFightingLevel = _maxFightingLevel;
		data.passState = _passState;
	end

	self:recordCommonTrialLevelId(_trialId, _maxFightingLevel);
end

-- 记录普通系列副本最大通关id
function Lua_DivineDomainTrialData:recordCommonTrialLevelId(trialId, maxLevelId)
	if self.commonTrialMaxLeveNum == nil then
		self.commonTrialMaxLeveNum = 0;
	end

	local bSpecialTrial = DataCenter.divineDomainTrialUtil:checkSpecialTrial(trialId);
	if not bSpecialTrial then 
		local levelCfg = DataCenter.divineDomainTrialUtil:getDivineDomainTrialLevelConfig(maxLevelId);
		if levelCfg ~= nil then
			local trialId = levelCfg.index;
			local floor = levelCfg.floor or 0;
			local trialData = self:getDivineDomainTrialData(trialId);
			if trialData ~= nil then
				if trialData.passState ~= 2 then
					floor = floor - 1;
				end
				floor = floor <= 0 and 0 or floor;
				self.commonTrialMaxLeveNum = self.commonTrialMaxLeveNum < floor and floor or self.commonTrialMaxLeveNum;
			end
		end
	end
end

function Lua_DivineDomainTrialData:getDivineDomainTrialData(divineDomainTrialId)
	if self.divineDomainTrialDic == nil or self.divineDomainTrialDic[divineDomainTrialId] == nil then
		return;
	end
	return self.divineDomainTrialDic[divineDomainTrialId];
end

-- 特殊试炼的monster
function Lua_DivineDomainTrialData:updateSpecialTrialData(boss_buff, curMaxFightLevel, boss_health)
	local _diffLevel = 1;
	local levelCfg = DataCenter.divineDomainTrialUtil:getDivineDomainTrialLevelConfig(curMaxFightLevel);
	if levelCfg ~= nil then
		_diffLevel = levelCfg.difficulty;
	end

	local bLastLevel = DataCenter.divineDomainTrialUtil:checkLastLevel(curMaxFightLevel);
	local _bossHP = bLastLevel and 0 or boss_health;
	local _hurtValue = bLastLevel and boss_health or 0;
	local _skills = {};

	local trialCfg = DataCenter.divineDomainTrialUtil:getDivineDomainTrialConfig(Lua_DivineDomainTrialUtil.SPECIALTRIAL);
	if trialCfg ~= nil and trialCfg.show_skill ~= nil then
		local obj = trialCfg.show_skill;
		for i = 1, #obj do
			_skills[#_skills + 1] = obj[i];
		end
	end
	
	for i = 1, #boss_buff do
		_skills[#_skills + 1] = tonumber(boss_buff[i]);
	end

	local data = self:getSpecialTrialData();
	if data == nil then
		data = {
			diffLevel = _diffLevel,
			bossHP = _bossHP,
			hurtValue = _hurtValue,
			skills = _skills
		};
		self.divineDomainTrialSpecialDic = data;
	else
		data.diffLevel = _diffLevel;
		data.bossHP = _bossHP;
		data.hurtValue = _hurtValue;
		data.skills = _skills;
	end
end

function Lua_DivineDomainTrialData:getSpecialTrialData()
	return self.divineDomainTrialSpecialDic;
end

-- function Lua_DivineDomainTrialData:setTrialLevelData(_trialId, _levelId, _levelState)
-- 	if self.divineDomainTrialLevelDic == nil then
-- 		self.divineDomainTrialLevelDic = {};
-- 	end

-- 	-- self.divineDomainTrialLevelDic[levelId] = levelState;
-- 	self.divineDomainTrialLevelDic[trialId] = { }
-- end

-- function Lua_DivineDomainTrialData:getLevelDataByTrialId(trialId)
-- 	if self.divineDomainTrialLevelDic == nil or self.divineDomainTrialLevelDic[trialId] == nil then
-- 		return;
-- 	end
-- 	return self.divineDomainTrialLevelDic[trialId];
-- end

-- 是否打开过
function Lua_DivineDomainTrialData:setOpenedDicByTrialId(trialId, bOpened)
	if self.divineDomainTrialOpenedDic == nil then
		self.divineDomainTrialOpenedDic = {};
	end

	local key = DataCenter.divineDomainTrialUtil:getTrialIndexByTrialId(trialId);
	self.divineDomainTrialOpenedDic[key] = bOpened;

	if bOpened then
		self:updateRedPoint();
	end
end

function Lua_DivineDomainTrialData:getOpenedDicByTrialId(trialId)
	if self.divineDomainTrialOpenedDic == nil then
		return;
	end

	local key = DataCenter.divineDomainTrialUtil:getTrialIndexByTrialId(trialId);
	return self.divineDomainTrialOpenedDic[key] ;
end

-- 剩余的免费次数
function Lua_DivineDomainTrialData:setCountDicByTrialId(trialId, count)
	if self.divineDomainTrialCountDic == nil then
		self.divineDomainTrialCountDic = {};
	end

	local key = DataCenter.divineDomainTrialUtil:getTrialIndexByTrialId(trialId);
	self.divineDomainTrialCountDic[key] = count;
end

function Lua_DivineDomainTrialData:getCountDicByTrialId(trialId)
	if self.divineDomainTrialCountDic == nil then
		return;
	end

	local key = DataCenter.divineDomainTrialUtil:getTrialIndexByTrialId(trialId);
	return self.divineDomainTrialCountDic[key] ;
end

-- 选择的buff
function Lua_DivineDomainTrialData:setBuffDicByTrialId(type, buffId)
	if self.divineDomainTrialBuffDic == nil then
		self.divineDomainTrialBuffDic = {};
	end

	self.divineDomainTrialBuffDic[type] = tonumber(buffId);
end

function Lua_DivineDomainTrialData:getBuffDicByTrialId(trialId)
	if self.divineDomainTrialBuffDic == nil then
		return;
	end

	local key = DataCenter.divineDomainTrialUtil:getBuffTypeByTrialId(trialId);
	return self.divineDomainTrialBuffDic[key];
end

-- 增益选择窗口
function Lua_DivineDomainTrialData:setSelectBuffData(buff_list, refresh_times, type)
	if self.divineDomainTrialSelectBuffDic == nil then
		self.divineDomainTrialSelectBuffDic = {};
	end

	local _buffs = {};
	for i = 1, #buff_list do
		_buffs[i] = tonumber(buff_list[i]);
	end

	self.divineDomainTrialSelectBuffDic[type] = _buffs;
	self:setBuffRefleshCount(refresh_times, type);
end

function Lua_DivineDomainTrialData:getSelectBuffDataByTrialId(trialId)
	if self.divineDomainTrialSelectBuffDic == nil then
		return;
	end

	local key = DataCenter.divineDomainTrialUtil:getBuffTypeByTrialId(trialId);
	return self.divineDomainTrialSelectBuffDic[key];
end

-- 清空buff选择数据
function Lua_DivineDomainTrialData:cleanSelectBuffData(trialId)
	if self.divineDomainTrialSelectBuffDic == nil then
		return;
	end

	local key = DataCenter.divineDomainTrialUtil:getBuffTypeByTrialId(trialId);
	self.divineDomainTrialSelectBuffDic[key] = nil;
end

function Lua_DivineDomainTrialData:setBuffRefleshCount(refresh_times, type)
	if self.divineDomainTrialBuffRefleshCount == nil then
		self.divineDomainTrialBuffRefleshCount = {};
	end

	self.divineDomainTrialBuffRefleshCount[type] = refresh_times;
end

function Lua_DivineDomainTrialData:getBuffRefleshCount(trialId)
	if self.divineDomainTrialBuffRefleshCount == nil then
		return;
	end

	local type = DataCenter.divineDomainTrialUtil:getBuffTypeByTrialId(trialId);
	return self.divineDomainTrialBuffRefleshCount[type];
end

-- 根据关卡是否已通关
function Lua_DivineDomainTrialData:checkLevelState(levelId) --返回关卡状态    1:通关  2： 可挑战 3： 不可挑战
	local cfg = DataCenter.divineDomainTrialUtil:getDivineDomainTrialLevelConfig(levelId);
	if cfg == nil then
		return 3;
	end

	local trialId = cfg.index;
	local trialData = self:getDivineDomainTrialData(trialId);
	if trialData == nil then
		return 3;
	end

	-- 副本未开启
	if trialData.bOpened ~= true or DataCenter.divineDomainTrialUtil:checkOpened(trialData.clockTime) ~= true then
		return 3;
	end

	-- 特殊副本最后一关可重复挑战
	local bSpecialTrial = DataCenter.divineDomainTrialUtil:checkSpecialTrial(trialId);
	if bSpecialTrial and trialData.maxFightingLevel == levelId and trialData.passState == 2 then
		return 2;
	end

	-- 通关
	if trialData.maxFightingLevel > levelId or (trialData.maxFightingLevel == levelId and trialData.passState == 2) then
		return 1;
	end

	-- 可挑战
	if trialData.maxFightingLevel == levelId and trialData.passState == 1 then
		return 2;
	end 

	return 3;
end

-- buff界面是否处于选择模式
function Lua_DivineDomainTrialData.Set:bBuffSelectMode(value)
	self._bBuffSelectMode = value;
end

function Lua_DivineDomainTrialData.Get:bBuffSelectMode()
	if self._bBuffSelectMode == nil then
		self._bBuffSelectMode = false;
	end
	return self._bBuffSelectMode;
end

-- buff界面中选择的buffId
function Lua_DivineDomainTrialData.Set:curSelectingBuffId(buffId)
	self._curSelectingBuffId = buffId;
end

function Lua_DivineDomainTrialData.Get:curSelectingBuffId(buffId)
	if self._curSelectingBuffId == nil then
		self._curSelectingBuffId = 0
	end
	return self._curSelectingBuffId;
end

function Lua_DivineDomainTrialData:setBuffSelectMode(bSelectMode, trialId)
	if bSelectMode then
		self.bBuffSelectMode = true;
		self.curSelectingBuffId = nil;
	else
		local buffId = self:getBuffDicByTrialId(trialId);
		self.bBuffSelectMode = false;
		self.curSelectingBuffId = buffId;
	end
end

function Lua_DivineDomainTrialData:getSortedRewardIdsByType(type)
	local list = self:getRewardIdListByType(type);
	if list == nil then
		return;
	end
	list:sortf(self, self.sortAchievementReward, type);
	return list;
end

function Lua_DivineDomainTrialData:sortAchievementReward(a, b, type)
	local aState = self:getRewardStateById(type, a) or 0;
	local bState = self:getRewardStateById(type, b) or 0;

	local aSort = DataCenter.divineDomainTrialUtil:getDivineDomainTrialTaskCfg(a); 
	local bSort = DataCenter.divineDomainTrialUtil:getDivineDomainTrialTaskCfg(b); 
	aSort = aSort ~= nil and aSort.sort or 0;
	bSort = bSort ~= nil and bSort.sort or 0;
	if aState == bState then
		return aSort < bSort;
	else
		if bState == 1 then
			return false;
		elseif aState ~= 1 and bState == 0 then
			return false;
		end
	end
	return true;
end

function Lua_DivineDomainTrialData:checkTrialOpened(trialId, bShowTip)
	local trialData = self:getDivineDomainTrialData(trialId);
	if trialData == nil then
		return;
	end

	local clockTime = trialData.clockTime;
	local bOpened = DataCenter.divineDomainTrialUtil:checkOpened(clockTime);
	if not bOpened then
		if self.resetFlag and bShowTip == true then
			OperateAlert.showLangValue(764041);
		end
	end
	return bOpened;
end

-------红点
function Lua_DivineDomainTrialData:updateRedPoint()
	local spRewardState = self:getRewardRedPointState(Lua_DivineDomainTrialUtil.FIGHTING_REWARD); -- 特殊副本奖励
	local acRewardState = self:getRewardRedPointState(Lua_DivineDomainTrialUtil.ACHIEVEMENT_REWARD); -- 成就奖励
	local rewardState = spRewardState or acRewardState;

	local bShow = rewardState;
	if not bShow then
		bShow = self:updateRedPointByTrialId(Lua_DivineDomainTrialUtil.SPECIALTRIAL);
	end
	DataCenter.redPointData:updateRedPointState(Lua_RedPointList.DivineDomainTrial_Special, bShow);

	bShow = rewardState;
	if not bShow then
		bShow = self:updateRedPointByTrialId(Lua_DivineDomainTrialUtil.ORGINTRIAL); -- 普通副本
	end
	DataCenter.redPointData:updateRedPointState(Lua_RedPointList.DivineDomainTrial_Trial, bShow);
end

function Lua_DivineDomainTrialData:updateRedPointByTrialId(trialId)
	local residueCount = self:getCountDicByTrialId(trialId) or 0;
	local bOpened = self:getOpenedDicByTrialId(trialId) == true;
	local bShow = residueCount > 0 and not bOpened;
	return bShow;
end

function Lua_DivineDomainTrialData:getRewardRedPointState(type)
	if self.rewardIds == nil or self.rewardIds[type] == nil then
		return false;
	end

	local id;
	local bShow;
	local arr = self.rewardIds[type];
	for j = 1, arr:getSize() do
		id = arr[j];
		bShow = self:getRewardStateById(type, id) == 1;
		if bShow then
			break;
		end
	end

	local redPointId = type == Lua_DivineDomainTrialUtil.FIGHTING_REWARD and Lua_RedPointList.DivineDomainTrial_Special_Reward or Lua_RedPointList.DivineDomainTrial_Achieve_Reward;
	DataCenter.redPointData:updateRedPointState(redPointId, bShow);
	return bShow;
end

-- 跨天重置
function Lua_DivineDomainTrialData:resetSendMSg()
	if self.resetFlag == 1 then
		self.resetFlag = 0;
		self:setOpenedDicByTrialId(Lua_DivineDomainTrialUtil.ORGINTRIAL, false); -- 
		self:setOpenedDicByTrialId(Lua_RedPointList.DivineDomainTrial_Trial, false); -- 

		Lua_ProxyManager.getProxy("Lua_DivineDomainTrialProxy"):sGetDivineDomainInfo(0);
        Lua_ProxyManager.getProxy("Lua_DivineDomainTrialProxy"):sGetAcheiveRewardInfo(Lua_DivineDomainTrialUtil.ACHIEVEMENT_REWARD);
        Lua_ProxyManager.getProxy("Lua_DivineDomainTrialProxy"):sGetAcheiveRewardInfo(Lua_DivineDomainTrialUtil.FIGHTING_REWARD);
        Lua_ProxyManager.getProxy("Lua_DivineDomainTrialProxy"):sNoticeBossReset(); 
	end
end

function Lua_DivineDomainTrialData:checkUnlockShopItem(type, levelNum, diff) -- type: 1:普通系列副本最大关卡是否满足， 2 特殊副本
	local unLock = false;
	if type == 1 then -- 普通副本
		if self.commonTrialMaxLeveNum ~= nil then
			self.commonTrialMaxLeveNum = self.commonTrialMaxLeveNum ~= nil and self.commonTrialMaxLeveNum or 0;
			unLock = self.commonTrialMaxLeveNum >= levelNum;
		end
	elseif type == 2 then -- 特殊副本
		local data = self:getSpecialTrialData();
		diff = diff or 1;
		if data ~= nil then
			if data.diffLevel > diff then
				unLock = true;
			elseif data.diffLevel == diff then
				local trialData = self:getDivineDomainTrialData(Lua_DivineDomainTrialUtil.SPECIALTRIAL);
				if trialData ~= nil then
					local maxLevelId = trialData.maxFightingLevel;
					local levelCfg = DataCenter.divineDomainTrialUtil:getDivineDomainTrialLevelConfig(maxLevelId);
					if levelCfg ~= nil then
						local floor = levelCfg.floor or 0;
						if floor > levelNum or (floor == levelNum and trialData.passState == 2) then
							unLock = true;
						end
					end
				end
			end
		end
	end

	return unLock;
end

-- 是否扫荡刷新关卡数据
function Lua_DivineDomainTrialData:checkWipeOut()
	return self.bWipeOut;
end