Lua_DivineDomainTrialUtil = class("Lua_DivineDomainTrialUtil");
Lua_DivineDomainTrialUtil.ACHIEVEMENT_REWARD = 1; -- 成就奖励
Lua_DivineDomainTrialUtil.FIGHTING_REWARD = 2; -- 挑战奖励

Lua_DivineDomainTrialUtil.ORGINTRIAL = 1; -- 普通副本
Lua_DivineDomainTrialUtil.SPECIALTRIAL = 5; -- 特殊副本
function Lua_DivineDomainTrialUtil:ctor()
end

function Lua_DivineDomainTrialUtil:dispose()
	self._commonCostItem = nil;
	self._specialCostItem = nil;
	self._specialLevelIds = nil;
	self._showSkills = nil;
end

function Lua_DivineDomainTrialUtil:checkSpecialTrial(trialId)
	return trialId == Lua_DivineDomainTrialUtil.SPECIALTRIAL;
end

function Lua_DivineDomainTrialUtil:getTrialIndexByTrialId(trialId)
	local bSpecialTrial = self:checkSpecialTrial(trialId);
	local key = bSpecialTrial and Lua_DivineDomainTrialUtil.SPECIALTRIAL or 1;
	return key;
end

-- 根据trialId获取buff选择的类型
function Lua_DivineDomainTrialUtil:getBuffTypeByTrialId(trialId)
	local bSpecialTrial = self:checkSpecialTrial(trialId);
	local key = bSpecialTrial and 2 or 1; -- 1 普通 2特殊
	return key;
end

function Lua_DivineDomainTrialUtil:checkSpecialTrialLevel(levelId)
	return Mathf.Floor(levelId / 10000) == Lua_DivineDomainTrialUtil.SPECIALTRIAL;
	-- return trialId == Lua_DivineDomainTrialUtil.SPECIALTRIAL;
end

function Lua_DivineDomainTrialUtil:getEntranceItemIds()
	local trial_ids = getConfig("domain_trial_const", "trial_ids", "arr_value");
	return trial_ids;
end

function Lua_DivineDomainTrialUtil:getDivineDomainTrialConfig(trialId)
	return getConfig("domain_trial_list", trialId);
end

function Lua_DivineDomainTrialUtil:getDivineDomainTrialLevelConfig(levelId)
	return getConfig("domain_trial_stage", levelId);
end

function Lua_DivineDomainTrialUtil:getDivineDomainTrialRecommendConfig(recommendId)
	return getConfig("domain_trial_recommend", recommendId);
end

function Lua_DivineDomainTrialUtil:getDivineDomainTrialResource()
	return getConfig("domain_trial_const", "resource_ids", "arr_value");
end

function Lua_DivineDomainTrialUtil.Get:divineDomainTrialTip()
	return getConfig("domain_trial_const", "tips", "desc");
end

function Lua_DivineDomainTrialUtil:getDivineDomainTrialTaskCfg(id)
	return getConfig("domain_trial_task", id);
end

function Lua_DivineDomainTrialUtil:getTotalFreeCountByTrialId(trialId)
	local index = self:getTrialIndexByTrialId(trialId);
	if index == Lua_DivineDomainTrialUtil.SPECIALTRIAL then
		return 6;
	else
		return 6;
	end
end

function Lua_DivineDomainTrialUtil:getTrialIdByLevelId(levelId)
	if levelId == nil then
		return;
	end

	return Mathf.Floor(levelId / 10000);
end

function Lua_DivineDomainTrialUtil:checkOpened(time)
	time = tonumber(time);
	local bOpened;
	if time <= 0 then
		bOpened = false
	else
		bOpened = DataCenter.roleData.serverTime - time < 0;
	end
	return bOpened;
end

function Lua_DivineDomainTrialUtil:checkLastLevel(levelId)
	local cfg = self:getDivineDomainTrialLevelConfig(levelId);
	if cfg == nil then
		return;
	end

	return cfg.last == 1;
end

function Lua_DivineDomainTrialUtil:getLevelIndexByTrialId(trialId)
	local cfg = self:getDivineDomainTrialConfig(trialId);
	if cfg == nil then
		return;
	end

	local difficult = 1;
	local specialTrialData = DataCenter.divineDomainTrialData:getSpecialTrialData();
	if specialTrialData ~= nil then
		difficult = specialTrialData.diffLevel;
	end
	local list = self:getDivineDomainTrialLevelIdsByTrialId(trialId, difficult);
	local count = list ~= nil and #list or 1;

	local trialData = DataCenter.divineDomainTrialData:getDivineDomainTrialData(trialId);
	local levelId = trialData.maxFightingLevel; 
	cfg = self:getDivineDomainTrialLevelConfig(levelId);
	local floor = cfg and cfg.floor or 1;

	local index = count - floor;
	index = index == 0 and 1 or index;
	return index;
end

function Lua_DivineDomainTrialUtil:getAdmireCount()
	return getConfig("domain_trial_const", "like_times", "value");
end

function Lua_DivineDomainTrialUtil:enableSatisfyLevel(levelId)
	local cfg = self:getDivineDomainTrialLevelConfig(levelId);
	if cfg == nil then
		return;
	end
	local grade = cfg.level or 1;
	return grade <= DataCenter.roleData.level, grade;
end

-- 特殊关卡是否可扫荡
function Lua_DivineDomainTrialUtil:checkSpecialWipeOut(levelId)
	local cfg = self:getDivineDomainTrialLevelConfig(levelId);
	if cfg == nil then
		return;
	end

	return cfg.wipe_out == 1;
end

-- 普通副本消耗道具打关
function Lua_DivineDomainTrialUtil.Get:commonCostItem()
	if self._commonCostItem == nil then
		local costId = getConfig("domain_trial_const", "common_item", "value");
		self._commonCostItem = {costId, 1};
	end
	return self._commonCostItem;
end

-- 特殊副本消耗道具打关
function Lua_DivineDomainTrialUtil.Get:specialCostItem()
	if self._specialCostItem == nil then
		local costId = getConfig("domain_trial_const", "special_item", "value");
		self._specialCostItem = {costId, 1};
	end
	return self._specialCostItem;
end

function Lua_DivineDomainTrialUtil:getCostItem(bSpecialTrial, levelId)
	if bSpecialTrial then
		return self.specialCostItem;
	else
		return self.commonCostItem;
	end
end

function Lua_DivineDomainTrialUtil:getSceneIdByTrialId(trialId)
	local sceneId;
	local cfg = self:getDivineDomainTrialConfig(trialId);
	if cfg ~= nil then
		sceneId = cfg.scene_id;
	end
	return sceneId or 40112;
end

function Lua_DivineDomainTrialUtil:getLevelIdByTrialIdAndDiff(trialId, diffId, levelIndex)
	local levelId;
	local bSpecialTrial = self:checkSpecialTrial(trialId);
	if bSpecialTrial then
		levelId = trialId * 10000 + diffId * 1000 + levelIndex;
	else
		levelId = trialId * 10000 + levelIndex;
	end
	return levelId;
end

function Lua_DivineDomainTrialUtil:getDivineDomainTrialLevelIdsByTrialId(trialId, diffLevel)
	local cfg = self:getDivineDomainTrialConfig(trialId);
	if cfg == nil then
		return;
	end

	if self._levelIds == nil then
		self._levelIds = {};
	end

	if self._levelIds[trialId] == nil then
		local ids = {};
		local cfgLevels = cfg.level_list; -- [3,4,5]
		if cfgLevels ~= nil then
			local levelId;
			local maxLevelIndex;
			for i = 1, #cfgLevels do
				ids[i] = {};
				maxLevelIndex = cfgLevels[i] or 1;
				for j = maxLevelIndex, 1, -1 do
					levelId = self:getLevelIdByTrialIdAndDiff(trialId, i, j);
					if not self:checkLastLevel(levelId) then
						ids[i][#ids[i] + 1] = levelId;
					end
				end
			end
		end
		self._levelIds[trialId] = ids;
	end

	local bSpecialTrial = self:checkSpecialTrial(trialId);
	diffLevel = diffLevel or 1;
	if bSpecialTrial then
		return self._levelIds[trialId] ~= nil and self._levelIds[trialId][diffLevel] or nil;
	else
		diffLevel = 1;
		return self._levelIds[trialId] ~= nil and self._levelIds[trialId][diffLevel] or nil;
	end
end

function Lua_DivineDomainTrialUtil:getTrailPassRewardCfg(rewardId)
	return getConfig("domain_trial_pass_reward", rewardId);
end

-- 预览奖励
function Lua_DivineDomainTrialUtil:getPreviewRewardsByTrial(trialId)
	if self._previewRewards == nil then
		self._previewRewards = {};
	end

	local cfg = self:getDivineDomainTrialConfig(trialId);
	if cfg == nil then
		return;
	end

	if self._previewRewards[trialId] == nil then
		local previewIds = cfg.reward_ids;
		if previewIds == nil then
			return;
		end

		local rewards = {};
		local rewardCfg;
		local obj;
		for i = 1, #previewIds do
			rewardCfg = self:getTrailPassRewardCfg(previewIds[i]);
			if rewardCfg ~= nil and rewardCfg.reward ~= nil then
				for j = 1, #rewardCfg.reward do
					obj = tonumber(rewardCfg.reward[j][1][1]);
					if obj ~= nil then
						rewards[#rewards + 1] = obj;
					end
				end

			end
		end

		table.sort( rewards, function(a, b) return a > b; end );
		self._previewRewards[trialId] = rewards;
	end

	return self._previewRewards[trialId];
end

function Lua_DivineDomainTrialUtil:getShowSkillByTrialId(trialId)
	if self._showSkills == nil then
		self._showSkills = {};
	end

	local skills = self._showSkills[trialId];
	if skills == nil then
		skills = {};
		local cfg = self:getDivineDomainTrialConfig(trialId);
		if cfg ~= nil then
			local obj = cfg.show_skill;
			if obj ~= nil then
				for i = 1, #obj do
					skills[#skills + 1] = obj[i];
				end
			end

			obj = cfg.boss_buff;
			if obj ~= nil then
				for i = 1, #obj do
					skills[#skills + 1] = obj[i];
				end
			end
		end
		self._showSkills[trialId] = skills;
	end
	return skills;
end

function Lua_DivineDomainTrialUtil:getDifficultListByDifficult(difficult)
	self:initHurtReward();
	local list = self._hurtValueDir[difficult];
	return list or {};
end

function Lua_DivineDomainTrialUtil:initHurtReward()
	if self._hurtValueDir == nil then
		self._hurtValueDir = {};
		local config = getConfig("domain_trial_hurt_value");
		local type;
		local list;
		for k, v in pairs(config) do
			type = v.difficult;
			list = self._hurtValueDir[type];
			if list == nil then
				list = {};
				self._hurtValueDir[type] = list;
			end

			list[#list + 1] = v.hurt;
		end

		for k, v in pairs(self._hurtValueDir) do
			table.sort( v, function(a, b)
				return a < b;
				end
			 )
		end
	end
end

-- 特殊副本最后一关
function Lua_DivineDomainTrialUtil:getFightShowInfo(bossHurt, levelId)
	local difficult = 1;
	local levelCfg = self:getDivineDomainTrialLevelConfig(levelId);
	if levelCfg ~= nil then
		difficult = levelCfg.difficulty or difficult;
	end

	local list = self:getDifficultListByDifficult(difficult);
    local len = #list;
    local idx;
    local curValue;
    local prevValue;
    local isFinally = false;
    for i = 1, len do
        if bossHurt < list[i] then
            idx = i;
            break;
        end
    end

    if idx == nil then
        idx = len;
        isFinally = true;
    end
    curValue = list[idx];
    if idx > 1 then
        prevValue = list[idx - 1];
    else
        prevValue = 0;
    end
        
    return idx, curValue, prevValue, isFinally;
end

-- 是否是神域试炼特殊副本且是最后一关：伤害挑战
function Lua_DivineDomainTrialUtil:checkHurtFight(levelId)
	local levelCfg = self:getDivineDomainTrialLevelConfig(levelId);
	if levelCfg ~= nil and levelCfg.index then
		local bSpecialTrial = self:checkSpecialTrial(levelCfg.index);
		if bSpecialTrial then
			return self:checkLastLevel(levelId);
		end
	end

	return false;
end