-- 特效
Lua_Effect = class("Lua_Effect");
function Lua_Effect:ctor()
	self:superCall("ctor");

	self.effectMoveToTargetTime = 0.4; -- 特效移动固定时间
end

function Lua_Effect:dispose()
    if self._bDispose then
        return;
    end

    self._bDispose = true;
    self:updateRoleHide(true);
    if self._data ~= nil then
		DataCenter.effectManager:returnEffectVo(self._data);
		self._data = nil;
	end

    if self.effectItem ~= nil then
    	self.effectItem:dispose();
    	self.effectItem = nil;
    end
end

function Lua_Effect:clean()
	if self.effectItem ~= nil then
		self.effectItem:clean();
	end

	if self._data ~= nil then
		DataCenter.effectManager:returnEffectVo(self._data);
		self._data = nil;
	end

	self:updateRoleHide(true);
	self.bEndPlay = false;
	self.endTime = 0;
	self.moveType = 0;
    self.moveTime = 0;
    self.delayMoveTime = 0;
    self.moveStartTime = 0;
    self._bEndMove = true;
    self._moveTargetsIdx = 0;
    self._startPosX = 0;
    self._startPosY = 0;
    self._startPosZ = 0;
    self._endPosX = 0;
    self._endPosY = 0;
    self._endPosZ = 0;
    self._controllPosX = 0;
    self._controllPosY = 0;
    self._controllPosZ = 0;
end

function Lua_Effect:checkDisposeEffect(sid, fid, rid)
    if self._data ~= nil and self._data.sceneId == sid and self._data.fightId == fid then
        if self._data.configData.moveType == 0 then
            local hang = self._data.configData.hang;
            if hang == 1 or hang == 2 then
                return self._data.selfRoleId == rid;
            end
        end
    end
    return false;
end

function Lua_Effect:canDispose()
	return self.bEndPlay or self._data == nil or self._data.configData == nil;
end

function Lua_Effect.Get:sceneId()
	return self._data ~= nil and self._data.sceneId or 0;
end

function Lua_Effect.Get:fightId()
	return self._data ~= nil and self._data.fightId or 0;
end

function Lua_Effect.Get:speed()
	return self._speed;
end

function Lua_Effect.Get:speedRate()
	return self._spdRate;
end

function Lua_Effect.Get:data()
	return self._data;
end

function Lua_Effect:setData(vo)
	self._data = vo;
	if not self:canPlayEffect() then
        if self.effectItem ~= nil then
            -- print("Lua_Effect:setData clean", self.effectItem.filePath, self.effectItem.filePath.modelName, getTraceback())
        	self.effectItem:clean();
        end
        self.bEndPlay = true;
        return;
    end

    self.bEndPlay = false;
    if self.effectItem == nil then
    	self.effectItem = EffectItem();
    end
    self.effectItem:setLuaCallBack(self, self.onLoadCompleted);

    -- 设置父节点
    local cfg = self._data.configData;
    if cfg.moveType > 0 then -- 飞行特效挂世界
    	if not self.effectItem:setSceneParent(self._data.sceneId, self._data.fightId) then
    		self.effectItem:setThingParent();
    	end
    else
        if cfg.hang == 1 then -- 挂avatar
        	if not self.effectItem:setThingHangPath(self._data.sceneId, self._data.fightId, self._data.selfRoleId, cfg.hangPath) then
        		if not self.effectItem:setThingHangPath(self._data.sceneId, self._data.fightId, self._data.selfRoleId) then
        			local p = self._data:getParent();
        			if p ~= nil then
        				self.effectItem:setParent(p);
        			else
	        			if not self.effectItem:setSceneParent(self._data.sceneId, self._data.fightId) then
	        				self.effectItem:setThingParent();
	        			end
	        		end
        		end
        	end
        elseif cfg.hang == 2 then
            if not self.effectItem:setThingHangPath(self._data.sceneId, self._data.fightId, self._data.selfRoleId) then
				local p = self._data:getParent();
				if p ~= nil then
					self.effectItem:setParent(p);
				else
	    			if not self.effectItem:setSceneParent(self._data.sceneId, self._data.fightId) then
	    				self.effectItem:setThingParent();
	    			end
	    		end
			end
        else
        	local p = self._data:getParent();
			if p ~= nil then
				self.effectItem:setParent(p);
			else
    			if not self.effectItem:setSceneParent(self._data.sceneId, self._data.fightId) then
    				self.effectItem:setThingParent();
    			end
    		end
        end
    end

    cfg.effAvatarParent = self.effectItem.parent; -- 用于计算位置
    -- print("effAvatarParent", cfg.effAvatarParent.name);
    self.effectItem.hasCamera = cfg.hasCamera;
    local modelName;
    if cfg.mirrorType == 1 then
        modelName = cfg.path .. (self._data.formationType == Lua_FightDefine.FIGHT_DEFENDER and "_b" or "_a");
    else
        modelName = cfg.path;
    end
    -- print("self.effectItem:setUrl", cfg.effectPath, modelName);
    self.effectItem:setUrl(cfg.effectPath, modelName);

    -- 移动
    self.moveType = cfg.moveType;
    -- print("移动类型", modelName, self.moveType);
    if self.moveType > 0 then -- 需要位移
        self._moveTargetsIdx = 1;
        self.moveTime = cfg.moveTime;
        self.delayMoveTime = cfg.delayMoveTime;
        self._startPosX, self._startPosY, self._startPosZ = self._data:getStartPos(self._moveTargetsIdx);
        self._endPosX, self._endPosY, self._endPosZ = self._data:getEndPos(self._moveTargetsIdx);
        -- print("移动", modelName, self.moveTime, self.delayMoveTime, self._startPosX, self._startPosY, self._startPosZ, self._endPosX, self._endPosY, self._endPosZ)
        if self.moveType == 2 or self.moveType == 3 then -- 抛物线
        	self._controllPosX, self._controllPosY, self._controllPosZ = self._data:getControllPos(self._startPosX, self._startPosY, self._startPosZ, self._endPosX, self._endPosY, self._endPosZ);
       	end
        if self.moveTime <= 0 then 
        	self.moveTime = self.effectMoveToTargetTime; 
        end
        self.moveStartTime = 0;
        self._bEndMove = false;
        self.effectItem:setLocation(self._startPosX, self._startPosY, self._startPosZ);
        -- print("Lua_Effect 需要位移", self.moveType, self._startPosX, self._startPosY, self._startPosZ, self._endPosX, self._endPosY, self._endPosZ, self.moveTime);
    end

    if cfg.isContactTwoEff or cfg.isContactAllEff or cfg.isThroughAllEff then
        if self.moveType == 0 then
            self._startPosX, self._startPosY, self._startPosZ = self._data:getStartPos(self._moveTargetsIdx);
            self._endPosX, self._endPosY, self._endPosZ = self._data:getEndPos(self._moveTargetsIdx);
            self.effectItem:setLocation(self._startPosX, self._startPosY, self._startPosZ);
        end
        local rx, ry, rz = self._data:getDirByToPos(self._startPosX, self._startPosY, self._startPosZ, self._endPosX, self._endPosY, self._endPosZ, 0, 0, 0);
        self.effectItem:setRot(rx, ry, rz);
        local sz = self._data:getScaVecByTwoPoint(self._startPosX, self._startPosY, self._startPosZ, self._endPosX, self._endPosY, self._endPosZ);
        self.effectItem:setScale(1, 1, sz);
    else
    	if self.moveType <= 0 then
    		self.effectItem:setLocation(self._data.orginPosX, self._data.orginPosY, self._data.orginPosZ);
    	end
        self.effectItem:setRot(self._data.effRotX, self._data.effRotY, self._data.effRotZ);
	    self.effectItem:setScale(1, 1, self._data.effSca);
    end
    
    self:updateRoleHide(false);
    self:setPlaySpeed(self._data.speed, self._data.speedRate);
    self.endTime = self._data.effTime > 0 and self._data.effTime or cfg.time;
end

function Lua_Effect:onLoadCompleted(val)
	if val < 0 then -- 加载失败
        print("onLoadCompleted 加载失败", val, self._data.configData.effectPath);
		self.bEndPlay = true;
	end
end

function Lua_Effect:canPlayEffect()
	if self._data == nil then
        print("Lua_Effect:canPlayEffect() self._data == nil")
		return false;
	end

    if not Scene3DManager.instance:isCurShowScene(self._data.sceneId, self._data.fightId) then
        print("Lua_Effect:canPlayEffect() isCurShowScene false", self._data.sceneId, self._data.fightId);
        return false;
    end

    if self._data.configData.moveType <= 0 then
        local hang = self._data.configData.hang;
        if hang == 2 or hang == 1 then
            if self._data.owner == nil then
                print("Lua_Effect:canPlayEffect() self._data.owner == nil", hang);
                return false;
            end
        end
    end

    return true;
end

function Lua_Effect:setPlaySpeed(speed, spdRate)
    self._spdRate = spdRate;
    self._speed = speed;
    if self.effectItem ~= nil then
    	self.effectItem:setPlaySpeed(speed, spdRate);
    end
end

function Lua_Effect:updateRoleHide(bShow)
    if self._data == nil then
        return;
    end

    local roleHide = self._data.configData.roleHide;
    if roleHide == 1 or roleHide == 3 then
    	if self._data.owner ~= nil then
    		self._data.owner.player:setActive(bShow);
    	end
    end

    if roleHide == 2 or roleHide == 3 then
    	if self._data.target ~= nil then
    		self._data.target.player:setActive(bShow);
    	end
    end
end

function Lua_Effect:frameScript(dt)
	dt = dt * self._spdRate;
	if self.endTime > 0 then
		self.endTime = self.endTime - dt;
		if self.endTime <= 0 then
			self.bEndPlay = true;
		end
	end

	if not self._bEndMove then
        if self.moveType == 1 then -- 直线
            self:straightLineMove(dt);
        elseif self.moveType == 2 or self.moveType == 3 then -- 抛物线 
            self:paraCurveMove(dt);
        end
    end
end

function Lua_Effect:straightLineMove(dt)
    if self.delayMoveTime > 0 then
        self.delayMoveTime = self.delayMoveTime - dt;
        if self.delayMoveTime <= 0 then
            dt = -self.delayMoveTime;
            self.delayMoveTime = 0;
        else
            dt = 0;
        end
    end

    self.moveStartTime = self.moveStartTime + dt;
    local r = self.moveStartTime / self.moveTime;
    if r >= 1 then
        self.moveStartTime = 0;
        self._moveTargetsIdx = self._moveTargetsIdx + 1;
        if self.effectItem ~= nil then
        	self.effectItem:setLocation(self._endPosX, self._endPosY, self._endPosZ);
        end
        if self._data:checkIsEnd(self._moveTargetsIdx) then
            self.moveTime = 0;
            self:arriveTarget();
        else
        	self._startPosX, self._startPosY, self._startPosZ = self._data:getStartPos(self._moveTargetsIdx);
            self._endPosX, self._endPosY, self._endPosZ = self._data:getEndPos(self._moveTargetsIdx);
            if self.effectItem ~= nil then
                local rx, ry, rz = self._data:getDirByToPos(self._startPosX, self._startPosY, self._startPosZ, self._endPosX, self._endPosY, self._endPosZ, self.effectItem.rotx, self.effectItem.roty, self.effectItem.rotz);
                self.effectItem:setRot(rx, ry, rz);
            end
        end
    elseif self.effectItem ~= nil then
    	local px = self._startPosX + (self._endPosX - self._startPosX) * r;
    	local py = self._startPosY + (self._endPosY - self._startPosY) * r;
    	local pz = self._startPosZ + (self._endPosZ - self._startPosZ) * r;
    	self.effectItem:setLocation(px, py, pz);
    end
end

function Lua_Effect:paraCurveMove(dt)
    if self.delayMoveTime > 0 then
        self.delayMoveTime = self.delayMoveTime - dt;
        if self.delayMoveTime <= 0 then
            dt = -self.delayMoveTime;
            self.delayMoveTime = 0;
        else
            dt = 0;
        end
    end

    self.moveStartTime = self.moveStartTime + dt;
    local r = self.moveStartTime / self.moveTime;
    if r >= 1 then
        self.moveStartTime = 0;
        self._moveTargetsIdx = self._moveTargetsIdx + 1;
        if self.effectItem ~= nil then
        	self.effectItem:setLocation(self._endPosX, self._endPosY, self._endPosZ);
        end
        if self._data:checkIsEnd(self._moveTargetsIdx) then
            self.moveTime = 0;
            self:arriveTarget();
        else
            self._startPosX, self._startPosY, self._startPosZ = self._data:getStartPos(self._moveTargetsIdx);
            self._endPosX, self._endPosY, self._endPosZ = self._data:getEndPos(self._moveTargetsIdx);
            self._controllPosX, self._controllPosY, self._controllPosZ = self._data:getControllPos(self._startPosX, self._startPosY, self._startPosZ, self._endPosX, self._endPosY, self._endPosZ);
        end
    elseif self.effectItem ~= nil then
        local px, py, pz = Lua_BezierUtils.CalculateCubicBezierPointVal(r, 
        	self._startPosX, self._startPosY, self._startPosZ, 
        	self._controllPosX, self._controllPosY, self._controllPosZ, 
        	self._endPosX, self._endPosY, self._endPosZ);
        self.effectItem:setLookAt(px, py, pz);
        self.effectItem:setLocation(px, py, pz);
    end
end

function Lua_Effect:arriveTarget()
    self._bEndMove = true;
    if self._data.configData.isThroughAllEff then
        self.endTime = self._data.configData.seriesMoveTime; 
    end
end