-- 特效管理
checkRequire("Lua_Effect");
checkRequire("Lua_EffectVo");
Lua_EffectManager = class("Lua_EffectManager");
function Lua_EffectManager:ctor()
	self:superCall("ctor");
    
    self._curTime = getTime();
    self.showingList = Array();
    self.collectDataList = Array();
    self._idleVosList = Array();
    self._idleEffsList = Array();
end

function Lua_EffectManager:dispose()
	
end

function Lua_EffectManager:createEffectVo()
    local vo = nil;
    if self._idleVosList:getSize() > 0 then
    	vo = self._idleVosList:remove();
    end
    if vo == nil then
    	vo = Lua_EffectVo.new();
    end
    return vo;
end

function Lua_EffectManager:returnEffectVo(vo)
	if vo == nil then
		return;
	end
	vo:clean();
	self._idleVosList:add(vo);
end

function Lua_EffectManager:createEffect()
    local eff = nil;
    if self._idleEffsList:getSize() > 0 then
    	eff = self._idleEffsList:remove();
    end
    if eff == nil then
    	eff = Lua_Effect.new();
    end
    return eff;
end

function Lua_EffectManager:returnEffect(eff)
	if eff == nil then
		return;
	end
	eff:clean();
	self._idleEffsList:add(eff);
end

function Lua_EffectManager:playCollectedEffect()
    local t = getTime();
    local deltaTime = t - self._curTime;
    self._curTime = t;

    local eff;
    local len = self.showingList:getSize();
    while (len > 0) do
        eff = self.showingList[len];
        if eff:canDispose() then
        	self:returnEffect(eff);
            self.showingList:remove(len);
        else
            eff:frameScript(deltaTime);
        end
        len = len - 1;
    end

    local vo;
    len = self.collectDataList:getSize();
    while (len > 0) do
        vo = self.collectDataList[len];
        vo:onEnterFrame(deltaTime);
        if vo.bShow then
            self.collectDataList:remove(len);
            eff = self:createEffect();
            eff:setData(vo);
            self.showingList:add(eff);
        end
        len = len - 1;
    end
end

function Lua_EffectManager:getEffContactTime(effCfg, contactCount)
    if effCfg ~= nil and (effCfg.isContactAllEff or effCfg.isThroughAllEff) then
        return (contactCount - 1) * (effCfg.moveTime + effCfg.seriesMoveTime);
    end
    return 0;
end

function Lua_EffectManager:playActionEffect(sceneId, fightId, effId, px, py, pz, parent, selfRId, targetRId, spd, speed, speedRate, dir, startPos, endPos, targetsList, selfFormationType, selfCampPosId) 
    local config = Lua_EffectConfigData.getConfigData(effId);
    if config == nil then
        return;
    end

    local num = config.num <= 0 and 1 or config.num;
    local interval = config.interval;
    local delayTime = config.delayTime;
    sceneId = (sceneId == nil or sceneId <= 0) and Scene3DManager.instance.curShowSceneId or sceneId;
    if (config.isContactTwoEff or config.isContactAllEff) and targetsList ~= nil then
        local count = targetsList:getSize();
        for i = 1, count do
            selfRId = (i == 1) and selfRId or targetsList[i - 1];
            targetRId = targetsList[i];
            for j = 1, num do
                local vo = self:createEffectVo();
                vo.parent = parent;
                vo.sceneId = sceneId;
                vo.fightId = fightId;
                vo.effectId = effId;
                vo.selfRoleId = selfRId;
                vo.targetRoleId = (targetRId ~= nil and targetRId > 0) and targetRId or selfRId;
                vo.spd = spd;
                vo.speed = speed;
                vo.speedRate = speedRate;
                vo.startFormationPos = startPos;
                vo.endFormationPos = endPos;
                vo.formationType = selfFormationType;
                vo.selfCampPosId = selfCampPosId;
                local owner = vo.self;
                if owner ~= nil then
                    vo.posx = owner.data.px;
                    vo.posy = owner.data.py;
                    vo.posz = owner.data.pz;
                    local target = vo.target;
                    if target ~= nil then
                        vo.dir = MathUtil.getDirectionByXY(vo.posx, vo.posz, target.data.px, target.data.pz);
                    end
                end
                vo.time = j * interval + delayTime + i * (config.seriesMoveTime + config.moveTime);
                if config.isContactAllEff then
                	vo.effTime = (count - i) * (config.seriesMoveTime + config.moveTime);
                end
                self.collectDataList:add(vo);
            end
        end
    else
        for i = 1, num do
            local vo = self:createEffectVo();
            vo.parent = parent;
            vo.sceneId = sceneId;
            vo.fightId = fightId;
            vo.effectId = effId;
            vo.posx = px;
            vo.posy = py;
            vo.posz = pz;
            vo.dir = dir;
            vo.selfRoleId = selfRId;
            vo.targetRoleId = (targetRId ~= nil and targetRId > 0) and targetRId or selfRId;
            vo.spd = spd;
            vo.speed = speed;
            vo.speedRate = speedRate;
            vo.startFormationPos = startPos;
            vo.endFormationPos = endPos;
            vo.formationType = selfFormationType;
            vo.selfCampPosId = selfCampPosId;
            vo.time = i * interval + delayTime;
            vo.targetsList = targetsList;
            self.collectDataList:add(vo);
        end
    end
end

function Lua_EffectManager:checkDisposeEffect(sceneId, fightId)
    if sceneId == nil then sceneId = Scene3DManager.instance.curShowSceneId; end
    if fightId == nil then fightId = Scene3DManager.instance.curFightRecordId; end
    local len = self.showingList:getSize();
    while (len > 0) do
        local eff = self.showingList[len];
        if eff.sceneId == sceneId and eff.fightId == fightId then
            self.showingList:remove(len);
            self:returnEffect(eff);
        end
        len = len - 1;
    end

    len = self.collectDataList:getSize();
    while (len > 0) do
        local vo = self.collectDataList[len];
        if vo.sceneId == sceneId and vo.fightId == fightId then
            self.collectDataList:remove(len);
            self:returnEffectVo(vo);
        end
        len = len - 1;
    end
end

function Lua_EffectManager:checkDisposeEffectByRId(sceneId, fightId, roleId)
	local len = self.showingList:getSize();
    while (len > 0) do
        local eff = self.showingList[len];
        if eff:checkDisposeEffect(sceneId, fightId, roleId) then
            self.showingList:remove(len);
            self:returnEffect(eff);
        end
        len = len - 1;
    end

    len = self.collectDataList:getSize();
    while (len > 0) do
        local vo = self.collectDataList[len];
        if vo.sceneId == sceneId and vo.fightId == fightId and vo.selfRoleId == roleId then
            self.collectDataList:remove(len);
            self:returnEffectVo(vo);
        end
        len = len - 1;
    end
end

function Lua_EffectManager:updateEffPlaySpeed(sceneId, fightId, roleId, spd, rate)
    for i = 1, self.showingList:getSize() do
        local eff = self.showingList[i];
        if eff.sceneId == sceneId and eff.fightId == fightId and eff.data.selfRoleId == roleId then
        	eff.data.speed = spd;
            eff.data.speedRate = rate;
            eff:setPlaySpeed(spd, rate);
        end
    end

    for i = 1, self.collectDataList:getSize() do
        local vo = self.collectDataList[i];
        if vo.sceneId == sceneId and vo.fightId == fightId and vo.selfRoleId == roleId then
            vo.speed = spd;
            vo.speedRate = rate;
        end
    end
end