-- 超值基金（本地）
checkRequire("Lua_SpecialFundsVo");
Lua_SpecialFundsData = class("Lua_SpecialFundsData");
function Lua_SpecialFundsData:ctor()
	self:superCall("ctor");
end

function Lua_SpecialFundsData:dispose()
	if self.rewardDic ~= nil then
		for k,v in pairs(self.rewardDic) do
			v:dispose();
			v = nil;
		end
		self.rewardDic = nil;
	end
	self._curDrawDay = nil;
	self._worldLv = nil;
	self._endTick = nil;
	self._hasDrawDay = nil;
	self.showFundsRewardDic = nil;
	Lua_CommonUtil.removeTimer(self, self.delayCallFun);
	self:superCall("dispose");
end

function Lua_SpecialFundsData:fromServer50197058(msg)
	if msg == nil then
		return;
	end
	if self.rewardDic == nil then
		self.rewardDic = {};
	end

	self._curDrawDay = msg.curr_index; -- 当前可领取的最大天数
	self._worldLv = msg.world_lvl; -- 世界等级
	self._endTick = tonumber(msg.timestamp); -- 结束时间戳

	local obj, vo;
	local redState = false;
	for i = 1, #msg.new_reward_list do
		obj = msg.new_reward_list[i];
		vo = self:getFundsVoInfo(obj.type);
		if vo == nil then
			vo = Lua_SpecialFundsVo.new();
			self.rewardDic[obj.type] = vo;
		end
		vo:updateFundsVoInfo(obj);

		if vo.lockState and vo.fundsRedState and not redState then
			redState = true;
		end
	end
	redState = redState and not self.isCloseFunds; -- 有可领取奖励/解锁/未关闭活动
	DataCenter.redPointData:updateRedPointState(Lua_RedPointList.ExcellentAct_Funds_Special, redState);

	local actVo = DataCenter.internalActData:getActivityVo(Lua_ActivityList.ID_SpecialFunds);
	if self.isCloseFunds then
		actVo:fromServerInfo(0, 0);
	else
		actVo:fromServerInfo(nil, nil);
	end
	if self.remainTime > 0 then
		Lua_CommonUtil.delayCall(self.remainTime, self, self.delayCallFun);
	end
end

function Lua_SpecialFundsData:delayCallFun()
	self.showFundsRewardDic = nil;
	Lua_CommonUtil.removeTimer(self, self.delayCallFun);
	Lua_ProxyManager.getProxy("Lua_ExcellentActivityProxy"):sSpecialFundsInfo();
end

function Lua_SpecialFundsData.Get:curDrawDay()
	return self._curDrawDay or 0;
end

function Lua_SpecialFundsData.Get:worldLv()
	return self._worldLv or 1;
end

function Lua_SpecialFundsData.Get:remainTime()
	local tick = self._endTick or 0;
	local time = tick - DataCenter.roleData.serverTime;
	return time <= 0 and 0 or time;
end

function Lua_SpecialFundsData:getFundsVoInfo(gradeType)
	if gradeType == nil or self.rewardDic == nil or self.rewardDic[gradeType] == nil then
		return nil;
	end
	return self.rewardDic[gradeType];
end

function Lua_SpecialFundsData:getShowFundsInfoArr(gradeType)
	local gradeInfo = self:getFundsVoInfo(gradeType);
	if gradeInfo == nil then
		return nil;
	end
	if self.showFundsRewardDic == nil then
		self.showFundsRewardDic = {};
	end
	if self.showFundsRewardDic[gradeType] == nil then
		local list = gradeInfo.rewardList;
		local len = #list;
		local key = self.rewardKey;
		local config, tvoInfo, tvo;
		local itemDic = {};
		local arr = Array();
		for i = 1, len do
			config = self:getFundsInfo(list[i].key);
			tvoInfo = itemDic[config[key][1]];
			if tvoInfo == nil then
				tvoInfo = {id = config[key][1], amount = config[key][2]};
				itemDic[config[key][1]] = tvoInfo;
			else
				tvoInfo.amount = tvoInfo.amount + config[key][2];
			end
		end
		for k,v in pairs(itemDic) do
			tvo = Lua_ToolVo.createToolVo(k);
			tvo:fromServer(v);
			arr:add(tvo);
		end
		arr:sort("sortWeight", false);
		self.showFundsRewardDic[gradeType] = arr;
	end
	return self.showFundsRewardDic[gradeType];
end

function Lua_SpecialFundsData.Get:rewardKey()
	local list = self.worldLvList;
	local len = #list;
	local worldLv = DataCenter.specialFundsData.worldLv;
	local id = list[len][3];
	for i = 1, len do
		if list[1][1] <= worldLv and list[1][2] >= worldLv then
			id = list[1][3];
			break;
		end
	end
	return id;
end





---------------------------------------- 配置 ----------------------------------------
function Lua_SpecialFundsData:getFundsInfo(id)
	return getConfig("new_overflow_fund", id);
end

-- 世界等级奖励配置
function Lua_SpecialFundsData.Get:worldLvList()
	return DataCenter.operationalActUtil:getActConstConfig("overflow_fund_world_reward", 4);
end

-- 充值id
function Lua_SpecialFundsData.Get:rechargeIdList()
	return DataCenter.operationalActUtil:getActConstConfig("overflow_fund_recharge_id", 4);
end

-- 充值立得
function Lua_SpecialFundsData.Get:instantReward()
	return DataCenter.operationalActUtil:getActConstConfig("overflow_fund_recharge_reward", 4);
end

-- 总价值
function Lua_SpecialFundsData.Get:totalReward()
	return DataCenter.operationalActUtil:getActConstConfig("overflow_fund_total_reward", 4);
end

-- 是否强制关闭超值基金
function Lua_SpecialFundsData.Get:isCloseFunds()
	return DataCenter.operationalActUtil:getActConstConfig("overflow_fund_stop", 1) == 1;
end