Lua_FormTeamCompetitionData = class("Lua_FormTeamCompetitionData")
function Lua_FormTeamCompetitionData:ctor()
	self.close_tick = 0;
	self.curStageEndTick = 0;
end

function Lua_FormTeamCompetitionData:dispose()
	self.curCompetitionState = nil;
	self.curCompetitionStage = nil;
	self.curStageEndTick = 0;
	self.close_tick = 0;
	self.curCompetitionNum = nil;
	self.joinLimiting = nil;
	self.leaderTeamIndexs = nil;

	self.teamId = nil;
	self.bLeader = nil;
	self.teamScore = nil;
	self.teamForceNum = nil;
	self.teamDataDir = nil;
	self.teamIndexs = nil;
	self.rewardDic = nil;
	self.fightTeamDir = nil;
	self.fightTeamIndexes = nil;
	self.exchangeTeamIndex = nil;
	self.selectedPlayerId = nil;
	self.selectedPlayerIndex = nil;
	self.mySelfTeamInfo = nil;
	self.saveChatInputStr = nil;
	self.bMemberRecruitInfo = nil;
	self.curRewardSettleNum = nil;
	self.bInvitateInfo = nil;
end

function Lua_FormTeamCompetitionData:fromServer50215001(msg)
	if msg == nil then
		return;
	end

	self.curCompetitionState = msg.step; -- 0未开启 1组队 2战斗 3结束组队
	self.curStageEndTick = tonumber(msg.step_tick) or 0; -- 阶段结束时间戳
	self.close_tick = tonumber(msg.close_tick) or 0; 
	self.curCompetitionNum = tonumber(msg.settle);
	self.curRewardSettleNum = msg.reward_settle;
	self.bApplicateName = msg.join == 1;


	local data = msg.my_team;
	if data ~= nil then
		local playerId;
		self.teamId = tonumber(data.team_id);
		self.curCompetitionStage = data.grade;
	else
		self.teamId = nil;
		self.curCompetitionStage = nil;
	end

	self.bLeader = tonumber(msg.is_leader) == 1;
	self:setMySelfTeamInfo(msg.my_team);
end

function Lua_FormTeamCompetitionData:fromServer50215004(msg)
	if msg == nil then
		return;
	end

	self.curRewardSettleNum = msg.reward_settle;
	local data = msg.reward_state;
	for i = 1, #data do
		self:updateRewardDicById(tonumber(data[i].key), data[i].value);
	end

	self:updateRewardRedPoint();
end

--请求组队赛挑战界面(返回)
function Lua_FormTeamCompetitionData:fromServer50215009(msg)
	if msg == nil then
		return;
	end

	local data = msg.fight_list;
	self:updateFightTeamDir(data);
	self:setMySelfTeamInfo(msg.my_team);
end

--请求组队赛挑战阵容界面(返回)
function Lua_FormTeamCompetitionData:fromServer50215010(msg)
	if msg == nil then
		return;
	end

	self.fightedNum = msg.fight_times; -- 已战斗次数

	self:setMySelfTeamInfo(msg.my_team);
end

-- 请求组队赛个人房间信息(返回)
function Lua_FormTeamCompetitionData:fromServer50215021(msg)
	if msg == nil then
		return;
	end

	self:updateApplicationRedPoint(msg);
	self:setMySelfTeamInfo(msg.my_team);
end

-- 请求创建组队赛队伍(返回)
function Lua_FormTeamCompetitionData:fromServer50215013(msg)
	if msg == nil then
		return;
	end

	self.teamId = tonumber(msg.team_id);
end

function Lua_FormTeamCompetitionData:updateTeamDataDir(teamdata)
	self.teamDataDir = {};

	if teamdata == nil then
		self:resetLeaderTeamIndex();
		return
	end

	local teamIndex = { 0, 0, 0};

	local obj;
	local index;
	local bMySelf;
	local playerId;
	local info = teamdata.member_show;
	for i = 1, #info do
		obj = info[i];
		index = obj and obj.index or 0;
		playerId = tonumber(obj.id)
		if playerId ~= nil and playerId > 0 then
			teamIndex[index] = 1;
			self.teamDataDir[playerId] = obj;
			self:updateLeaderTeamIndex(playerId, index);

			bMySelf = DataCenter.roleData:isMyself(playerId);
			if bMySelf then
				self.bLeader = obj.job == 1;
				self.teamId = tonumber(teamdata.team_id);
				self.curCompetitionStage = teamdata.grade;
			end
		end
	end

	for i = 1, #teamIndex do
		if teamIndex[i] ~= 1 then
			self:updateLeaderTeamIndex(0, i);
		end
	end
end

function Lua_FormTeamCompetitionData:setTeamIndexs()
	self.teamIndexs = {};
	local indexes = self.leaderTeamIndexs;
	if indexes ~= nil then
		for i = 1, #indexes do
			self.teamIndexs[i] = indexes[i];
		end
	end
end

function Lua_FormTeamCompetitionData:getPlayerDataByPlayerId(playerId)
	playerId = tonumber(playerId);
	if self.teamDataDir == nil or self.teamDataDir[playerId] == nil then
		return;
	end

	return self.teamDataDir[playerId];
end

function Lua_FormTeamCompetitionData:resetTeamIndex()
	self.teamIndexs = { 0, 0, 0};
end

function Lua_FormTeamCompetitionData:resetLeaderTeamIndex()
	self.leaderTeamIndexs = { 0, 0, 0};
end

function Lua_FormTeamCompetitionData:updateTeamIndex(playerId, index)
	if self.teamIndexs == nil then
		self:resetTeamIndex();
	end

	if playerId == nil or index == nil then
		return;
	end

	self.teamIndexs[index] = playerId;
end

function Lua_FormTeamCompetitionData:updateLeaderTeamIndex(playerId, index)
	if self.leaderTeamIndexs == nil then
		self:resetLeaderTeamIndex();
	end

	if playerId == nil or index == nil then
		return;
	end

	self.leaderTeamIndexs[index] = playerId;
end

function Lua_FormTeamCompetitionData:getPlayerIdByIndex(index)
	if self.teamIndexs == nil or self.teamIndexs[index] == nil then
		return;
	end

	return self.teamIndexs[index];
end

function Lua_FormTeamCompetitionData:getLeaderTeamsPlayerIdByIndex(index)
	if self.leaderTeamIndexs == nil or self.leaderTeamIndexs[index] == nil then
		return;
	end

	return self.leaderTeamIndexs[index];
end





function Lua_FormTeamCompetitionData:setSelectedPlayerId(playerId, index)
	self.selectedPlayerId = playerId;
	self.selectedPlayerIndex = index;
end

function Lua_FormTeamCompetitionData:checkCurTimeType() -- 1:未开始， 2：报名期中   3：战斗期  4：休战期
	local timeType;
	if self.curCompetitionState == 0 then -- 0未开启 1组队 2战斗 3结束组队
		timeType = 1;
	elseif self.curCompetitionState == 1 then
		timeType = 2;
	elseif self.curCompetitionState == 2 then
		timeType = 3;
	elseif DataCenter.formTeamCompetitionUtil:checkNoFightStage(self.close_tick) then
		timeType = 4;
	-- elseif self.curCompetitionState == 3 then
		-- timeType = 4;
	end

	return timeType, self.curStageEndTick;
end

function Lua_FormTeamCompetitionData:updateRewardDicById(rewardId, state) 
	if self.rewardDic == nil then
		self.rewardDic = {};
	end

	self.rewardDic[rewardId] = state; -- 0不可领取 1可领取 2已领取
end

function Lua_FormTeamCompetitionData:getRewardDataById(id) -- team_arena_reward配置表id
	local rewardId = DataCenter.formTeamCompetitionUtil:calRewardIdByType(id, self.curRewardSettleNum, 2)

	if self.rewardDic == nil or self.rewardDic[rewardId] == nil then
		return;
	end

	return self.rewardDic[rewardId];
end

function Lua_FormTeamCompetitionData:getRewardIdsByType(type)
	local ids = DataCenter.formTeamCompetitionUtil:getRewardIdsByType(type);
	if type == Lua_FormTeamCompetitionUtil.PROCESSREWARD then
		table.sort( ids, function(a, b) 
				local aState = self:getRewardDataById(a) or -1;
				local bState = self:getRewardDataById(b) or -1;
				if aState == bState then
					return a < b;
				else
					if bState == 1 then
						return false;
					elseif aState ~= 1 and bState == 0 then
						return false;
					end
				end
				return true;
			end);
	end
	return ids;
end

function Lua_FormTeamCompetitionData:updateFightTeamDir(fightTeamData)
	self.fightTeamDir = {};
	self.fightTeamIndexes = {};

	if fightTeamData == nil then
		return;
	end

	local index;
	local teamId;
	local teamObj;
	for i = 1, #fightTeamData do
		teamObj = fightTeamData[i] or {};
		teamId = teamObj.fight_team and teamObj.fight_team.team_id or i;
		teamId = tonumber(teamId);
		index = i;
		self.fightTeamDir[teamId] = teamObj;
		self.fightTeamIndexes[index] = teamId;
	end
	self:updateFightNumRedPoint();
end

function Lua_FormTeamCompetitionData:getFightTeamDirByTeamId(teamId)
	if self.fightTeamDir == nil or self.fightTeamDir[teamId] == nil then
		return;
	end

	return self.fightTeamDir[teamId];
end

function Lua_FormTeamCompetitionData:getFightTeamIndexes()
	return self.fightTeamIndexes;
end

function Lua_FormTeamCompetitionData:recordExchangeTeamIndex(index)
	self.exchangeTeamIndex = index;
end

function Lua_FormTeamCompetitionData:checkPassFight(bTip)
	local needNum = getConfig("team_arena_const", "skip_fight", "value") or 0;
	local enable = self.fightedNum >= needNum;
	if enable ~= true and bTip then
		local str = LanUtil.formateStr(772126, needNum);
		OperateAlert.show(str);
	end

	return enable;
end

function Lua_FormTeamCompetitionData:checkLeader()
	return self.bLeader == true;
end

function Lua_FormTeamCompetitionData:setJoinLimiting(value)
	self.joinLimiting = value;
end

function Lua_FormTeamCompetitionData:checkTeamEnoughPeroson()
	if self.leaderTeamIndexs == nil then
		self:resetLeaderTeamIndex();
	end

	local bEnough = true;
	local teamIndexes = self.leaderTeamIndexs;
	for i = 1, #teamIndexes do
		if teamIndexes[i] == nil or teamIndexes[i] == 0 then
			bEnough = false;
			break;
		end
	end

	return bEnough;
end

-- 报名
function Lua_FormTeamCompetitionData:checkApplicateName()
	return self.bApplicateName;
end

-- 最后一天
function Lua_FormTeamCompetitionData:checkCompetitionLast()
	local closeTick = self.close_tick or 0;
	local bCompetitionLast = DataCenter.formTeamCompetitionUtil:checkCompetitionLast(closeTick);
	return bCompetitionLast;
end

-- 战斗期 期间内每天的休战期
function Lua_FormTeamCompetitionData:checkCompetitionNoFightInFightingDay()
	if self.curCompetitionState == 2 then
		local curTime = DataCenter.roleData.serverTime;
		return DataCenter.formTeamCompetitionUtil:checkCompetitionNoFightInFightingDay(curTime);
	end
end


function Lua_FormTeamCompetitionData:checkJoinTeam()
	return self.teamId ~= nil and self.teamId > 0;
end

function Lua_FormTeamCompetitionData:checkFightingPlayers()
	local bShow;
	local teamIds = self:getFightTeamIndexes();
	if teamIds ~= nil and #teamIds > 0 then
		bShow = teamIds[1] > 0;
	end

	return bShow;
end

function Lua_FormTeamCompetitionData:getCurCompetitionStage()
	return self.curCompetitionStage;
end

function Lua_FormTeamCompetitionData:setMySelfTeamInfo(myTeamData)
	self.mySelfTeamInfo = myTeamData;

	self:updateTeamDataDir(myTeamData);
	self:updateMemberRecruitRedPoint();
	self:updateInvitateInfoRedPoint();
	Lua_MsgUtil.callFunc("UPDATE_FORMTEAMCOMPETITION_MYTEAM_TEAMINFO");
end

function Lua_FormTeamCompetitionData:getMySelfTeamInfo()
	return self.mySelfTeamInfo;
end

function Lua_FormTeamCompetitionData:setSaveChatInfo(str)
	self.saveChatInputStr = str;
end

function Lua_FormTeamCompetitionData.Get:teamId()
	return self._teamId;
end

function Lua_FormTeamCompetitionData.Set:teamId(value)
	if self._teamId ~= nil and self._teamId > 0 and self._teamId ~= value then
		DataCenter.chatData:clearChatDataByChannel(Lua_ChatData.CHANNEL_FORMTEAMCOMPETITION);
	end
	self._teamId = value;
end

function Lua_FormTeamCompetitionData.Get:curRewardSettleNum()
	return self._curRewardSettleNum or 1;
end

function Lua_FormTeamCompetitionData.Set:curRewardSettleNum(value)
	self._curRewardSettleNum = value;
end


----------------红点----------------
function Lua_FormTeamCompetitionData:updateRewardRedPoint()
	local redPointState;
	local ids = DataCenter.formTeamCompetitionUtil:getRewardIdsByType(Lua_FormTeamCompetitionUtil.PROCESSREWARD);
	if ids ~= nil and #ids > 0 then
		for i = 1, #ids do
			if self:getRewardDataById(ids[i]) == 1 then
				redPointState = true;
				break;
			end
		end
	end

	DataCenter.redPointData:updateRedPointState(Lua_RedPointList.FormTeamCompetation_OverViewTab_Reward, redPointState, false);
end

-- 存在未挑战的
function Lua_FormTeamCompetitionData:updateFightNumRedPoint()
	local bShow;
	local bLastDay = self:checkCompetitionLast();
	local bBanFight = self:checkCompetitionNoFightInFightingDay();
	if bLastDay ~= true and bBanFight ~= true then
		local teamIds = self:getFightTeamIndexes();
		if teamIds ~= nil then
			local data;
			for i = 1, #teamIds do
				data = self:getFightTeamDirByTeamId(teamIds[i]);
				if data ~= nil and data.win ~= 1 and data.fight_times <= 0 then
					bShow = true;
					break;
				end
			end
		end
	end

	DataCenter.redPointData:updateRedPointState(Lua_RedPointList.FormTeamCompetation_FightTab_FightNum, bShow, false);
end

-- 挑战记录未查看
function Lua_FormTeamCompetitionData:updateFightRecordRedPoint(bShow)

	DataCenter.redPointData:updateRedPointState(Lua_RedPointList.FormTeamCompetation_OverViewTab_Record, bShow, true);
	DataCenter.redPointData:updateRedPointState(Lua_RedPointList.FormTeamCompetation_FightTab_Record, bShow, true);
end

-- 人数已满
function Lua_FormTeamCompetitionData:updateApplicationRedPoint(msg)
	local bShow;
	if DataCenter.formTeamCompetitionData:checkCompetitionLast() then
		bShow = false;
	elseif msg.join ~= 1 and self.bLeader then
		bShow = true;
		local info;
		local count = 0;
		local data = msg.my_team and msg.my_team.member_show or {};
		for i = 1, #data do
			info = data[i];
			if info and tonumber(info.id) > 0 then
				count = count + 1;
				if info.already == 0 and info.job ~= 1 then
					bShow = false;
					break;
				end
			end
		end

		if bShow and count < 3 then
			bShow = false;
		end
	end

	DataCenter.redPointData:updateRedPointState(Lua_RedPointList.FormTeamCompetation_TeamTab_Application, bShow, false);
end

-- 己方队伍成员招募
function Lua_FormTeamCompetitionData:updateMemberRecruitRedPoint(bShow)
	local bLeader = self:checkLeader();
	if not bLeader then
		bShow = false;
	else
		if bShow == nil then 
			bShow = self.bMemberRecruitInfo;--DataCenter.redPointData:getRedPointStateById(Lua_RedPointList.FormTeamCompetation_TeamTab_MemberRecruit);
		else
			self.bMemberRecruitInfo = bShow;
		end
	end

	if bShow then
		local checkPerson = self:checkTeamEnoughPeroson();
		if checkPerson then
			bShow = false;
		end
	end
	DataCenter.redPointData:updateRedPointState(Lua_RedPointList.FormTeamCompetation_TeamTab_MemberRecruit, bShow, false);
end

-- 队伍大厅邀请信息
function Lua_FormTeamCompetitionData:updateInvitateInfoRedPoint(bShow)
	if bShow == nil then 
		bShow = self.bInvitateInfo;
	else
		self.bInvitateInfo = bShow;
	end

	if bShow then
		if self:checkJoinTeam() then
			bShow = false;
		end
	end
	DataCenter.redPointData:updateRedPointState(Lua_RedPointList.FormTeamCompetation_TeamTab_InvitateInfo, bShow, false);
end