Lua_FormTeamCompetitionUtil = class("Lua_FormTeamCompetitionUtil")
Lua_FormTeamCompetitionUtil.COMPETITIONREWARD = 1; -- 赛季奖励
Lua_FormTeamCompetitionUtil.DAYREWARD = 2; -- 每日奖励
Lua_FormTeamCompetitionUtil.PROCESSREWARD = 3; -- 进度奖励

Lua_FormTeamCompetitionUtil.LEADERPASS = 0; -- 队长通过
Lua_FormTeamCompetitionUtil.EVERYONEPASS = 1;-- 任何人可加入

Lua_FormTeamCompetitionUtil.FRIENDTYPE = 1; -- 盟友招募
Lua_FormTeamCompetitionUtil.RECOMMENDTYPE = 2; -- 推荐招募
Lua_FormTeamCompetitionUtil.APPLYFORTYPE = 3;-- 申请列表

function Lua_FormTeamCompetitionUtil:ctor()
end

function Lua_FormTeamCompetitionUtil:dispose()
	self.rewardIds = nil;
	self.lastRewardSettleNum = nil;
end

function Lua_FormTeamCompetitionUtil:getRewardIdsByType(type)
	local curRewardSettleNum = DataCenter.formTeamCompetitionData.curRewardSettleNum;
	if self.lastRewardSettleNum ~= curRewardSettleNum then
		self.rewardIds = nil;
	end

	if self.rewardIds == nil then
		self.rewardIds = { 
			[Lua_FormTeamCompetitionUtil.COMPETITIONREWARD] = {},
			[Lua_FormTeamCompetitionUtil.DAYREWARD] = {},
			[Lua_FormTeamCompetitionUtil.PROCESSREWARD] = {}
		};

		local c_type;
		local ids;
		local config = getConfig("team_arena_reward");
		for k, v in pairs(config) do
			if v.season == curRewardSettleNum then
				if v.settle_reward ~= nil then
					c_type = Lua_FormTeamCompetitionUtil.COMPETITIONREWARD;
					ids = self.rewardIds[c_type];
					ids[#ids + 1] = k;
				end

				if v.daily_reward ~= nil then
					c_type = Lua_FormTeamCompetitionUtil.DAYREWARD;
					ids = self.rewardIds[c_type];
					ids[#ids + 1] = k;
				end


				if v.process_reward ~= nil then
					c_type = Lua_FormTeamCompetitionUtil.PROCESSREWARD;
					ids = self.rewardIds[c_type];
					ids[#ids + 1] = k;
				end
			end
		end

		for i = 1, #self.rewardIds do
			table.sort( self.rewardIds[i], function(a, b) 
					return a > b;
				end);
		end

		self.lastRewardSettleNum = curRewardSettleNum;
	end

	return self.rewardIds[type];
end


function Lua_FormTeamCompetitionUtil:getTeamLevel(gradeId, type) -- type:1： 只查看段位  2：段位，url
	type = type or 1;
	gradeId = gradeId or 101;
	local cfg = getConfig("team_arena_grade", gradeId);
	if cfg == nil then
		return;
	end

	local levelStr = cfg.name;
	if type == 1 then
		return levelStr;
	elseif type == 2 then
		return levelStr, cfg.path;
	end
end

function Lua_FormTeamCompetitionUtil:checkFightBtnState(fightNum)
	fightNum = fightNum or 0;
	local cfg = getConfig("team_arena_const", "fight_cost", "jsonvalue");
	if cfg == nil then
		return;
	end

	local costId, costNum;
	local index = fightNum > #cfg and #cfg or fightNum;
	if cfg[index] ~= nil then
		costId = 201000001;
		costNum = cfg[index];
	end
	
	return costId, costNum;
end

function Lua_FormTeamCompetitionUtil:checkCompetitionLast(close_tick)
	local bCompetitionLast;
	close_tick = tonumber(close_tick) or 0;
	local limitDay = getConfig("team_arena_const", "close_team_day", "value") or 0;
	bCompetitionLast = close_tick - DataCenter.roleData.serverTime <= 86400 * limitDay;
	return bCompetitionLast;
end

function Lua_FormTeamCompetitionUtil:getCompetitionStartTime(close_tick)
	local startTick;
	close_tick = tonumber(close_tick) or 0;
	if close_tick > 0 then
		local limitDay = getConfig("team_arena_const", "team_up_day", "value") or 0;
		limitDay = limitDay + getConfig("team_arena_const", "fight_day", "value") or 0;
		startTick = close_tick - 86400 * limitDay;
	else
		startTick = 0;
	end
	return startTick;
end

-- 休赛期
function Lua_FormTeamCompetitionUtil:checkNoFightStage(close_tick)
	close_tick = close_tick or 0;
	close_tick = tonumber(close_tick) - 86400 or 0;
	local time;
	if close_tick > 0 then
		local cfg = getConfig("team_arena_const", "settle_stop_time", "jsonvalue");
		local hour = cfg ~= nil and cfg[1] or 0;
		local min = cfg ~= nil and cfg[2] or 0;

		local curDay = os.date("%d", close_tick);
		local curMonth = os.date("%m", close_tick);
		local curYear = os.date("%Y", close_tick);
		time = os.time({year = curYear, month = curMonth, day = curDay, month = curMonth, hour = hour, min = min});
	else
		time = 0;
	end
	return time - DataCenter.roleData.serverTime <= 0;
end

-- 战斗期 期间内 不可战斗
function Lua_FormTeamCompetitionUtil:checkCompetitionNoFightInFightingDay(curTime)
	local time;
	curTime = tonumber(curTime) or 0;
	if curTime > 0 then
		local cfg = getConfig("team_arena_const", "daily_stop_time", "jsonvalue");
		local hour = cfg ~= nil and cfg[1] or 0;
		local min = cfg ~= nil and cfg[2] or 0;

		local curDay = os.date("%d", curTime);
		local curMonth = os.date("%m", curTime);
		local curYear = os.date("%Y", curTime);
		time = os.time({year = curYear, month = curMonth, day = curDay, month = curMonth, hour = hour, min = min});
	else
		time = 0;
	end

	return time - DataCenter.roleData.serverTime <= 0;
end

function Lua_FormTeamCompetitionUtil:calRewardIdByType(id, rewardSettle, type) --rewardSettle:奖励赛季
	if id == nil then
		return;
	end

	type = type or 1;
	rewardSettle = rewardSettle or 1;
	local rewardId;
	if type == 1 then -- 计算team_arena_reward配置表id
		if getConfig("team_arena_grade", id) == nil then 
			return;
		end

		rewardId = id + rewardSettle * 1000;
	else -- 计算team_arena_grade配置表id
		if getConfig("team_arena_reward", id) == nil then 
			return;
		end

		rewardId = id % 1000;
	end

	return rewardId;
end

