-- 垂钓控制器
-- 以下动作或有共用情况但含义不一样
-- 钓鱼流程：
-- @无鱼获待机动作->甩竿动作->等鱼动作->请求服务端数据->返回后执行1/2/3，没有返回执行4
-- 1 鱼没有上钩跑了，播放脱钩动作，之后回到@
-- 2 鱼上钩了且鱼线断了，播放断线动作，之后回到@
-- 3 鱼上钩了且鱼线没断，播放上钩动作，之后停在有鱼获待机动作（这里的有鱼获藏品/杂物/鱼/水中生物都有可能，处理完回到@）
-- 4 直接执行断线动作，之后回到@
Lua_GoFishingController = class("Lua_GoFishingController");
-- 钓鱼动作
Lua_GoFishingController.Action_NoFishIdle = 1; -- 待机动画-无鱼获
Lua_GoFishingController.Action_SpinningRod = 2; -- 甩竿动画
Lua_GoFishingController.Action_WaitingFish = 3; -- 等鱼动画
Lua_GoFishingController.Action_SmallFish = 4; -- 小鱼动画
Lua_GoFishingController.Action_BigFish = 5; -- 大鱼动画
Lua_GoFishingController.Action_Break = 6; -- 断线动画
Lua_GoFishingController.Action_GetFish = 7; -- 上钩动画
Lua_GoFishingController.Action_HasFishIdle = 8; -- 待机动画-有鱼获
Lua_GoFishingController.Action_NoFish = 9; -- 脱钩动画

-- 钓鱼动作特效
Lua_GoFishingController.ActionEff_NoFishIdle = "cdzj_fish_01"; -- 待机动画-无鱼获
Lua_GoFishingController.ActionEff_SpinningRod = "cdzj_fish_02"; -- 甩竿动画
Lua_GoFishingController.ActionEff_WaitingFish = "cdzj_fish_03"; -- 等鱼动画
Lua_GoFishingController.ActionEff_SmallFish = "cdzj_fish_04"; -- 小鱼动画
Lua_GoFishingController.ActionEff_BigFish = "cdzj_fish_05"; -- 大鱼动画
Lua_GoFishingController.ActionEff_Break = "cdzj_fish_06"; -- 断线动画
Lua_GoFishingController.ActionEff_GetFish = "cdzj_fish_07"; -- 上钩动画
Lua_GoFishingController.ActionEff_HasFishIdle = "cdzj_fish_08"; -- 待机动画-有鱼获
Lua_GoFishingController.ActionEff_NoFish = "cdzj_fish_07"; -- 脱钩动画
function Lua_GoFishingController:ctor()
	self:superCall("ctor");
	self._isFishing = false;
	self._fishingStep = Lua_GoFishingController.Action_NoFishIdle;
	self._fishingTime = 0;
	self._isAuto = false;
	self._goFishingCheckTime = 0;
end

function Lua_GoFishingController:dispose()
	self:stopTimer();
	self.owner = nil;
	self.callBack = nil;
	self._isFishing = false;
	self._fishingStep = Lua_GoFishingController.Action_NoFishIdle;
	self._fishingTime = 0;
	self._isAuto = false;
	self._goFishingCheckTime = 0;
end

function Lua_GoFishingController:initState()
	if self._isAuto then
		local aniStartTime = self:getStepTime(self._fishingStep) - self._fishingTime;
		self:aniCallBackHandle(aniStartTime);
	else
		local getFish = DataCenter.goFishingData.getFish;
		if getFish.fishing and getFish.fishing > 0 then
			self._fishingStep = Lua_GoFishingController.Action_HasFishIdle; -- 待机-有鱼获
		else
			self._fishingStep = Lua_GoFishingController.Action_NoFishIdle; -- 待机-无鱼获
		end
		self._fishingTime = 0;
		self._isFishing = false;
		self:aniCallBackHandle(0);
	end
end

function Lua_GoFishingController:fromServer50242007()
	if not self._isFishing then
		return;
	end

	self._goFishingCheckTime = 0;
	if DataCenter.goFishingData.isAttach then
		if DataCenter.goFishingData.size == 1 then
			self._fishingStep = Lua_GoFishingController.Action_SmallFish; -- 小鱼
		else
			self._fishingStep = Lua_GoFishingController.Action_BigFish; -- 大鱼
		end
	else
		self._fishingStep = Lua_GoFishingController.Action_NoFish; -- 脱钩
	end
	self._fishingTime = self:getStepTime(self._fishingStep);
	self:aniCallBackHandle(0);
end

-- 开始钓鱼
function Lua_GoFishingController:playGoFishing(isAuto)
	self._isAuto = isAuto;
	if self._isFishing then
		return;
	end
	self._isFishing = true;

	self._fishingStep = Lua_GoFishingController.Action_SpinningRod; -- 甩竿
	self._fishingTime = self:getStepTime(self._fishingStep);
	self:aniCallBackHandle(0);
	self:startTimer();
end

function Lua_GoFishingController:startTimer()
	if self.bTimer ~= true then
		self.bTimer = true;
		self.startTime = getTime();
		Lua_CommonUtil.addTimer("Lua_GoFishingController", 0.01, self, self.onTimer);
	end
end

function Lua_GoFishingController:stopTimer()
	if self.bTimer == true then
		self.bTimer = false;
		Lua_CommonUtil.removeTimer(self, self.onTimer);
	end
end

function Lua_GoFishingController:onTimer()
	local t = getTime();
	local dt = t - self.startTime;
	self.startTime = t;

	if self._goFishingCheckTime > 0 then
		self._goFishingCheckTime = self._goFishingCheckTime - dt;
		if self._goFishingCheckTime <= 0 then
			self._goFishingCheckTime = 0;
			self:removeGoFishingCheck();
		end
		return;
	end

	self._fishingTime = self._fishingTime - dt;
	if self._fishingTime > 0 then
		return;
	end

	if self._fishingStep == Lua_GoFishingController.Action_NoFishIdle then
		-- 结束当前钓鱼流程，进行下一轮自动钓鱼处理
		self:stopTimer();
		self:autoGoFishing();
	elseif self._fishingStep == Lua_GoFishingController.Action_SpinningRod then
		-- 甩竿结束，进入等鱼
		self._fishingStep = Lua_GoFishingController.Action_WaitingFish;
		self._fishingTime = self:getStepTime(self._fishingStep);
		self:aniCallBackHandle(0);
	elseif self._fishingStep == Lua_GoFishingController.Action_WaitingFish then
		-- 等鱼结束，请求钓鱼，处理容错
		Lua_ProxyManager.getProxy("Lua_GoFishingProxy"):sStartGoFishing();
		self._goFishingCheckTime = DataCenter.goFishingUtil.fishingTime;
	elseif self._fishingStep == Lua_GoFishingController.Action_SmallFish or self._fishingStep == Lua_GoFishingController.Action_BigFish then
		if DataCenter.goFishingData.isGet then
			self._fishingStep = Lua_GoFishingController.Action_GetFish; -- 上钩
		else
			self._fishingStep = Lua_GoFishingController.Action_Break; -- 断线
		end
		self._fishingTime = self:getStepTime(self._fishingStep);
		self:aniCallBackHandle(0);
	elseif self._fishingStep == Lua_GoFishingController.Action_Break or self._fishingStep == 9 then
		-- 脱钩、断线，流程回到待机状态
		Lua_ProxyManager.getProxy("Lua_GoFishingProxy"):sGoFishingCheckout();
		self._isFishing = false;
		Lua_ProxyManager.getProxy("Lua_GoFishingProxy"):sGoFishingInfo(0);
		self._fishingStep = Lua_GoFishingController.Action_NoFishIdle;
		self._fishingTime = self:getStepTime(self._fishingStep);
		self:aniCallBackHandle(0);
	elseif self._fishingStep == Lua_GoFishingController.Action_GetFish then
		-- 上钩结束，进入中鱼展示
		Lua_ProxyManager.getProxy("Lua_GoFishingProxy"):sGoFishingCheckout();
		self._fishingStep = Lua_GoFishingController.Action_HasFishIdle;
		self._fishingTime = self:getStepTime(self._fishingStep);
		self:aniCallBackHandle(0);
	elseif self._fishingStep == Lua_GoFishingController.Action_HasFishIdle then
		-- 成功钓鱼，结束当前钓鱼流程，进行下一轮自动钓鱼处理
		self:stopTimer();
		self._isFishing = false;
		self:autoGoFishing();
		Lua_ProxyManager.getProxy("Lua_GoFishingProxy"):sGoFishingInfo(0);
	end
end

function Lua_GoFishingController:autoGoFishing()
	if not self._isAuto then
		return;
	end
	self:checkAutoHandel();
end

function Lua_GoFishingController:checkAutoHandel(operateType)
	local totalfiveSaveNum = DataCenter.goFishingData.totalfiveSaveNum;
	local saveFish = DataCenter.goFishingData.saveFish; -- 鱼护中的鱼
	local getFish = DataCenter.goFishingData.getFish; -- 未处理的鱼
	local curNum = (getFish.fishing and getFish.fishing > 0) and #saveFish + 1 or #saveFish;
	if curNum >= totalfiveSaveNum + 1 then
		local sub_type = getConfig("item", getFish.fishing, "sub_type");
		if sub_type == Lua_PackDefine.GOFISHING_RUBBISH then -- 杂物
			if self.isFishIndent then
				Lua_ProxyManager.getProxy("Lua_GoFishingProxy"):sDealWithFish(getFish.unique_id, 2, 0); -- 提交订单
			else
				Lua_ProxyManager.getProxy("Lua_GoFishingProxy"):sDealWithFish(getFish.unique_id, 0, 0); -- 出售
			end
			self:checkAutoProtection();
		elseif sub_type == Lua_PackDefine.GOFISHING_COLLECTION then -- 藏品
			Lua_ProxyManager.getProxy("Lua_GoFishingProxy"):sDealWithFish(getFish.unique_id, 3, 0); -- 放入背包
			self:checkAutoProtection();
		else -- 鱼/水中生物
			if self.isFishIndent then
				Lua_ProxyManager.getProxy("Lua_GoFishingProxy"):sDealWithFish(getFish.unique_id, 2, 0); -- 提交订单
				self:checkAutoProtection();
			else
				self._isAuto = false;
				OperateAlert.showLangValue(794089);
				if operateType == 1 then
					Lua_WindowUtil.openWin("Lua_GoFishingGetResultWindow", Lua_WinParam.ACTION_OPEN);
				end
			end
		end
	else
		if getFish.fishing and getFish.fishing > 0 then -- 有未处理的鱼获
			local sub_type = getConfig("item", getFish.fishing, "sub_type");
			if sub_type == Lua_PackDefine.GOFISHING_RUBBISH then -- 杂物
				if self.isFishIndent then
					Lua_ProxyManager.getProxy("Lua_GoFishingProxy"):sDealWithFish(getFish.unique_id, 2, 0); -- 提交订单
				else
					Lua_ProxyManager.getProxy("Lua_GoFishingProxy"):sDealWithFish(getFish.unique_id, 0, 0); -- 出售
				end
			elseif sub_type == Lua_PackDefine.GOFISHING_COLLECTION then -- 藏品
				Lua_ProxyManager.getProxy("Lua_GoFishingProxy"):sDealWithFish(getFish.unique_id, 3, 0); -- 放入背包
			else -- 鱼/水中生物
				Lua_ProxyManager.getProxy("Lua_GoFishingProxy"):sDealWithFish(getFish.unique_id, 1, 0); -- 存入鱼护
			end
		end
		self:checkAutoProtection();
	end
end

function Lua_GoFishingController:checkAutoProtection()
	if DataCenter.goFishingData.gofishingOpenTime <= 0 then
		self._isAuto = false;
	end

	local hasCount = DataCenter.packData:getItemCountById(Lua_ResourcesDefine.GOFISHING_STAMINA);
	if hasCount <= 0 then
		self._isAuto = false;
	end

	local getFish = DataCenter.goFishingData.getFish;
	if getFish and getFish.fishing > 0 then
		local color = getConfig("item", getFish.fishing, "color") or 0;
		if self._isCheckNewAtlas and getFish.is_new == 2 then -- 新获得
			self._isAuto = false;
			OperateAlert.showLangValue(794090);
		elseif self._isCheckNewRecord and (getFish.is_new == 2 or getFish.is_new == 1) then -- 新记录
			self._isAuto = false;
			OperateAlert.showLangValue(794090);
		elseif self._isCheckQuality and color >= self.quality then -- 品质
			self._isAuto = false;
			OperateAlert.showLangValue(794090);
		end
	end

	if not self._isAuto then
		return;
	end
	self:playGoFishing(true);
end

function Lua_GoFishingController.Get:isFishIndent()
	local getFish = DataCenter.goFishingData.getFish;
	local sub_type = getConfig("item", getFish.fishing, "sub_type");
	local fishingIndentList = DataCenter.goFishingData.fishingIndentList;
	for i = 1, #fishingIndentList do
		taskInfo = DataCenter.goFishingUtil:getFishTaskInfo(fishingIndentList[i].task_id);
		if taskInfo and taskInfo.temp
			and (getFish.fishing == taskInfo.temp or sub_type == taskInfo.temp or (taskInfo.task_type == 5 and tvo.color >= taskInfo.temp))
			and taskInfo.process > fishingIndentList[i].process then
				return true;
		end
	end
	return false;
end

function Lua_GoFishingController:removeGoFishingCheck()
	LogManager.LogError(LanUtil.formateStr("超过{1}秒服务端没有返回，直接执行断线处理", DataCenter.goFishingUtil.fishingTime), false);
	self._fishingStep = Lua_GoFishingController.Action_Break;
	self._fishingTime = self:getStepTime(self._fishingStep);
	self:aniCallBackHandle(0);
end

function Lua_GoFishingController:addAniSetFun(owner, callBack)
	self.owner = owner;
	self.callBack = callBack;
end

function Lua_GoFishingController:aniCallBackHandle(time)
	if self.owner and self.callBack then
		time = time or 0;
		if self._fishingStep == Lua_GoFishingController.Action_NoFishIdle then -- 待机动画-无鱼获
			self.callBack(self.owner, self._fishingStep, Lua_GoFishingController.ActionEff_NoFishIdle, time);
		elseif self._fishingStep == Lua_GoFishingController.Action_SpinningRod then -- 甩竿动画
			self.callBack(self.owner, self._fishingStep, Lua_GoFishingController.ActionEff_SpinningRod, time);
		elseif self._fishingStep == Lua_GoFishingController.Action_WaitingFish then -- 等鱼动画
			self.callBack(self.owner, self._fishingStep, Lua_GoFishingController.ActionEff_WaitingFish, time);
		elseif self._fishingStep == Lua_GoFishingController.Action_SmallFish then -- 小鱼动画
			self.callBack(self.owner, self._fishingStep, Lua_GoFishingController.ActionEff_SmallFish, time);
		elseif self._fishingStep == Lua_GoFishingController.Action_BigFish then -- 大鱼动画
			self.callBack(self.owner, self._fishingStep, Lua_GoFishingController.ActionEff_BigFish, time);
		elseif self._fishingStep == Lua_GoFishingController.Action_Break then -- 断线动画
			self.callBack(self.owner, self._fishingStep, Lua_GoFishingController.ActionEff_Break, time);
		elseif self._fishingStep == Lua_GoFishingController.Action_GetFish then -- 上钩动画
			self.callBack(self.owner, self._fishingStep, Lua_GoFishingController.ActionEff_GetFish, time);
		elseif self._fishingStep == Lua_GoFishingController.Action_HasFishIdle then -- 待机动画-有鱼获
			self.callBack(self.owner, self._fishingStep, Lua_GoFishingController.ActionEff_HasFishIdle, time);
		elseif self._fishingStep == Lua_GoFishingController.Action_NoFish then -- 脱钩动画
			self.callBack(self.owner, self._fishingStep, Lua_GoFishingController.ActionEff_GetFish, time);
		end
	end
end

function Lua_GoFishingController:removeAniSetFun()
	self.owner = nil;
	self.callBack = nil;
end

function Lua_GoFishingController:getStepTime(value)
	if value == Lua_GoFishingController.Action_NoFishIdle then -- 待机-无鱼获
		return 2;
	elseif value == Lua_GoFishingController.Action_SpinningRod then -- 甩竿
		return 1.233;
	elseif value == Lua_GoFishingController.Action_WaitingFish then -- 等鱼
		local speed = DataCenter.goFishingData:getTheFishAttrValue("fish_speed");
		local time = 2 * (1 - speed);
		print("处理上鱼时间：", time);
		return time;
	elseif value == Lua_GoFishingController.Action_SmallFish then -- 小鱼
		return 2;
	elseif value == Lua_GoFishingController.Action_BigFish then -- 大鱼
		return 2;
	elseif value == Lua_GoFishingController.Action_Break then -- 断线
		return 1;
	elseif value == Lua_GoFishingController.Action_GetFish then -- 上钩
		return 1;
	elseif value == Lua_GoFishingController.Action_HasFishIdle then -- 待机-有鱼获
		return 0.5;
	elseif value == Lua_GoFishingController.Action_NoFish then -- 脱钩
		return 1;
	end
	return 0;
end

function Lua_GoFishingController.Get:isFishing()
	return self._isFishing;
end

function Lua_GoFishingController.Get:isAuto()
	return self._isAuto;
end

function Lua_GoFishingController.Set:isAuto(value)
	self._isAuto = value;
end

function Lua_GoFishingController.Set:isCheckQuality(value)
	self._isCheckQuality = value;
end

function Lua_GoFishingController.Get:isCheckQuality()
	return self._isCheckQuality;
end

function Lua_GoFishingController.Set:quality(value)
	self._quality = value;
end

function Lua_GoFishingController.Get:quality()
	return self._quality or 2;
end

function Lua_GoFishingController.Set:isCheckNewRecord(value)
	self._isCheckNewRecord = value;
end

function Lua_GoFishingController.Get:isCheckNewRecord()
	return self._isCheckNewRecord;
end

function Lua_GoFishingController.Set:isCheckNewAtlas(value)
	self._isCheckNewAtlas = value;
end

function Lua_GoFishingController.Get:isCheckNewAtlas()
	return self._isCheckNewAtlas;
end