-- 钓鱼
Lua_GoFishingData = class("Lua_GoFishingData");
function Lua_GoFishingData:ctor()
end

function Lua_GoFishingData:dispose()
	self._saveFish = nil;
	self._getFish = nil;
	self._privilegeTime = nil;
	self._aquariumId = nil;
	self._fisheryId = nil;
	self._fishTimes = nil;
	self._endTick = nil;
	self.fishingGearDic = nil;
	self._lureId = nil;
	self._fishingIndentList = nil;
	self._activeUp = nil;
	self._isAttach = nil;
	self._size = nil;
	self._isGet = nil;
	self._aquariumList = nil;
	self.fisheryDic = nil;
	self.fishingAllAtlasDic = nil;
	self.fishingAtlasDic = nil;
	self.isRefreshAtlas = true;
	self._firstPlayer = nil;
	self._firstFish = nil;
	self.isPopupEnter = nil; -- 是否弹过功能入口界面
end

function Lua_GoFishingData:fromServer50242001(msg)
	if msg == nil then
		return;
	end
	self.isRefreshSaveFish = true;
	self.isNeedRefreshAttrInfo = true;

	self._saveFish = msg.save_fish; -- 鱼护内保存鱼
	self._getFish = msg.get_fish; -- 已钓上鱼
	self._privilegeTime = msg.vip_open; -- 特权结束时间戳
	self._aquariumId = msg.room_level; -- 水族馆等级
	self._fisheryId = msg.fishery; -- 渔场id
	self._fishTimes = msg.fish_times; -- 总钓鱼次数
	self._endTick = msg.close_tick; -- 结束时间戳
	self:updateStaminaRedPointState();
	self:updateAquariumRedPointState();
end

function Lua_GoFishingData.Get:saveFish()
	local saveFishList = self._saveFish or {};
	if self.isRefreshSaveFish then
		self.isRefreshSaveFish = false;
		table.sort( saveFishList,
			function(a, b)
				local tvoA = Lua_ToolVo.createToolVo(a.fishing);
				local tvoB = Lua_ToolVo.createToolVo(b.fishing);
				if tvoA.color == tvoB then
					if tvoA.id == tvoB.id then
						return a.length > b.length;
					else
						return tvoA.id > tvoB.id;
					end
				else
					return tvoA.color > tvoB.color;
				end
			end
		);
	end
	return saveFishList;
end

function Lua_GoFishingData.Get:getFish()
	return self._getFish or {};
end

function Lua_GoFishingData.Get:privilegeTime()
	local time = self._privilegeTime or 0;
	return time - DataCenter.roleData.serverTime;
end

function Lua_GoFishingData.Get:aquariumId()
	return self._aquariumId or 101; -- 水族馆默认id
end

function Lua_GoFishingData.Get:aquariumInfo()
	return DataCenter.goFishingUtil:getTheAquariumInfo(self.aquariumId);
end

function Lua_GoFishingData.Get:fisheryId()
	return self._fisheryId or 1; -- 1默认渔场
end

function Lua_GoFishingData.Get:fishTimes()
	return self._fishTimes or 0;
end

function Lua_GoFishingData.Get:gofishingOpenTime()
	local time = self._endTick or 0;
	return time - DataCenter.roleData.serverTime;
end

function Lua_GoFishingData.Get:totalfiveSaveNum()
	local num = DataCenter.goFishingUtil.fishSaveNum;
	if self.privilegeTime > 0 then
		num = num + DataCenter.goFishingUtil.vipFishSaveNum;
	end
	return num;
end

function Lua_GoFishingData:fromServer50242002(msg)
	if msg == nil then
		return;
	end
	if self.fishingGearDic == nil then
		self.fishingGearDic = {};
	end

	self._lureId = tonumber(msg.lure);
	self.isNeedRefreshAttrInfo = true;

	local obj, config;
	for i = 1, #msg.gear_list do
		obj = msg.gear_list[i];
		config = DataCenter.goFishingUtil:getTheFishingGearInfo(obj.gear_id);
		if config then
			self.fishingGearDic[config.gear_type] = obj;
		end
	end
end

function Lua_GoFishingData:getTheFishGearInfo(gearType)
	if gearType == nil or self.fishingGearDic == nil or self.fishingGearDic[gearType] == nil then
		return nil;
	end
	return self.fishingGearDic[gearType];
end

function Lua_GoFishingData.Get:lureId()
	return self._lureId or Lua_ResourcesDefine.ENDLESSLURE;
end

function Lua_GoFishingData.Get:allGearAttrArr()
	local fishGearInfo, config, clearAttrDic;
	local dic = {};
	if self.fishingGearDic then
		for i = 1, 4 do
			fishGearInfo = self:getTheFishGearInfo(i);
			if fishGearInfo then
				config = DataCenter.goFishingUtil:getTheFishingGearInfo(fishGearInfo.gear_id);
				dic = DataCenter.attributesUtil:getAttrDic(config.base_prop, dic);
				clearAttrDic = {};
				if fishGearInfo.prop and #fishGearInfo.prop > 0 then
					for j = 1, #fishGearInfo.prop do
						clearAttrDic[fishGearInfo.prop[j].name] = fishGearInfo.prop[j].value;
					end
				end
				dic = DataCenter.attributesUtil:getAttrDic(clearAttrDic, dic);
			end
		end
	end
	return DataCenter.attributesUtil:getAttrArrByDic(dic);
end

function Lua_GoFishingData:fromServer50242005(msg)
	if msg == nil then
		return;
	end

	-- 领取>未达成>已完成，draw_state：0不可领取 1可领取 2已领取
	table.sort( msg.fish_task,
		function(a, b)
			if a.draw_state == b.draw_state then
				return a.task_id < b.task_id;
			else
				local aWeight = (a.draw_state + 1) % 3;
				local bWeight = (b.draw_state + 1) % 3;
				return aWeight > bWeight;
			end
		end
	);
	self._fishingIndentList = msg.fish_task; -- 每日订单任务状态
	self._activeUp = msg.active_up; -- 活动倍率提升 默认1

	local obj;
	local indentRedState = false;
	for i = 1, #msg.fish_task do
		if msg.fish_task[i].draw_state == 1 then
			indentRedState = true;
			break;
		end
	end
	DataCenter.redPointData:updateRedPointState(Lua_RedPointList.GoFishing_Indent, indentRedState);
end

function Lua_GoFishingData.Get:fishingIndentList()
	return self._fishingIndentList or {};
end

function Lua_GoFishingData.Get:activeUp()
	return self._activeUp or 1;
end

function Lua_GoFishingData:fromServer50242007(msg)
	if msg == nil then
		return;
	end

	self._isAttach = msg.is_attach == 1; -- 是否咬钩 1是
	self._size = msg.size; -- 大鱼类型 1普通鱼 2大鱼 3极大鱼（播放不同的动画）
	self._isGet = msg.is_get == 1; -- 是否上钩 1是（判断是否断线）
	self._getFish = msg.fishing_reward; -- 渔获
	self:updateStaminaRedPointState();
end

function Lua_GoFishingData.Get:isAttach()
	return self._isAttach;
end

function Lua_GoFishingData.Get:size()
	return self._size or 1;
end

function Lua_GoFishingData.Get:isGet()
	return self._isGet;
end

function Lua_GoFishingData:fromServer50242016(msg)
	if msg == nil then
		return;
	end

	if self.aquariumId ~= msg.room_level then
		OperateAlert.showLangValue(794095);
	end
	self._aquariumId = msg.room_level; -- 水族馆等级
	table.sort( msg.room_fish, function(a, b) return a.index < b.index end );
	self._aquariumList = msg.room_fish;
end

function Lua_GoFishingData.Get:aquariumList()
	return self._aquariumList or {};
end

function Lua_GoFishingData:fromServer50242018(msg)
	if msg == nil then
		return;
	end
	if self.fisheryDic == nil then
		self.fisheryDic = {};
	end
	self.fisheryDic[msg.fishery] = msg;

	local fisheryArr = DataCenter.goFishingUtil.fisheryArr;
	local fisheryRedState = false;
	for i = 1, fisheryArr:getSize() do
		fisheryInfo = self.fisheryDic[fisheryArr[i]];
		if fisheryInfo and fisheryInfo.is_unlock ~= 1 then
			local num = 0;
			for i = 1, #fisheryInfo.task_state do
				obj = fisheryInfo.task_state[i];
				taskInfo = DataCenter.goFishingUtil:getFishTaskInfo(fisheryInfo.task_state[i].key);
				if obj.value >= taskInfo.process then
					num = num + 1;
				end
			end
			fisheryRedState = num >= #fisheryInfo.task_state and #fisheryInfo.task_state > 0;
			if fisheryRedState then
				break;
			end
		end
	end
	DataCenter.redPointData:updateRedPointState(Lua_RedPointList.GoFishing_Fishery, fisheryRedState);
end

function Lua_GoFishingData:getTheIndexByFisheryId(fisheryId)
	local fisheryArr = DataCenter.goFishingUtil.fisheryArr;
	local idx = 1;
	for i = 1, fisheryArr:getSize() do
		if fisheryId == fisheryArr[i] then
			idx = i;
			break;
		end
	end
	return idx;
end

function Lua_GoFishingData:fromServer50242019(msg)
	if msg == nil then
		return;
	end
	self._fisheryId = msg.fishery; -- 渔场id
end

function Lua_GoFishingData:fromServer50242020(msg)
	if msg == nil then
		return;
	end
	if self.fishingAllAtlasDic == nil then
		self.fishingAllAtlasDic = {}; -- 类型 = 渔场/藏品
		self.fishingAtlasDic = {}; -- 图鉴id = 对应图鉴信息
		self.fishingNewAtlasDic = {}; -- 类型 = 是否有新获得
	end
	self.fishingAllAtlasDic[msg.book_type] = msg.book_list;

	local obj;
	for i = 1, #msg.book_list do
		obj = msg.book_list[i];
		self.fishingAtlasDic[obj.book_id] = obj;
		self.fishingNewAtlasDic[obj.book_id] = false;
		for j = 1, #obj.book_list do
			if not self.fishingNewAtlasDic[msg.book_type] and obj.book_list[j].other > 0 and obj.book_list[j].value == 0 then
				self.fishingNewAtlasDic[obj.book_id] = true;
				break;
			end
		end
	end
	self.isRefreshAtlas = true;
	self:updateCollectionUpLvRedPointState();
	self:updateNewAtlasRedPointState();
end

function Lua_GoFishingData:getTheFishAtlasInfoByType(bookType)
	if bookType == nil or self.fishingAllAtlasDic == nil or self.fishingAllAtlasDic[bookType] == nil then
		return nil;
	end
	return self.fishingAllAtlasDic[bookType];
end

function Lua_GoFishingData:getTheFishAtlasInfoById(atlasId)
	if atlasId == nil or self.fishingAtlasDic == nil or self.fishingAtlasDic[atlasId] == nil then
		return nil;
	end
	return self.fishingAtlasDic[atlasId];
end

-- 新获得的鱼
function Lua_GoFishingData:getTheNewFishAtlasInfoByType(bookType)
	if bookType == nil or self.fishingNewAtlasDic == nil or self.fishingNewAtlasDic[bookType] == nil then
		return false;
	end
	return self.fishingNewAtlasDic[bookType];
end

function Lua_GoFishingData:getTheFishAtlasUnLockNumByType(bookType)
	if self.isRefreshAtlas or self.unlockAtlasNumDic == nil then
		self.isRefreshAtlas = false;
		self.unlockAtlasNumDic = {};

		local fishAtlasArr = DataCenter.goFishingUtil.fishAtlasArr;
		local atlasIdArr, isUnLock;
		for i = 1, fishAtlasArr:getSize() do -- 类型：渔场/藏品
			self.unlockAtlasNumDic[fishAtlasArr[i]] = 0;
			atlasIdArr = DataCenter.goFishingUtil.fishAtlasDic[fishAtlasArr[i]];
			for j = 1, atlasIdArr:getSize() do -- 对应类型下的图鉴id
				atlasVo = self:getTheFishAtlasInfoById(atlasIdArr[j]);
				isUnLock = true;
				if atlasVo then
					for k = 1, #atlasVo.book_list do -- 对应id下的物品是否获得
						if atlasVo.book_list[k].other <= 0 then
							isUnLock = false;
							break;
						end
					end
				end
				if isUnLock then
					self.unlockAtlasNumDic[fishAtlasArr[i]] = self.unlockAtlasNumDic[fishAtlasArr[i]] + 1;
				end
			end
		end
	end
	return self.unlockAtlasNumDic[bookType] or 0;
end

function Lua_GoFishingData:fromServer80242001(msg)
	if msg == nil then
		return;
	end

	self._endTick = msg.close_tick; -- 结束时间戳
	self._firstPlayer = msg.show_role; -- 榜一玩家
	self._firstFish = msg.fish; -- 榜一大鱼
	self:updateStaminaRedPointState();

	self.isPopupEnter = true;
	self:checkPopupInfo();
end

function Lua_GoFishingData.Get:firstPlayer()
	return self._firstPlayer or {};
end

function Lua_GoFishingData.Get:firstFish()
	return self._firstFish or {};
end

function Lua_GoFishingData:checkPopupInfo()
	if Lua_WindowUtil.isWindowShowing("Lua_ExperienceWindow") and self.isPopupEnter then
		Lua_WindowUtil.openWin("Lua_GoFishingEntranceWindow", Lua_WinParam.ACTION_OPEN);
		self.isPopupEnter = false;
	end
end

function Lua_GoFishingData:getTheFishAttrValue(propId)
	if self.isNeedRefreshAttrInfo or self.fishAttrValueDic == nil then
		self.isNeedRefreshAttrInfo = false;
		self.fishAttrValueDic = {};

		local fishGearInfo, config;
		if self.fishingGearDic then
			for i = 1, 4 do
				fishGearInfo = self:getTheFishGearInfo(i);
				if fishGearInfo then
					config = DataCenter.goFishingUtil:getTheFishingGearInfo(fishGearInfo.gear_id);
					self.fishAttrValueDic = DataCenter.attributesUtil:getAttrDic(config.base_prop);
					self.fishAttrValueDic = DataCenter.attributesUtil:getAttrDic(fishGearInfo.prop, self.fishAttrValueDic);
				end
			end
		end
		if self.privilegeTime > 0 then
			local vipProp = DataCenter.goFishingUtil:getGoFishingPrivilegeConfig("vip_prop", 4);
			self.fishAttrValueDic = DataCenter.attributesUtil:getAttrDic(vipProp, self.fishAttrValueDic);
		end
	end
	return self.fishAttrValueDic[propId] or 0;
end

function Lua_GoFishingData:getTheFishGearRedInfo(gearType)
	local gearInfo = self:getTheFishGearInfo(gearType);
	if gearInfo == nil then
		return false;
	end

	local gearConfig = DataCenter.goFishingUtil:getTheFishingGearInfo(gearInfo.gear_id);
	local list = gearConfig.up_cost;
	local gearUpLvRed = false; -- 渔具升级
	local gearUpgradeRed = false; -- 渔具进阶
	if list then
		local num = 0;
		for i = 1, #list do
			hasCount = DataCenter.packData:getItemCountById(list[i][1]);
			if hasCount >= list[i][2] then
				num = num + 1;
			end
		end
		gearUpLvRed = num >= #list;
	else
		local taskInfo;
		local taskCount = #gearInfo.task_state;
		local num = 0;
		for i = 1, #gearInfo.task_state do
			taskInfo = DataCenter.goFishingUtil:getFishTaskInfo(gearInfo.task_state[i].key);
			if gearUpgradeRed and gearInfo.task_state[i].value >= taskInfo.process then
				num = num + 1;
			end
		end
		gearUpgradeRed = num >= #gearInfo.task_state;
	end
	return gearUpLvRed or gearUpgradeRed;
end

-- 钓鱼体力红点
function Lua_GoFishingData:updateStaminaRedPointState(dic)
	if dic then
		local isRefresh = false;
		for k,_ in pairs(dic) do
			if k == Lua_ResourcesDefine.GOFISHING_STAMINA then
				isRefresh = true;
				break;
			end
		end
		if not isRefresh then
			return;
		end
	end

	if DataCenter.funcOpenData:isOpened(10320) then
		local hasCount = DataCenter.packData:getItemCountById(Lua_ResourcesDefine.GOFISHING_STAMINA);
		local isCanGoFishing = (self.getFish.fishing == nil or self.getFish.fishing == 0) and hasCount > 0 and self.gofishingOpenTime > 0;
		DataCenter.redPointData:updateRedPointState(Lua_RedPointList.GoFishing_CanFishClick, isCanGoFishing);
	end
end

-- 钓鱼水族馆红点
function Lua_GoFishingData:updateAquariumRedPointState(dic)
	if DataCenter.funcOpenData:isOpened(10320) then
		local aquariumRed = false;
		if self.aquariumInfo.up_cost ~= nil then
			local list = self.aquariumInfo.up_cost;
			local num = 0;
			for i = 1, #list do
				hasCount = DataCenter.packData:getItemCountById(list[i][1]);
				if dic and dic[list[i][1]] == nil then
					return;
				end
				if hasCount >= list[i][2] then
					num = num + 1;
				end
			end
			aquariumRed = num >= #list;
		end
		DataCenter.redPointData:updateRedPointState(Lua_RedPointList.GoFishing_AquariumUpLv, aquariumRed);
	end
end

-- 钓鱼藏品升级红点
function Lua_GoFishingData:updateCollectionUpLvRedPointState(dic)
	if DataCenter.funcOpenData:isOpened(10320) then
		if dic then
			local isRefresh = false;
			for k,_ in pairs(dic) do
				local sub_type = getConfig("item", k, "sub_type");
				if sub_type == Lua_PackDefine.GOFISHING_COLLECTION then
					isRefresh = true;
					break;
				end
			end
			if not isRefresh then
				return;
			end
		end

		local upLvCostArr = DataCenter.goFishingData.upLvCostArr;
		local exp = 0;
		for i = 1, upLvCostArr:getSize() do
			exp = exp + upLvCostArr[i].exp * upLvCostArr[i].count;
		end

		local collectionRed = false;
		local collectionList = self:getTheFishAtlasInfoByType(0);
		if collectionList then
			for i = 1, #collectionList do
				local atlasUpLvId = DataCenter.goFishingUtil:getFishAtlasUpLvId(collectionList[i].book_id, collectionList[i].level);
				local config = DataCenter.goFishingUtil:getFishAtlasUpLvInfo(atlasUpLvId);
				if config.next_id and config.need_exp - collectionList[i].exp <= exp then
					collectionRed = true;
					break;
				end
			end
		end
		DataCenter.redPointData:updateRedPointState(Lua_RedPointList.GoFishing_Atlas_Collection, collectionRed and not self.isFirstOpenFishCollection);
	end
end

-- 钓鱼解锁新图鉴红点
function Lua_GoFishingData:updateNewAtlasRedPointState()
	local fishAtlasArr = DataCenter.goFishingUtil.fishAtlasArr;
	local arr;
	local newAtlasRed = false;
	for i = 1, fishAtlasArr:getSize() do
		arr = DataCenter.goFishingUtil.fishAtlasDic[fishAtlasArr[i]];
		for j = 1, arr:getSize() do
			if DataCenter.goFishingData:getTheNewFishAtlasInfoByType(arr[j]) then
				newAtlasRed = true;
				break;
			end
		end
	end
	DataCenter.redPointData:updateRedPointState(Lua_RedPointList.GoFishing_Atlas_NewAtlas, newAtlasRed);
end




















------------------------------ 藏品升级数据 ------------------------------
function Lua_GoFishingData.Get:upLvCostArr()
	local uidArr = DataCenter.packData:getUnIdList(Lua_PackDefine.GOFISHING, Lua_PackDefine.GOFISHING_COLLECTION);
	local tvo, len, itemNum, count;
	local arr = Array();
	local limitNumt = DataCenter.goFishingUtil.showMaxNum;
	for i = 1, uidArr:getSize() do
		tvo = DataCenter.packData:getItemVoByUnId(uidArr[i]);
		if tvo ~= nil then
			itemNum = DataCenter.packData:getItemCountById(tvo.id);
			len = Mathf.Ceil(itemNum / limitNumt);
			for j = 1, len do
				if j < len then
					count = limitNumt;
				else
					count = itemNum - limitNumt * (j - 1);
				end
				tvo = Lua_ToolVo.createToolVo(tvo.id, count);
				tvo.tagUid = tvo.id.."00000000000"..j;
				arr:add(tvo);
			end
		end
	end
	return arr;
end

-- 一键选择藏品材料
function Lua_GoFishingData:getTheCostInfo(needExp, showCostArr, selectNumDic)
	local tvo;
	local provideExp; -- 可提供的经验
	local canSelectNum; -- 可以选择的数量
	for i = 1, showCostArr:getSize() do
		tvo = showCostArr[i];
		provideExp = tvo.exp;
		if selectNumDic[tvo.tagUid] == nil then -- 当前材料未选择
			selectNumDic[tvo.tagUid] = 0;
			canSelectNum = tvo.count;
		else
			canSelectNum = tvo.count - selectNumDic[tvo.tagUid]; -- 已选择的材料需要筛选掉
		end

		local bBreak = false;
		for i = 1, canSelectNum do
			if needExp > 0 then
				needExp = needExp - provideExp;
				selectNumDic[tvo.tagUid] = selectNumDic[tvo.tagUid] + 1;
			else
				bBreak = true;
				break;
			end
		end
		if bBreak then
			break;
		end
	end
	return selectNumDic;
end

-- 当前材料提供多少经验
function Lua_GoFishingData:getExpByCostDic(selectDic)
	local exp;
	local num = 0;
	for itemId, count in pairs(selectDic) do
		if string.len(itemId) >= 20 then
			itemId = string.sub(itemId, 1, 9);
		end
		exp = getConfig("item", tonumber(itemId), "exp");
		if exp ~= nil then
			num = num + exp * count;
		end
	end
	return num;
end

-- 选择的经验可以升到几级和剩余的经验 
function Lua_GoFishingData:getTheEndLevelAndLeftExp(hasSelectExp, atlasUpLvId, showSelectExp)
	local bUpLevel = true; -- 是否可以升级
	local needExp = self:getTheNextExpByAtlasUpLvId(atlasUpLvId) - hasSelectExp;
	local config;
	while bUpLevel do
		if needExp > 0 and showSelectExp - needExp >= 0 then -- 够升一级
			bUpLevel = true;
			config = DataCenter.goFishingUtil:getFishAtlasUpLvInfo(atlasUpLvId);
			if config.next_id then
				showSelectExp = showSelectExp - needExp;
				atlasUpLvId = config.next_id;
			else
				bUpLevel = false;
			end
			needExp = self:getTheNextExpByAtlasUpLvId(config.next_id);
		else
			bUpLevel = false;
		end
	end
	return showSelectExp, atlasUpLvId;
end

function Lua_GoFishingData:getTheNextExpByAtlasUpLvId(atlasUpLvId)
	if atlasUpLvId == nil then
		return 0;
	end
	local config = DataCenter.goFishingUtil:getFishAtlasUpLvInfo(atlasUpLvId);
	if config.next_id then
		return config.need_exp;
	else
		return 0;
	end
end