checkRequire("Lua_GodSeaDanVo");
Lua_GodSeaPeakData = class("Lua_GodSeaPeakData")
function Lua_GodSeaPeakData:ctor()
	self:superCall("ctor");
	self.danInfo = {};
	self.weekReward = {};
	self:dispose();
end

function Lua_GodSeaPeakData:dispose()
	self._clearing_time = nil;
	self._start_time = nil;
	self._dan_id = nil;
	self._challenge_time = nil;
	self._buy_num = nil;
	self._current_ranking = nil;
	self._win_times = nil;
	self._is_skip = nil;
	if self._dan_info then
		for i = 1, #self._dan_info do
			id = self._dan_info[i].dan_id;
			if self.danInfo[id] ~= nil then
				self.danInfo[id]:fromServer(nil);
			end
		end
		self._dan_info = nil;
	end
	table.clear(self.weekReward);
	self.hideLineupList = nil;
	self.leftLikeTimes = 0;
end

function Lua_GodSeaPeakData.Get:isInTime()
	return DataCenter.roleData:checkInTime(self.startTick, self.endTick);
end

-- 开始时间戳
function Lua_GodSeaPeakData.Get:startTick()
	return self._start_time or 0;
end

-- 结算时间戳
function Lua_GodSeaPeakData.Get:endTick()
	return self._clearing_time or 0;
end

-- 0为赛区正常，1为休赛期
function Lua_GodSeaPeakData.Get:acquiredState()
	return self._acquired_state or 0;
end

-- 积分
function Lua_GodSeaPeakData.Get:integral()
	return self._integral or 0;
end

-- 当前段位
function Lua_GodSeaPeakData.Get:curDanId()
	if self._dan_id == nil then
		local arr = DataCenter.godSeaPeakUtil.danList;
		local len = arr:getSize();
		return arr[len];
	end
	return self._dan_id;
end

-- 剩余挑战次数
function Lua_GodSeaPeakData.Get:leftFightTimes()
	return self._challenge_time or 0;
end

-- 剩余挑战次数
function Lua_GodSeaPeakData.Get:leftBuyTimes()
	return DataCenter.godSeaPeakUtil.buyLimitTimes - self.buyTimes;
end

-- 挑战次数购买数量
function Lua_GodSeaPeakData.Get:buyTimes()
	return self._buy_num or 0;
end

-- 当前排名
function Lua_GodSeaPeakData.Get:curRank()
	return self._current_ranking or 0;
end

-- 胜利次数
function Lua_GodSeaPeakData.Get:winTimes()
	return self._win_times or 0;
end

-- 是否跳过战斗
function Lua_GodSeaPeakData.Get:isSkip()
	return self._is_skip == 1;
end

-- 段位信息
function Lua_GodSeaPeakData:getDanVoById(id)
	if self.danInfo[id] == nil then
		local config = getConfig("god_sea_dan", id);
		if config == nil then
			return;
		end
		self.danInfo[id] = Lua_GodSeaDanVo.new(id);
	end
	return self.danInfo[id];
end

-- 周奖励状态
function Lua_GodSeaPeakData:getWeekRewardState(id)
	return self.weekReward[id] or 0;
end

function Lua_GodSeaPeakData:fromServer50240001(msg)
	if msg == nil then
		return;
	end

	self._clearing_time = tostring(msg.clearing_time);
	self._start_time = tostring(msg.start_time);
	self._acquired_state = msg.acquired_state;
	self._integral = msg.integral;
	self._dan_id = msg.dan_id;
	self._challenge_time = msg.challenge_time;
	self._buy_num = msg.buy_num;
	self._current_ranking = msg.current_ranking;
	self._win_times = msg.win_time;
	self._dan_info = msg.dan_info;
	self._is_skip = msg.is_skip;

	if self.isInTime then
		DataCenter.redPointData:updateRedPointState(Lua_RedPointList.GodSeaPeak_Fight, self.leftFightTimes > 0);
	else
		DataCenter.redPointData:updateRedPointState(Lua_RedPointList.GodSeaPeak_Fight, false);
	end

	local info;
	local vo;
	local state = false;
	for i = 1, #msg.dan_info do
		info = msg.dan_info[i];
		vo = self:getDanVoById(info.dan_id);
		if vo ~= nil then
			vo:fromServer(info);
		end
		if info.reward_state == 1 then
			state = true;
		end
	end
	DataCenter.redPointData:updateRedPointState(Lua_RedPointList.GodSeaPeak_DanReward, state);

	state = false;
	for i = 1, #msg.week_award do
		info = msg.week_award[i];
		self.weekReward[info.key] = info.value;
		if info.value == 1 then
			state = true;
		end
	end
	DataCenter.redPointData:updateRedPointState(Lua_RedPointList.GodSeaPeak_Box, state);

	self.hideLineupList = msg.team_indexes;
end

function Lua_GodSeaPeakData:fromServer50240002(msg)
	if msg == nil then
		return;
	end

	local state = false;
	for i = 1, #msg.dan_award do
		vo = self:getDanVoById(msg.dan_award[i].key);
		vo.rewardState = msg.dan_award[i].value;
		if msg.dan_award[i].value == 1 then
			state = true;
		end
	end
	DataCenter.redPointData:updateRedPointState(Lua_RedPointList.GodSeaPeak_DanReward, state);
end

function Lua_GodSeaPeakData:fromServer50240003(msg)
	if msg == nil then
		return;
	end

	local state = false;
	for i = 1, #msg.task_award do
		if msg.task_award[i].ex_value == 1 then
			state = true;
			break;
		end
	end
	DataCenter.redPointData:updateRedPointState(Lua_RedPointList.GodSeaPeak_Task, state);
end

function Lua_GodSeaPeakData:fromServer50240009(msg)
	if msg == nil then
		return;
	end
	self._challenge_time = msg.challenge_time;
	self._buy_num = msg.buy_num;
end

function Lua_GodSeaPeakData:fromServer50240013(msg)
	if msg == nil then
		return;
	end
	self.hideLineupList = msg.indexes;
end

function Lua_GodSeaPeakData:fromServer50240016(msg)
	if msg == nil then
		return;
	end

	local receivedDic = {};
	for i,v in ipairs(msg.reward_list) do
		receivedDic[v] = 1;
	end	

	local arr = DataCenter.godSeaPeakUtil:getDrawRewardByLv(msg.level_of_luck);
	local config;
	local state = false;
	for i = 1, arr:getSize() do
		config = DataCenter.godSeaPeakUtil:getRewardConfig(arr[i]);
		if msg.luck_point >= config.luck_point and receivedDic[config.id] ~= 1 then
			state = true;
			break;
		end
	end
	DataCenter.redPointData:updateRedPointState(Lua_RedPointList.GodSeaPeak_Treasure_Reward, state);
end

function Lua_GodSeaPeakData:fromServer50240022(msg)
	if msg == nil then
		return;
	end
	self._is_skip = msg.type;
end

-- 上次积分
function Lua_GodSeaPeakData.Get:oldScore()
	return self._old_score or 0;
end

-- 最新积分
function Lua_GodSeaPeakData.Get:newScore()
	return self._new_score or 0;
end

-- 上次段位
function Lua_GodSeaPeakData.Get:oldDan()
	return self._old_dan or 0;
end

-- 最新积分
function Lua_GodSeaPeakData.Get:newDan()
	return self._new_dan or 0;
end

function Lua_GodSeaPeakData:fromServer50240023(msg)
	if msg == nil then
		return;
	end
	print(msg.old_score, msg.new_score, msg.old_dan, msg.new_dan)
	self._old_score = msg.old_score; -- 上次积分
	self._new_score = msg.new_score; -- 最新积分
	self._old_dan = msg.old_dan; -- 上次段位
	self._new_dan = msg.new_dan; -- 最新段位
end

function Lua_GodSeaPeakData:checkTreasureDrawRed()
	if not DataCenter.funcOpenData:isOpened(10316) then
		return;
	end

	local cost = getConfig("god_sea_const", "once_item_gift", "json_value1", 1);
	local hasCount = DataCenter.packData:getItemCountById(cost[1]);
	local bValue = hasCount >= cost[2];
	if not bValue then
		cost = getConfig("god_sea_const", "once_gift", "json_value1", 1);
		hasCount = DataCenter.packData:getItemCountById(cost[1]);
		bValue = hasCount >= cost[2];
	end
	DataCenter.redPointData:updateRedPointState(Lua_RedPointList.GodSeaPeak_Treasure_Draw, bValue);
end