Lua_HeroLineupVo = class("Lua_HeroLineupVo")
function Lua_HeroLineupVo:ctor(id, config)
    self._id = id -- 阵容类型
    self._config = config

    self.detachmentDic = {}
    self.heroNumDic = {} -- [lineupIdx] = num
    self.heroStandDic = {} -- [uid] = standIdx
    self.heroTeamDic = {} -- [id] = lineupIdx * 10000 + standIdx
    self.standNum = 10 -- 站位数量
    self.artifactNum = 3 -- 神器数量
    self.beastNum = 3; -- 异兽数量
    self.connectTab = {} -- 激活的连携技能
    self.beastTeamDic = {}; --异兽所在的队伍
end

function Lua_HeroLineupVo.Get:id()
    return self._id
end

function Lua_HeroLineupVo.Get:config()
    if self._config == nil then
        self._config = getConfig("hero_camp_lineup", self.id)
    end
    return self._config
end

function Lua_HeroLineupVo:fromServer(msg)
    -- self.detachment_list = msg.detachment_list
    if self.detachmentDic then
        table.clear(self.detachmentDic)
    end
    self:initHeroInfo()
    local info
    for i = 1, #msg.detachment_list do
        info = msg.detachment_list[i]
        if info.index <= self.team_number then
            self.detachmentDic[info.index] = info
            self:setStandType(info.index, info.stand_type)
            self:setHero(info.index, info.location_list)
            self:setArtifact(info.index, info.artifact_list)
            self:setBeast(info.index, info.strange_beasts_list)
        end
    end
end

function Lua_HeroLineupVo:getDetachmentInfo(index)
    return self.detachmentDic[index]
end

function Lua_HeroLineupVo:checkLineupIsChange(lineupIdx)
    local info = self:getDetachmentInfo(lineupIdx)
    if info then
        local standType = self:getStandType(lineupIdx)
        if info.stand_type ~= standType then -- 站位类型
            return true
        end

        local list = info.location_list
        local key
        local value
        local arr = self:getHeroArr(lineupIdx)
        for i = 1, #list do
            key = tonumber(list[i].key)
            value = tonumber(list[i].value)
            if value ~= arr[key] then
                return true
            end
        end

        list = info.artifact_list
        arr = self:getArtifactArr(lineupIdx)
        for i = 1, #list do
            key = tonumber(list[i].key)
            value = tonumber(list[i].value)
            if value ~= arr[key] then
                return true
            end
        end

        list = info.strange_beasts_list
        arr = self:getBeastArr(lineupIdx)
        for i = 1, #list do
            key = tonumber(list[i].key)
            value = tonumber(list[i].value)
            if value ~= arr[key] then
                return true
            end
        end

        arr = self:getHeroArr(lineupIdx)
        list = info.location_list
        for i = 1, arr:getSize() do
            if arr[i] > 0 and arr[i] ~= self:getValueByList(list, i) then
                return true
            end
        end

        arr = self:getArtifactArr(lineupIdx)
        list = info.artifact_list
        for i = 1, arr:getSize() do
            if arr[i] > 0 and arr[i] ~= self:getValueByList(list, i) then
                return true
            end
        end

        arr = self:getBeastArr(lineupIdx)
        list = info.strange_beasts_list
        for i = 1, arr:getSize() do
            if arr[i] > 0 and arr[i] ~= self:getValueByList(list, i) then
                return true
            end
        end
    else
        local num = self:getHeroNumByLineupIdx(lineupIdx)
        if num <= 0 then
            local arr = self:getArtifactArr(lineupIdx)
            for i = 1, arr:getSize() do
                if arr[i] > 0 then
                    return true
                end
            end

            arr = self:getBeastArr(lineupIdx)
            for i = 1, arr:getSize() do
                if arr[i] > 0 then
                    return true
                end
            end
        end
        return num > 0
    end
    return false
end

function Lua_HeroLineupVo:getValueByList(list, key)
    for i = 1, #list do
        if tonumber(list[i].key) == key then
            return tonumber(list[i].value)
        end
    end
    return nil
end

-------------------- 本地数据 --------------------
-- [lineupIdx] = 站位类型
function Lua_HeroLineupVo.Get:standTypeDic()
    if self._standTypeDic == nil then
        self._standTypeDic = {}
        for i = 1, self.team_number do
            self._standTypeDic[i] = self.standType
        end
    end
    return self._standTypeDic
end

-- [lineupIdx] = 英雄唯一ID数组
function Lua_HeroLineupVo.Get:heroDic()
    if self._heroDic == nil then
        self._heroDic = {}
        for i = 1, self.team_number do
            self._heroDic[i] = Array()
            for j = 1, self.standNum do
                self._heroDic[i]:add(0)
            end
        end
    end
    return self._heroDic
end

-- [lineupIdx] = 神器ID数组
function Lua_HeroLineupVo.Get:artifactDic()
    if self._artifactDic == nil then
        self._artifactDic = {}
        for i = 1, self.team_number do
            self._artifactDic[i] = Array()
            for j = 1, self.artifactNum do
                self._artifactDic[i]:add(0)
            end
        end
    end
    return self._artifactDic
end

-- [lineupIdx] = 异兽ID数组
function Lua_HeroLineupVo.Get:beastDic()
    if self._beastDic == nil then
        self._beastDic = {}
        for i = 1, self.team_number do
            self._beastDic[i] = Array()
            for j = 1, self.beastNum do
                self._beastDic[i]:add(0)
            end
        end
    end
    return self._beastDic
end

function Lua_HeroLineupVo:setStandType(lineupIdx, value)
    self.standTypeDic[lineupIdx] = value
end

function Lua_HeroLineupVo:setHero(lineupIdx, list)
    local standIdx
    local uniqueId
    local num = 0
    local heroVo
    for i = 1, #list do
        standIdx = tonumber(list[i].key)
        uniqueId = tonumber(list[i].value)
        heroVo = DataCenter.heroLineupData:getHeroVo(self.id, uniqueId)

        self.heroStandDic[uniqueId] = standIdx
        self.heroDic[lineupIdx][standIdx] = uniqueId
        num = num + 1
        if heroVo then
            self.heroTeamDic[heroVo.id] = lineupIdx * 10000 + standIdx
        end
    end
    self.heroNumDic[lineupIdx] = num
end

function Lua_HeroLineupVo:setHeroById(lineupIdx, index, uniqueId)
    local num = self.heroNumDic[lineupIdx]
    num = num or 0
    uniqueId = uniqueId or 0
    local heroVo
    if uniqueId > 0 then
        if self.heroDic[lineupIdx][index] ~= uniqueId then
            self.heroDic[lineupIdx][index] = uniqueId
            num = num + 1
        end
        self.heroStandDic[uniqueId] = index

        heroVo = DataCenter.heroLineupData:getHeroVo(self.id, uniqueId)
        if heroVo then
            self.heroTeamDic[heroVo.id] = lineupIdx * 10000 + index
        end
    else
        uniqueId = self.heroDic[lineupIdx][index]
        if uniqueId > 0 then
            num = num - 1
            heroVo = DataCenter.heroLineupData:getHeroVo(self.id, uniqueId)
            if heroVo then
                self.heroTeamDic[heroVo.id] = nil
            end
            self.heroStandDic[uniqueId] = nil
        end
        self.heroDic[lineupIdx][index] = 0
    end
    self.heroNumDic[lineupIdx] = num
end

function Lua_HeroLineupVo:setArtifact(lineupIdx, list)
    local obj
    for i = 1, #list do
        obj = list[i]
        self.artifactDic[lineupIdx][obj.key] = obj.value
    end
end

function Lua_HeroLineupVo:setArtifactById(lineupIdx, index, id)
    id = id or 0
    self.artifactDic[lineupIdx][index] = id
end

function Lua_HeroLineupVo:setBeast(lineupIdx, list)
    local obj
    for i = 1, #list do
        obj = list[i]
        self.beastDic[lineupIdx][obj.key] = obj.value
        self.beastTeamDic[obj.value] = lineupIdx;
    end
end

function Lua_HeroLineupVo:setBeastById(lineupIdx, index, id)
    id = id or 0
    self.beastDic[lineupIdx][index] = id
end

function Lua_HeroLineupVo:setBeastTeamById(id, lineupIdx)
    self.beastTeamDic[id] = lineupIdx;
end

function Lua_HeroLineupVo:getBeastTeamById(id)
    return self.beastTeamDic[id];
end

-------------------- 布阵调用 --------------------

function Lua_HeroLineupVo:initHeroInfo()
    for _, v in pairs(self.heroDic) do
        for i = 1, v:getSize() do
            v[i] = 0
        end
    end
    for _, v in pairs(self.artifactDic) do
        for i = 1, v:getSize() do
            v[i] = 0
        end
    end
    for _, v in pairs(self.beastDic) do
        for i = 1, v:getSize() do
            v[i] = 0
        end
    end
    if self.heroNumDic then
        table.clear(self.heroNumDic)
    end
    if self.heroStandDic then
        table.clear(self.heroStandDic)
    end
    if self.heroTeamDic then
        table.clear(self.heroTeamDic)
    end
    if self.beastTeamDic then
        table.clear(self.beastTeamDic);
    end
end

function Lua_HeroLineupVo:initLocalInfo()
    self.bInitLineUp = true
    self:initHeroInfo()
    local info
    for i = 1, self.team_number do
        info = self:getDetachmentInfo(i)
        if info then
            self:setStandType(info.index, info.stand_type)
            self:setHero(info.index, info.location_list)
            self:setArtifact(info.index, info.artifact_list)
            self:setBeast(info.index, info.strange_beasts_list)
        end
    end
end

function Lua_HeroLineupVo:initLineUpInfo()
    if self.heroNumDic then
        table.clear(self.heroNumDic)
    end
    if self.heroStandDic then
        table.clear(self.heroStandDic)
    end
    if self.heroTeamDic then
        table.clear(self.heroTeamDic)
    end
    local dic = self.heroDic
    local uniqueId
    local num
    local heroVo
    for i = 1, #dic do
        num = 0
        for j = 1, dic[i]:getSize() do
            uniqueId = dic[i][j]
            if uniqueId > 0 then
                heroVo = DataCenter.heroLineupData:getHeroVo(self.id, uniqueId)
                if heroVo then
                    num = num + 1
                    self.heroStandDic[heroVo.unique_id] = j
                    self.heroTeamDic[heroVo.id] = i * 10000 + j
                end
            end
        end
        self.heroNumDic[i] = num
    end
end

-- function Lua_HeroLineupVo:checkHeroConnect(lineupIdx, bPopupWin, nCheckConnect)
--     if self.bCheckConnect == true and nCheckConnect ~= true then
--         return
--     end
--     self.bCheckConnect = true

--     local arr = self.heroDic[lineupIdx]
--     local heroDic = {}
--     local campDic = {}
--     local careerDic = {}
--     local heroVo
--     local len = arr:getSize()
--     for i = 1, len do
--         if arr[i] > 0 then
--             heroVo = DataCenter.heroLineupData:getHeroVo(self.id, arr[i])
--             -- print(heroVo," uid = = ",arr[i])
--             heroDic[heroVo.id] = heroVo.id
--             if campDic[heroVo.campType] == nil then
--                 campDic[heroVo.campType] = Array()
--             end
--             campDic[heroVo.campType]:add(heroVo.id)

--             if careerDic[heroVo.career] == nil then
--                 careerDic[heroVo.career] = Array()
--             end
--             careerDic[heroVo.career]:add(heroVo.id)
--         end
--     end

--     if self.connectTab[lineupIdx] == nil then
--         self.connectTab[lineupIdx] = {}
--     end
--     local tab = self.connectTab[lineupIdx]
--     local activateDic = {}
--     local connVo
--     for i = 1, len do
--         if arr[i] > 0 then
--             heroVo = DataCenter.heroLineupData:getHeroVo(self.id, arr[i])
--             if heroVo and heroVo.connectId > 0 then
--                 connVo = tab[heroVo.connectId]
--                 if connVo == nil then
--                     connVo = Lua_HeroConnectVo.new(heroVo.connectId)
--                 else
--                     connVo:init()
--                 end
--                 connVo:setData(heroDic, campDic, careerDic)
--                 if connVo.isActivate then
--                     activateDic[heroVo.connectId] = connVo
--                     if bPopupWin and tab[heroVo.connectId] == nil then
--                         DataCenter.heroLineupData:addConnPopupWin(connVo)
--                     end
--                 end
--             end
--         end
--     end

--     for k, _ in pairs(tab) do
--         if activateDic[k] == nil then
--             tab[k] = nil
--         end
--     end
--     for k, v in pairs(activateDic) do
--         tab[k] = v
--     end
--     activateDic = nil
--     heroDic = nil
--     table.clear(campDic)
--     campDic = nil
--     table.clear(careerDic)
--     careerDic = nil
-- end

-- function Lua_HeroLineupVo:checkConnectIsActivate(lineupIdx, connectId)
--     self:checkHeroConnect(lineupIdx, false, true)
--     return self.connectTab[lineupIdx][connectId]
-- end

-- function Lua_HeroLineupVo:getConnectActivateNum(lineupIdx)
--     if self.connectTab == nil then
--         return 0
--     end
--     return table.length(self.connectTab[lineupIdx])
-- end

function Lua_HeroLineupVo:getHeroNumByLineupIdx(lineupIdx)
    if self.heroNumDic == nil then
        return 0
    end
    return self.heroNumDic[lineupIdx] or 0
end

-- 获得英雄所在队伍
function Lua_HeroLineupVo:getHeroLineupIdx(heroId)
    if self.heroTeamDic == nil or self.heroTeamDic[heroId] == nil then
        return nil
    end
    return Mathf.Floor(self.heroTeamDic[heroId] / 10000)
end

-- 获得英雄所在站位
function Lua_HeroLineupVo:getHeroStandIdx(hero_uid)
    if self.heroStandDic == nil then
        return nil
    end
    return self.heroStandDic[hero_uid]
end

-- 获得英雄所在分队索引、站位索引
function Lua_HeroLineupVo:getHeroStandIdxById(heroId)
    if self.heroTeamDic == nil or self.heroTeamDic[heroId] == nil then
        return nil, nil
    end
    local lineupIdx = Mathf.Floor(self.heroTeamDic[heroId] / 10000);
    local standIdx = Mathf.Floor(self.heroTeamDic[heroId] % 10000);
    return lineupIdx, standIdx
end

-- 获得站位索引上的英雄唯一ID
function Lua_HeroLineupVo:getHeroUidByStandIdx(lineupIdx, standIdx)
    if standIdx then
        lineupIdx = lineupIdx or 1
        return self.heroDic[lineupIdx][standIdx]
    end
    return nil
end

function Lua_HeroLineupVo:getHeroVoById(heroId)
    local lineupIdx, standIdx = self:getHeroStandIdxById(heroId);
    local uniqueId = self:getHeroUidByStandIdx(lineupIdx, standIdx);
    return DataCenter.heroLineupData:getHeroVo(self.id, uniqueId);
end

function Lua_HeroLineupVo:sortAndSwapByForce()
    if self.team_number <= 1 then
        return;
    end

    local len = self.team_number;
    local a;
    local b;
    local temp;
    for i = 1, len do
        for j = i + 1, len do
            a = DataCenter.heroLineupData:calcLineupForce(self.id, i);
            b = DataCenter.heroLineupData:calcLineupForce(self.id, j);
            if a < b then
                temp = self.standTypeDic[i];
                self.standTypeDic[i] = self.standTypeDic[j];
                self.standTypeDic[j] = temp;

                temp = self.heroDic[i];
                self.heroDic[i] = self.heroDic[j];
                self.heroDic[j] = temp;

                temp = self.artifactDic[i];
                self.artifactDic[i] = self.artifactDic[j];
                self.artifactDic[j] = temp;

                temp = self.beastDic[i];
                self.beastDic[i] = self.beastDic[j];
                self.beastDic[j] = temp;
            end
        end
    end
end

-------------------- 外部调用 --------------------

-- 获取布阵站位类型
function Lua_HeroLineupVo:getStandType(lineupIdx)
    if self.standTypeDic[lineupIdx] == nil then
        return self.standType
    end
    return self.standTypeDic[lineupIdx]
end

-- 获取上阵英雄列表
function Lua_HeroLineupVo:getHeroArr(lineupIdx)
    if self.heroDic[lineupIdx] == nil then
        return Array()
    end
    return self.heroDic[lineupIdx]
end

-- 获取上阵神器列表
function Lua_HeroLineupVo:getArtifactArr(lineupIdx)
    if self.artifactDic[lineupIdx] == nil then
        return Array()
    end
    return self.artifactDic[lineupIdx]
end

-- 获取上阵神器
function Lua_HeroLineupVo:getArtifactId(lineupIdx, index)
    if self.artifactDic[lineupIdx] == nil then
        return 0
    end
    return self.artifactDic[lineupIdx][index] or 0
end

-- 获取上阵异兽列表
function Lua_HeroLineupVo:getBeastArr(lineupIdx)
    if self.beastDic[lineupIdx] == nil then
        return Array()
    end
    return self.beastDic[lineupIdx]
end

-- 获取上阵异兽
function Lua_HeroLineupVo:getBeastId(lineupIdx, index)
    if self.beastDic[lineupIdx] == nil then
        return 0
    end
    return self.beastDic[lineupIdx][index] or 0
end

function Lua_HeroLineupVo:getLineupLimitNum(lineupIdx)
    if self.limitNumDic == nil then
        self.limitNumDic = {};
        local list = self.config.lineup_number;
        if list then
            for i,v in ipairs(list) do
                self.limitNumDic[i] = v;
            end
        end
    end
    local num = self.limitNumDic[lineupIdx];
    if num == nil then
        local standType = self:getStandType(lineupIdx);
        local list = DataCenter.heroLineupUtil:getStandListByType(standType);
        return #list;
    end
    return num;
end

-------------------- 配置数据 --------------------
function Lua_HeroLineupVo.Get:name()
    return self.config.name;
end

function Lua_HeroLineupVo.Get:sort()
    return self.config.sort or 0;
end

-- 允许全部下阵
function Lua_HeroLineupVo.Get:isCanNil()
    return self.config.can_nil == 1;
end

function Lua_HeroLineupVo.Get:team_number()
    return self.config.team_number or 1;
end

function Lua_HeroLineupVo.Get:standType()
    return self.config.stand_type or 1;
end

function Lua_HeroLineupVo.Get:campposId()
    return self.config.camppos_id or 0;
end

function Lua_HeroLineupVo.Get:bShowTrial()
    return self.config.bShowTrial == 1;
end

function Lua_HeroLineupVo.Get:templateId()
    return self.config.template_id;
end