checkRequire("Lua_HeroMaterialVo")
Lua_HeroMaterialData = class("Lua_HeroMaterialData")
function Lua_HeroMaterialData:ctor()
	self:superCall("ctor")
	self.materialVos = Array()
	self.materialVoDic = {}
end

function Lua_HeroMaterialData:dispose()
	self.curHeroUnId = nil
	self.selectedDic = nil
	self.downLineupDic = nil
	for k, vo in pairs(self.materialVoDic) do
		self.materialVos:add(vo)
		self.materialVoDic[k] = nil
	end
end

function Lua_HeroMaterialData:init(uniqueId)
	self.curHeroUnId = uniqueId
	self.selectedDic = {} -- 选中的英雄列表
end

function Lua_HeroMaterialData:createMaterialVo(index)
	local vo = self.materialVoDic[index];
	if vo == nil then
		local len = self.materialVos:getSize()
		if len > 0 then
			vo = self.materialVos:remove()
			vo.index = index
		else
			vo = Lua_HeroMaterialVo.new(index)
		end
		self.materialVoDic[index] = vo
	end
	return vo;
end

function Lua_HeroMaterialData:addHeroMaterial(index, heroUnId)
	if heroUnId == nil then
		return
	end
	if self.materialVoDic[index] then
		self.materialVoDic[index]:addHeroMaterial(heroUnId)
	end
	self.selectedDic[heroUnId] = 1
end

function Lua_HeroMaterialData:removeHeroMaterial(index, heroUnId)
	if heroUnId == nil then
		return
	end
	if self.materialVoDic[index] then
		self.materialVoDic[index]:removeHeroMaterial(heroUnId)
	end
	self.selectedDic[heroUnId] = nil
end

function Lua_HeroMaterialData:checkHeroIsSelected(uniqueId)
	return self.selectedDic[uniqueId] == 1
end

function Lua_HeroMaterialData:checkHeroIsSelectedByIdx(index, uniqueId)
	if uniqueId and self.materialVoDic[index] then
		return self.materialVoDic[index]:checkHeroIsSelected(uniqueId)
	end
	return false
end

function Lua_HeroMaterialData:getSelectedNum(index)
	if self.materialVoDic[index] then
		return self.materialVoDic[index].selectedNum
	end
	return 0
end

function Lua_HeroMaterialData:getHeroList(materialVo, bScreen)
	local arr = Array()
	local uidArr = DataCenter.heroData:getHeroUidArrByStar(materialVo.star)
	local vo
	for i = 1, uidArr:getSize() do
		if uidArr[i] ~= self.curHeroUnId then
			if self:checkHeroIsSelectedByIdx(materialVo.index, uidArr[i]) then
				arr:add(uidArr[i])
			else
				vo = DataCenter.heroData:getItemByUniqueId(uidArr[i]);
				local bValue = true;
				if bScreen then --需要条件筛选
					bValue = not vo.isUpLineup and not vo.lock_state and not vo.bcore;
				end
				if bValue then
					if materialVo.id > 0 then
						if vo.id == materialVo.id and vo.campType == materialVo.camp and vo.star == materialVo.star and not vo.is_league_transaction and not vo.trial_state then
							arr:add(uidArr[i])
						end
					elseif materialVo.camp > 0 then
						if vo.campType == materialVo.camp and vo.star == materialVo.star and not vo.is_league_transaction and not vo.trial_state then
							arr:add(uidArr[i])
						end
					elseif materialVo.star > 0 then
						if vo.star == materialVo.star and not vo.is_league_transaction then
							arr:add(uidArr[i])
						end
					end
				end
			end
		end
	end
	return arr
end

function Lua_HeroMaterialData:checkEnoughByVo(materialVo)
	local selectedNum = self:getSelectedNum(materialVo.index)
	local needNum = materialVo.num - selectedNum
	local arr = DataCenter.heroData:getHeroUidArrByStar(materialVo.star)
	for i = 1, arr:getSize() do
		vo = DataCenter.heroData:getItemByUniqueId(arr[i])
		if arr[i] ~= self.curHeroUnId and not self:checkHeroIsSelected(arr[i]) and not vo.isUpLineup and not vo.lock_state and not vo.is_league_transaction and not vo.trial_state then
			if materialVo.id > 0 then
				if vo.id == materialVo.id and vo.campType == materialVo.camp and vo.star == materialVo.star then
					needNum = needNum - 1
					if needNum <= 0 then
						return true
					end
				end
			elseif materialVo.camp > 0 then
				if vo.campType == materialVo.camp and vo.star == materialVo.star then
					needNum = needNum - 1
					if needNum <= 0 then
						return true
					end
				end
			elseif materialVo.star > 0 then
				if vo.star == materialVo.star then
					needNum = needNum - 1
					if needNum <= 0 then
						return true
					end
				end
			end
		end
	end
	return false
end

function Lua_HeroMaterialData:checkEnough(material)
	if material then
		for i = 1, #material do
			if self:getSelectedNum(i) < material[i][3] then
				return false
			end
		end
	end
	return true
end

-- 记录已解除上阵状态的英雄
function Lua_HeroMaterialData:updateLineupState()
	if self.selectedDic then
		for uid, state in pairs(self.selectedDic) do
			if state == 1 then
				self:setDownLineup(uid)
			end
		end
	end
end

function Lua_HeroMaterialData:setDownLineup(uniqueId)
	if self.downLineupDic == nil then
		self.downLineupDic = {};
	end
	self.downLineupDic[uniqueId] = 1
end

function Lua_HeroMaterialData:checkIsDownLineup(uniqueId)
	if self.downLineupDic and self.downLineupDic[uniqueId] == 1 then
		return true
	end
	return false
end

-- 选择的英雄所在的阵容和已选择的英雄所在的阵容数量 > 所在阵容的数量才可以下阵
function Lua_HeroMaterialData:checkLineupStateAndNum(selectUid, selectedList)
	local hvo;
	local config;
	local lineupType;
	local lineupDic = {};
	if selectedList then
		for i = 1, selectedList:getSize() do
			hvo = DataCenter.heroData:getItemByUniqueId(selectedList[i]);
			if hvo and hvo.lineup_state_list then
				for j = 1, #hvo.lineup_state_list do
					lineupType = hvo.lineup_state_list[j].key;
					config = getConfig("hero_camp_lineup", lineupType);
					if config and config.can_nil ~= 1 then
						if lineupDic[lineupType] == nil then
							lineupDic[lineupType] = 1;
						else
							lineupDic[lineupType] = lineupDic[lineupType] + 1;
						end
					end
				end
			end
		end
	else
		for k, _ in pairs(self.materialVoDic) do
			hvo = DataCenter.heroData:getItemByUniqueId(k);
			if hvo and hvo.lineup_state_list then
				for j = 1, #hvo.lineup_state_list do
					lineupType = hvo.lineup_state_list[j].key;
					config = getConfig("hero_camp_lineup", lineupType);
					if config and config.can_nil ~= 1 then
						if lineupDic[lineupType] == nil then
							lineupDic[lineupType] = 1;
						else
							lineupDic[lineupType] = lineupDic[lineupType] + 1;
						end
					end
				end
			end
		end
	end

	hvo = DataCenter.heroData:getItemByUniqueId(selectUid);
	if hvo and hvo.lineup_state_list then
		for j = 1, #hvo.lineup_state_list do
			lineupType = hvo.lineup_state_list[j].key;
			config = getConfig("hero_camp_lineup", lineupType);
			if config and config.can_nil ~= 1 then
				if lineupDic[lineupType] == nil then
					lineupDic[lineupType] = 1;
				else
					lineupDic[lineupType] = lineupDic[lineupType] + 1;
				end

				if lineupDic[lineupType] >= DataCenter.heroLineupData:getLineupHeroTotalNum(lineupType) then
					return false, config.name;
				end
			end
		end
	end

	return true;
end


-- 外部接口
------------------------------------------------------------
------------------------------------------------------------
------------------------------------------------------------
-- [[星级，阵营，数量，英雄ID]]
function Lua_HeroMaterialData:checkEnoughByList(material, ignoreUnId)
	if material then
		local id
		local star
		local camp
		local needNum
		local totalNum = 0
		local arr
		local vo
		local unique_id
		local bValue
		local selectedList = {}
		local selectedNum = 0
		for i = 1, #material do
			star = DataCenter.heroUtil:getHeroAwakenStarByQuality(material[i][1][1], material[i][1][2])
			camp = material[i][2]
			needNum = material[i][3]
			id = material[i][4]
			totalNum = totalNum + needNum

			arr = DataCenter.heroData:getHeroUidArrByStar(star)
			for j = 1, arr:getSize() do
				unique_id = arr[j]
				vo = DataCenter.heroData:getItemByUniqueId(unique_id)
				bValue = not vo.isUpLineup and not vo.lock_state and not vo.bcore
				if unique_id ~= ignoreUnId and selectedList[unique_id] == nil and bValue and not vo.is_league_transaction and not vo.trial_state then
					if id > 0 then
						if vo.id == id and vo.campType == camp and vo.star == star then
							needNum = needNum - 1
							selectedList[unique_id] = 1
							selectedNum = selectedNum + 1
							if needNum <= 0 then
								break
							end
						end
					elseif camp > 0 then
						if vo.campType == camp and vo.star == star then
							needNum = needNum - 1
							selectedList[unique_id] = 1
							selectedNum = selectedNum + 1
							if needNum <= 0 then
								break
							end
						end
					elseif star > 0 then
						if vo.star == star then
							needNum = needNum - 1
							selectedList[unique_id] = 1
							selectedNum = selectedNum + 1
							if needNum <= 0 then
								break
							end
						end
					end
				end
			end
		end

		return selectedNum >= totalNum
	end
	return true
end