-- 英雄战法 Vo
Lua_HeroTacticVo = class("Lua_HeroTacticVo", Lua_ToolVo)
function Lua_HeroTacticVo:ctor(id, count, config)
    self:superCall("ctor", id, count, config);
end

function Lua_HeroTacticVo:fromServer(msg)
    self:superCall("fromServer", msg);

    local tacticInfo = msg.fight_rule or msg;
    self._tacticLevel = tacticInfo.level; -- 等级
    self._tacticStar = tacticInfo.star; -- 星级
    self._tacticHeroUid = tonumber(tacticInfo.hero_unique_id); -- 英雄唯一id
    self._tacticInheritId = tonumber(tacticInfo.inherit_id); -- 附魂道具id(副战法)
    self._tacticAttachId = tonumber(tacticInfo.attach_id); -- 附魂给道具id(主战法)

    self.tacticProp = tacticInfo.prop; -- 总属性
    self.unique_id = tonumber(tacticInfo.item_unique_id); -- 战法道具唯一id
end

function Lua_HeroTacticVo:reset()
    self:superCall("reset");
    self._tacticLvlId = nil;
    self._tacticStarId = nil;
end

--------------------------------- 数据处理 ---------------------------------

function Lua_HeroTacticVo.Get:star()
    return self.tacticStar;
end

function Lua_HeroTacticVo.Get:tacticLevel()
    if self._tacticLevel == nil then
        self._tacticLevel = self.max_level;
    end
    return self._tacticLevel;
end

function Lua_HeroTacticVo.Set:tacticLevel(val)
    self._tacticLevel = val;
end

function Lua_HeroTacticVo.Get:tacticStar()
    if self._tacticStar == nil then
        self._tacticStar = self.max_star;
    end
    return self._tacticStar;
end

function Lua_HeroTacticVo.Set:tacticStar(val)
    self._tacticStar = val;
end

function Lua_HeroTacticVo.Get:tacticHeroUid()
    if self._tacticHeroUid == nil then
        self._tacticHeroUid = 0;
    end
    if self.tacticAttachId > 0 then
        local mainTacticVo = DataCenter.packData:getItemVoByUnId(self.tacticAttachId);
        return mainTacticVo.tacticHeroUid;
    else
        return self._tacticHeroUid;
    end
end

function Lua_HeroTacticVo.Get:tacticInheritId()
    if self._tacticInheritId == nil then
        self._tacticInheritId = 0;
    end
    return self._tacticInheritId;
end

function Lua_HeroTacticVo.Get:tacticAttachId()
    if self._tacticAttachId == nil then
        self._tacticAttachId = 0;
    end
    return self._tacticAttachId;
end

function Lua_HeroTacticVo.Get:tacticLvlId()
    if self._tacticLvlId == nil then
        self._tacticLvlId = self.lvl_prop * 1000 + self.tacticLevel;
    end
    return self._tacticLvlId;
end

function Lua_HeroTacticVo.Get:tacticStarId()
    if self._tacticStarId == nil then
        self._tacticStarId = self.id * 100 + self.tacticStar;
    end
    return self._tacticStarId;
end

function Lua_HeroTacticVo.Get:isActive()
    return self.count >= 1;
end

function Lua_HeroTacticVo.Get:tacticSort()
    local weight;
    if self.isActive then
        weight = (self.color * 100000 + self.tacticStar * 10000 + self.tacticLevel * 10 + self.id) * -100;
    else
        weight = (self.color * 100000 + self.min_star * 10000 + self.min_level * 10 + self.id) * -1;
    end
    return weight;
end

function Lua_HeroTacticVo.Get:activeNum()
    if self._activeNum == nil then
        local itemVo = Lua_ToolVo.createToolVo(self.chipId);
        self._activeNum = itemVo.synthesisNum;
    end
    return self._activeNum;
end

function Lua_HeroTacticVo.Get:showPropArr()
    if self.tacticProp ~= nil then
        return DataCenter.attributesUtil:getAttrArrByServer(self.tacticProp);
    else
        return DataCenter.heroTacticUtil:getAttrVoArr(self.tacticLvlId, self.tacticStarId);
    end
end

function Lua_HeroTacticVo.Get:tacticColorName()
    return Lua_ColorDefine.getColorStr(self.colorStr, self.tacticName);
end

function Lua_HeroTacticVo.Get:isUpgrade()
    if self.isActive and self.tacticLevel < self.max_level then
        return DataCenter.heroTacticUtil:isMeetUpgradeCost(self.tacticLvlId);
    end
    return false;
end

function Lua_HeroTacticVo.Get:isUpstar()
    if self.isActive and self.tacticStar < self.max_star then
        return DataCenter.heroTacticUtil:isMeetUpstarCost(self.tacticStarId);
    end
    return false;
end









--------------------------------- 配置数据 ---------------------------------

function Lua_HeroTacticVo.Get:extraConfig()
    if self._extraId ~= self.id then
        self._extraId = self.id;
        self._extraConfig = getConfig("fight_rule", self._extraId);
    end
    return self._extraConfig;
end

function Lua_HeroTacticVo.Get:lvl_prop()
    if self.extraConfig == nil or self.extraConfig.lvl_prop == nil then
        return 1;
    end
    return self.extraConfig.lvl_prop;
end

function Lua_HeroTacticVo.Get:min_level()
    if self.extraConfig == nil or self.extraConfig.level == nil then
        return 0;
    end
    return self.extraConfig.level;
end

function Lua_HeroTacticVo.Get:max_level()
    if self.extraConfig == nil or self.extraConfig.max_level == nil then
        return 0;
    end
    return self.extraConfig.max_level;
end

function Lua_HeroTacticVo.Get:min_star()
    if self.extraConfig == nil or self.extraConfig.star == nil then
        return 0;
    end
    return self.extraConfig.star;
end

function Lua_HeroTacticVo.Get:max_star()
    if self.extraConfig == nil or self.extraConfig.max_star == nil then
        return 0;
    end
    return self.extraConfig.max_star;
end

function Lua_HeroTacticVo.Get:fetter()
    if self.extraConfig == nil or self.extraConfig.fetter == nil then
        return nil;
    end
    return self.extraConfig.fetter;
end

function Lua_HeroTacticVo.Get:chipId()
    if self.extraConfig == nil or self.extraConfig.transfer_item == nil then
        return 0;
    end
    return self.extraConfig.transfer_item[1];
end

function Lua_HeroTacticVo.Get:tacticName()
    if self.extraConfig == nil or self.extraConfig.name == nil then
        return self.name;
    end
    return self.extraConfig.name;
end

function Lua_HeroTacticVo.Get:desc()
    if self.extraConfig == nil or self.extraConfig.desc == nil then
        return "";
    end
    return self.extraConfig.desc;
end

function Lua_HeroTacticVo.Get:atlas_img_path()
    if self.extraConfig == nil or self.extraConfig.atlas_img_path == nil then
        return "";
    end
    return self.extraConfig.atlas_img_path;
end

function Lua_HeroTacticVo.Get:effect_path()
    if self.extraConfig == nil or self.extraConfig.effect_path == nil then
        return "";
    end
    return self.extraConfig.effect_path;
end

function Lua_HeroTacticVo.Get:effect_name()
    if self.extraConfig == nil or self.extraConfig.effect_name == nil then
        return "";
    end
    return self.extraConfig.effect_name;
end

function Lua_HeroTacticVo.Get:spine_scale()
    if self.extraConfig == nil or self.extraConfig.spine_scale == nil then
        return 1;
    end
    return self.extraConfig.spine_scale;
end

function Lua_HeroTacticVo.Get:spine_offset()
    if self.extraConfig == nil or self.extraConfig.spine_offset == nil then
        return {0, 0};
    end
    return self.extraConfig.spine_offset;
end

function Lua_HeroTacticVo.Get:main_spine_scale()
    if self.extraConfig == nil or self.extraConfig.main_spine_scale == nil then
        return 1;
    end
    return self.extraConfig.main_spine_scale;
end

function Lua_HeroTacticVo.Get:main_spine_offset()
    if self.extraConfig == nil or self.extraConfig.main_spine_offset == nil then
        return {0, 0};
    end
    return self.extraConfig.main_spine_offset;
end

function Lua_HeroTacticVo.Get:sub_spine_scale()
    if self.extraConfig == nil or self.extraConfig.sub_spine_scale == nil then
        return 1;
    end
    return self.extraConfig.sub_spine_scale;
end

function Lua_HeroTacticVo.Get:sub_spine_offset()
    if self.extraConfig == nil or self.extraConfig.sub_spine_offset == nil then
        return {0, 0};
    end
    return self.extraConfig.sub_spine_offset;
end