-- 登录福利
Lua_LoginRewardData = class("Lua_LoginRewardData");
function Lua_LoginRewardData:ctor()
end

function Lua_LoginRewardData:dispose()
	self._loginIdArr = nil;
	self._loginIdRewardDic = nil;
	self._rechargeRewardList = nil;
	self._rechargeId = nil;
	self._lockState = nil;
	self._rearDay = nil;
	self._totalDay = nil;
	self._signinDay = nil;
end

function Lua_LoginRewardData:fromServer50108105(msg)
	if msg == nil then
		return;
	end
	if self._loginIdRewardDic == nil then
		self._loginIdRewardDic = {};
	end
	self.isNeedSort = true;

	local len = #msg.daily_attendance;
	if len > 0 then
		local signinObj, upgradeObj;
		self._loginIdArr = Array();
		for i = 1, len do
			signinObj = msg.daily_attendance[i]; -- 签到奖励
			upgradeObj = msg.premium_reward[i]; -- 超值奖励
			self._loginIdArr:add(signinObj.id);

			-- 签到奖励和超值奖励共用一个id
			if self._loginIdRewardDic[signinObj.id] == nil or self._loginIdRewardDic[upgradeObj.id] == nil then
				self._loginIdRewardDic[signinObj.id] = {};
			end
			self._loginIdRewardDic[signinObj.id].signinReward = signinObj;
			self._loginIdRewardDic[upgradeObj.id].upgradeReward = upgradeObj;
		end
	end
	self._rechargeRewardList = msg.seven_day; -- 累充奖励
	self._rechargeId = msg.premium_reward_state.key; -- 充值id
	self._lockState = msg.premium_reward_state.value; -- 0未解锁超值奖励
	self._rearDay = msg.rear_days; -- 最后第几天天数
	self._totalDay = msg.acc_days; -- 累计充值天数
	self._signinDay = msg.day; -- 登录天数

	local obj;
	self.loginRedState = false;
	for i = 1, self.loginIdArr:getSize() do
		obj = self:getLoginReward(self.loginIdArr[i]);
		if obj and (obj.signinReward and obj.signinReward.state == 1
			or not self.isLock and obj.upgradeReward and obj.upgradeReward.state == 1) then
				self.loginRedState = true;
				break;
		end
	end
	DataCenter.redPointData:updateRedPointState(Lua_RedPointList.LoginReward_Login, self.loginRedState);

	local redState = false;
	for i = 1, #msg.seven_day do
		if not redState and msg.seven_day[i].state == 1 then
			redState = true;
			break;
		end
	end
	DataCenter.redPointData:updateRedPointState(Lua_RedPointList.LoginReward_Recharge, redState);
end

function Lua_LoginRewardData.Get:loginIdArr()
	local arr = self._loginIdArr or Array();
	if self.isNeedSort then
		self.isNeedSort = false;
		arr:sort(nil, false);
	end
	return arr;
end

function Lua_LoginRewardData:getLoginReward(id)
	if self._loginIdRewardDic == nil or id == nil then
		return nil;
	end
	return self._loginIdRewardDic[id];
end

function Lua_LoginRewardData.Get:rechargeRewardList()
	local list = self._rechargeRewardList or {};
	if #list > 0 then
		table.sort( list,
			function(a, b)
				local aWeight = 0;
				if a.state == 0 then -- 未解锁
					aWeight = 2;
				else -- 已解锁
					if a.state == 3 then -- 未达成
						aWeight = 3
					elseif a.state == 1 then -- 可领取
						aWeight = 4
					elseif a.state == 2 then -- 已领取
						aWeight = 1;
					end
				end
				local bWeight = 0;
				if b.state == 0 then
					bWeight = 2;
				else
					if b.state == 3 then
						bWeight = 3
					elseif b.state == 1 then
						bWeight = 4
					elseif b.state == 2 then
						bWeight = 1;
					end
				end
				if aWeight == bWeight then
					return a.id < b.id;
				else
					return aWeight > bWeight;
				end
			end
		);
	end
	return list;
end

function Lua_LoginRewardData.Get:rechargeId()
	return self._rechargeId or 0;
end

-- true 未解锁
function Lua_LoginRewardData.Get:isLock()
	local state = self._lockState or 0;
	return state == 0;
end

function Lua_LoginRewardData.Get:rearDay()
	return self._rearDay or 0;
end

function Lua_LoginRewardData.Get:totalDay()
	return self._totalDay or 0;
end

function Lua_LoginRewardData.Get:signinDay()
	return self._signinDay or 1;
end

function Lua_LoginRewardData.Get:instantRewardArr()
	local obj, list, itemInfo, tvoInfo;
	local sortWeight = 1;
	local itemDic = {};
	for i = 1, self.loginIdArr:getSize() do
		obj = self:getLoginReward(self.loginIdArr[i]);
		if obj and self.loginIdArr[i] <= self.signinDay then
			if obj.signinReward and obj.signinReward.state == 1 then -- 签到奖励
				list = obj.signinReward.item;
				for j = 1, #list do
					itemInfo = list[j];
					tvoInfo = itemDic[itemInfo.id];
					if tvoInfo == nil then
						tvoInfo = {id = itemInfo.id, amount = itemInfo.amount, sortWeight = sortWeight};
						sortWeight = sortWeight + 1;
						itemDic[itemInfo.id] = tvoInfo;
					else
						tvoInfo.amount = tvoInfo.amount + itemInfo.amount;
					end
				end
			end
			if obj.upgradeReward then -- 超值奖励
				list = obj.upgradeReward.item;
				for k = 1, #list do
					itemInfo = list[k];
					tvoInfo = itemDic[itemInfo.id];
					if tvoInfo == nil then
						tvoInfo = {id = itemInfo.id, amount = itemInfo.amount, sortWeight = sortWeight};
						sortWeight = sortWeight + 1;
						itemDic[itemInfo.id] = tvoInfo;
					else
						tvoInfo.amount = tvoInfo.amount + itemInfo.amount;
					end
				end
			end
		end
	end
	local arr = Array();
	for k,v in pairs(itemDic) do
		arr:add(v);
	end
	arr:sort("sortWeight", false);
	return arr;
end

function Lua_LoginRewardData.Get:totalRewardArr()
	local obj, list, itemInfo, tvoInfo;
	local sortWeight = 1;
	local itemDic = {};
	for i = 1, self.loginIdArr:getSize() do
		obj = self:getLoginReward(self.loginIdArr[i]);
		if obj then
			if obj.upgradeReward then -- 超值奖励
				list = obj.upgradeReward.item;
				for k = 1, #list do
					itemInfo = list[k];
					tvoInfo = itemDic[itemInfo.id];
					if tvoInfo == nil then
						tvoInfo = {id = itemInfo.id, amount = itemInfo.amount, sortWeight = sortWeight};
						sortWeight = sortWeight + 1;
						itemDic[itemInfo.id] = tvoInfo;
					else
						tvoInfo.amount = tvoInfo.amount + itemInfo.amount;
					end
				end
			end
		end
	end
	local arr = Array();
	for k,v in pairs(itemDic) do
		arr:add(v);
	end
	arr:sort("sortWeight", false);
	return arr;
end