Lua_MainLineTaskVo = class("Lua_MainLineTaskVo")
function Lua_MainLineTaskVo:ctor()
	self:dispose();
end

function Lua_MainLineTaskVo:dispose()
	self.uid = 0;
	self.tick = 0;
	self.state = 0;
	self.send_cost = nil;
	if self._condArr ~= nil then
		self._condArr:clear();
		self._condArr = nil;
	end
	if self.selectedList then
		self.selectedList:clear();
	end
	self.isNew = false;
end

function Lua_MainLineTaskVo:fromServer(data)
	self.uid = tonumber(data.uid); -- 情报唯一id
	self.tick = data.tick; -- 派遣结束时间戳
	self.state = data.state; -- 状态：0未派遣，1派遣中，2派遣完成可领取
	self.send_cost = data.send_cost; -- 派遣消耗
	self.info_id = data.info_id; -- 情报配置id
end

-- 派遣剩余时间
function Lua_MainLineTaskVo.Get:leftTime()
	return self.tick - DataCenter.roleData.serverTime;
end

function Lua_MainLineTaskVo.Get:sortWeight()
	if self.state == 0 then
		return 10000 + self.color;
	elseif self.state == 1 then
		return 1000 - self.leftTime + self.color;
	elseif self.state == 2 then
		return 100000 + self.color;
	end
	return 0;
end

function Lua_MainLineTaskVo.Get:condArr()
	if self._condArr == nil then
		self._condArr = Array();
		if self.hero_cond ~= nil then
			if self.hero_cond[1] ~= nil then
				self._condArr:add({ condType = 1, condValue = self.hero_cond[1] }); -- 星级
			end
			if self.hero_cond[2] ~= nil then
				for i = 1, #self.hero_cond[2] do
					self._condArr:add({ condType = 2, condValue = self.hero_cond[2][i] }); -- 阵营
				end
			end
		end
	end
	return self._condArr;
end

function Lua_MainLineTaskVo.Get:costVo()
	if self.cost then
		local id
		local count
		if self.state == 0 then
			id = self.cost.id
			count = self.cost.amount
		else
			id = self.cost[1]
			count = self.cost[2]
		end
		if self._costVo == nil then
			self._costVo = Lua_ToolVo.createToolVo(id, count)
		else
			self._costVo:setData(id, count)
		end
		return self._costVo
	end
	return nil
end

function Lua_MainLineTaskVo.Get:cost()
	if self.state == 0 then
		return self.send_cost[1];
	elseif self.state == 1 then
		if self.speed_cost ~= nil then
			for i = 1, #self.speed_cost do
				if self.leftTime > self.speed_cost[i][1] then
					return self.speed_cost[i][2];
				end
			end
		end
	end
	return nil;
end

-------------------- 自动派遣 --------------------

function Lua_MainLineTaskVo.Get:bCostEnough()
	if self.costVo == nil then
		return true;
	end
	return DataCenter.packData:getItemCountById(self.costVo.id) >= self.costVo.count;
end

function Lua_MainLineTaskVo:getSendList()
	if self.selectedList == nil then
		self.selectedList = Array();
	else
		self.selectedList:clear();
	end
	
	local arr = Array();
	local heroArr, len;
	for i = 1, self.hero_num do
		heroArr, len = self:getHeroList();
		if len > 0 then
			self.selectedList:add(heroArr[1].unique_id);
			arr:add(heroArr[1].unique_id);
		end
	end
	heroArr:clear();
	heroArr = nil;
	return arr;
end

function Lua_MainLineTaskVo:getHeroList()
	local uidArr = DataCenter.heroData.allHeroUidArr;
	local arr = Array();
	local count = 0;
	local vo;
	for i = 1, uidArr:getSize() do
		vo = DataCenter.heroData:getItemByUniqueId(uidArr[i]);
		if not vo.send_info_state and self.selectedList:indexOf(vo.unique_id) == -1 and self:getCondEnoughNum(vo) > 0 then
			arr:add(vo);
			count = count + 1;
		end
	end
	arr:sortf(self, self.onHeroSortFun);
	return arr, count;
end

function Lua_MainLineTaskVo:onHeroSortFun(a, b)
	local _a = self:getCondEnoughNum(a);
	local _b = self:getCondEnoughNum(b);
	if _a < _b then
		return false;
	elseif _a == _b then
		if a.star > b.star then
			return false;
		elseif a.star == b.star then
			if a.id > b.id then
				return false;
			end
		end
	end
	return true;
end

function Lua_MainLineTaskVo:getCondEnoughNum(vo)
	local num = 0;
	local arr = self.condArr;
	for i = 1, arr:getSize() do
		if not DataCenter.mainLineData:checkCondIsSatisfy(self.selectedList, arr[i]) then
			if arr[i].condType == 1 then
				if vo.quality >= arr[i].condValue[1] and vo.quality_star >= arr[i].condValue[2] then
					num = num + 1;
				end
			elseif arr[i].condType == 2 then
				if vo.campType == arr[i].condValue then
					num = num + 1;
				end
			end
		end
	end
	return num;
end

-------------------- 配置数据 --------------------

function Lua_MainLineTaskVo.Get:config()
	return getConfig("mainline_info_send", self.info_id);
end

function Lua_MainLineTaskVo.Get:name()
	if self.config == nil then
		return nil;
	end
	return self.config.name;
end

function Lua_MainLineTaskVo.Get:color()
	if self.config == nil then
		return 1;
	end
	return self.config.color;
end

function Lua_MainLineTaskVo.Get:time()
	if self.config == nil then
		return 0;
	end
	return self.config.time;
end

function Lua_MainLineTaskVo.Get:speed_cost()
	if self.config == nil then
		return nil;
	end
	return self.config.speed_cost;
end

function Lua_MainLineTaskVo.Get:hero_num()
	if self.config == nil then
		return 0;
	end
	return self.config.hero_num;
end

function Lua_MainLineTaskVo.Get:hero_cond()
	if self.config == nil then
		return nil;
	end
	return self.config.hero_cond;
end

function Lua_MainLineTaskVo.Get:reward()
	if self.config == nil then
		return nil;
	end
	return self.config.reward;
end

function Lua_MainLineTaskVo.Get:des()
	if self.config == nil then
		return nil;
	end
	return self.config.des;
end