-- 藏品柜
checkRequire("Lua_CollectionCabinetConfigData");
checkRequire("Lua_CollectionCabinetVo");
Lua_CollectionCabinetData = class("Lua_CollectionCabinetData");
function Lua_CollectionCabinetData:ctor()
	self:superCall("ctor");

	self.colllectionColorIdsDic = {}; -- 藏品对应品质信息
	self.collectionIdsDic = {}; -- key：已激活的藏品id value：vo
	self._collectionActiveIdsArr = Array(); -- 已激活的藏品id

	-- 弹窗
	self.activeSucceedIdArr = Array(); -- 激活成功的藏品id
	self.upStarSucceedIdArr = Array(); -- 升星成功的藏品id
	self.growUpIdArr = Array(); -- 成长进度的藏品id
end

function Lua_CollectionCabinetData:dispose()
	if self.collectionIdsDic ~= nil then
		local len = self._collectionActiveIdsArr:getSize();
		while len > 0 do
			local key = self._collectionActiveIdsArr:remove(len);
			if key then
				self.collectionIdsDic[key]:dispose();
			end
			len = len - 1;
		end
	end
	if self.colllectionColorIdsDic ~= nil then
		for _,v in pairs(self.colllectionColorIdsDic) do
			v:dispose();
			v:clear();
		end
	end
	if self.activeSucceedIdArr ~= nil then
		self.activeSucceedIdArr:dispose();
		self.activeSucceedIdArr:clear();
	end
	if self.upStarSucceedIdArr ~= nil then
		self.upStarSucceedIdArr:dispose();
		self.upStarSucceedIdArr:clear();
	end
	if self.growUpIdArr ~= nil then
		self.growUpIdArr:dispose();
		self.growUpIdArr:clear();
	end
	self._diamondTimes = nil;
	self._guaranteeTimes = nil;
	self._curPeriodId = nil;
	self._curPeriodStartTime = nil;
	self._curPeriodEndTime = nil;
	self._nextPeriodId = nil;
	self._nextPeriodStartTime = nil;
	self._nexrPeriodEndTime = nil;

	self.isActiveIdSort = nil;
	self.isColorIdsSort = nil;

	self.allCostDic = nil;
	self.activeCostDic = nil;

	self.curShowDrawRewardDic = nil;
	self.curShowDrawRewardIdxDic = nil;
	self:superCall("dispose");
end

function Lua_CollectionCabinetData:fromServer50230002(msg)
	if msg == nil then
		return;
	end
	self._leftShareTimes = msg.total_share_count; -- 今日已分享次数
	self._lastShareTick = msg.last_share_tick; -- 上次分享时间

	local obj, collectId, vo;
	for i = 1, #msg.collection_article_list do
		obj = msg.collection_article_list[i];
		collectId = obj.collection_item_id;
		vo = self:getCollectionCabinetVo(collectId);
		if vo ~= nil then
			vo:fromServer(obj);
			if self._collectionActiveIdsArr:indexOf(collectId) == -1 then
				self._collectionActiveIdsArr:add(collectId);
			end
			if self.colllectionColorIdsDic[vo.color] == nil then
				self.colllectionColorIdsDic[vo.color] = Array();
			end
			if self.colllectionColorIdsDic[vo.color]:indexOf(collectId) == -1 then
				self.colllectionColorIdsDic[vo.color]:add(collectId);
			end
		end
	end
	self.isActiveIdSort = true;
	self.isColorIdsSort = true;
	self:updateCollectionCostInfo(true);
	self:checkCollectionActiveRedPoint(true);
end

function Lua_CollectionCabinetData:getCollectionCabinetVo(collectId)
	if collectId == nil then
		return;
	end
	collectId = tonumber(collectId);

	local vo = self.collectionIdsDic[collectId];
	if vo == nil then
		local config = getConfig("new_collection_cabinet", collectId);
		if config then
			vo = Lua_CollectionCabinetVo.new(collectId);
			self.collectionIdsDic[collectId] = vo;
		end
	end
	return vo;
end

function Lua_CollectionCabinetData:fromServer80230002(msg)
	if msg == nil then
		return;
	end

	if msg.change_type == 4 then -- change_type：1升级 2激活 3升星 4分享状态更新 5成长进度
		OperateAlert.showLangValue(787041);
		self._leftShareTimes = msg.total_share_count; -- 今日已分享次数
		self._lastShareTick = msg.last_share_tick; -- 上次分享时间
	end

	local obj, collectId, vo;
	for i = 1, #msg.collection_article_list do
		obj = msg.collection_article_list[i];
		collectId = obj.collection_item_id;
		vo = self:getCollectionCabinetVo(collectId);
		if vo ~= nil then
			if msg.change_type == 2 then
				if self.activeSucceedIdArr:indexOf(collectId) == -1 then
					self.activeSucceedIdArr:add(collectId);
				end
				if self._collectionActiveIdsArr:indexOf(collectId) == -1 then
					self._collectionActiveIdsArr:add(collectId);
				end
				if self.colllectionColorIdsDic[vo.color] == nil then
					self.colllectionColorIdsDic[vo.color] = Array();
				end
				if self.colllectionColorIdsDic[vo.color]:indexOf(collectId) == -1 then
					self.colllectionColorIdsDic[vo.color]:add(collectId);
				end
			elseif msg.change_type == 3 then
				if self.upStarSucceedIdArr:indexOf(collectId) == -1 then
					self.upStarSucceedIdArr:add(collectId);
				end
			elseif msg.change_type == 5 then
				if vo.isActive then
					-- print("Lua_CollectionCabinetData", obj.grow_progress, vo.growProgress);
					local changeProgress = obj.grow_progress - vo.growProgress; -- 改变的进度值
					if changeProgress > 0 then
						vo.changeProgress = changeProgress;
						if self.growUpIdArr:indexOf(collectId) == -1 then
							self.growUpIdArr:add(collectId);
						end
					end
				end
			end
			vo:fromServer(obj);
		end
	end
	self.isActiveIdSort = true;
	self.isColorIdsSort = true;
	self:showActiveCollection();
	self:showUpStarCollection();
	self:showGrowUpCollection();
	self:updateCollectionCostInfo(msg.change_type);
	self:checkCollectionActiveRedPoint(msg.change_type == 2);
end

function Lua_CollectionCabinetData:showActiveCollection()
	if self.activeSucceedIdArr and self.activeSucceedIdArr:getSize() > 0 and not Lua_WindowUtil.isWindowShowing("Lua_CollectionCabinetActiveSucceedWindow") then
		local collectId = self.activeSucceedIdArr:shift();
		Lua_WindowUtil.openWin("Lua_CollectionCabinetActiveSucceedWindow", Lua_WinParam.ACTION_OPEN, collectId);
	end
end

function Lua_CollectionCabinetData:showUpStarCollection()
	if self.upStarSucceedIdArr and self.upStarSucceedIdArr:getSize() > 0 and not Lua_WindowUtil.isWindowShowing("Lua_CollectionCabinetUpStarSucceedWindow") then
		local collectId = self.upStarSucceedIdArr:shift();
		Lua_WindowUtil.openWin("Lua_CollectionCabinetUpStarSucceedWindow", Lua_WinParam.ACTION_OPEN, collectId);
	end
end

function Lua_CollectionCabinetData:showGrowUpCollection()
	if self.growUpIdArr and self.growUpIdArr:getSize() > 0 and not Lua_WindowUtil.isWindowShowing("Lua_CollectionCabinetGrowUpWindow") then
		Lua_WindowUtil.openWin("Lua_CollectionCabinetGrowUpWindow", Lua_WinParam.ACTION_OPEN, self.growUpIdArr:shift());
	end
end

function Lua_CollectionCabinetData:checkCollectionActiveRedPoint(bCheck)
	local vo;
	local upStarRed = false;
	local upLvRed = false;
	for i = 1, self.collectionActiveArr:getSize() do
		vo = self:getCollectionCabinetVo(self.collectionActiveArr[i]);
		if not upStarRed and vo.isCanUpStar then
			upStarRed = true;
		end
		if not upLvRed and vo.isCanUpLv then
			upLvRed = true;
		end
		if upStarRed and upLvRed then
			break;
		end
	end
	DataCenter.redPointData:updateRedPointState(Lua_RedPointList.NewCollectionCabinet_Active, upStarRed or upLvRed);
	DataCenter.redPointData:updateRedPointState(Lua_RedPointList.NewCollectionCabinet_Chip_CanUpStar, upStarRed);

	if bCheck then
		local isCanActive = self:checkCollectionCanActiveRedPoint();
		DataCenter.redPointData:updateRedPointState(Lua_RedPointList.NewCollectionCabinet_Chip_CanActive, isCanActive);
	end
	-- DataCenter.packData:checkChipRedPoint();
end

function Lua_CollectionCabinetData:checkCollectionCanActiveRedPoint()
	local allIdsArr = DataCenter.collectionCabinetUtil.allCollectionIdsArr;
	if allIdsArr:getSize() == self.collectionActiveArr:getSize() then
		return false;
	end

	local vo;
	local isCanActive = false;
	for i = 1, allIdsArr:getSize() do
		vo = self:getCollectionCabinetVo(allIdsArr[i]);
		if not isCanActive and vo.isCanActive then
			isCanActive = true;
			break;
		end
	end
	return isCanActive;
end

function Lua_CollectionCabinetData:checkCollectionPackRedPoint(dic)
	if not DataCenter.funcOpenData:isOpened(10218) then
		return;
	end
	local bCheck = false;
	for k,v in pairs(dic) do
		if self.allCostDic and self.allCostDic[k] then
			bCheck = true;
			break;
		end
	end

	if bCheck then
		local allIdsArr = DataCenter.collectionCabinetUtil.allCollectionIdsArr;
		local upStarRed = false;
		local upLvRed = false;
		local isCanActive = false;
		for i = 1, allIdsArr:getSize() do
			vo = self:getCollectionCabinetVo(allIdsArr[i]);
			if not upStarRed and vo.isCanUpStar then
				upStarRed = true;
			end
			if not upLvRed and vo.isCanUpLv then
				upLvRed = true;
			end
			if not isCanActive and vo.isCanActive then
				isCanActive = true;
			end
			if upStarRed and upLvRed and isCanActive then
				break;
			end
		end
		DataCenter.redPointData:updateRedPointState(Lua_RedPointList.NewCollectionCabinet_Active, upStarRed or upLvRed);
		DataCenter.redPointData:updateRedPointState(Lua_RedPointList.NewCollectionCabinet_Chip_CanUpStar, upStarRed);
		DataCenter.redPointData:updateRedPointState(Lua_RedPointList.NewCollectionCabinet_Chip_CanActive, isCanActive);
		-- DataCenter.packData:checkChipRedPoint();
	end
end

function Lua_CollectionCabinetData:checkCollectionMoneyRedPoint(dic)
	if not DataCenter.funcOpenData:isOpened(10218) then
		return;
	end
	local bCheck = false;
	for k,v in pairs(dic) do
		if self.activeCostDic and self.activeCostDic[k] then
			bCheck = true;
			break;
		end
	end

	if bCheck then
		local vo;
		local upStarRed = false;
		local upLvRed = false;
		for i = 1, self.collectionActiveArr:getSize() do
			vo = self:getCollectionCabinetVo(self.collectionActiveArr[i]);
			if not upStarRed and vo.isCanUpStar then
				upStarRed = true;
			end
			if not upLvRed and vo.isCanUpLv then
				upLvRed = true;
			end
			if upStarRed and upLvRed then
				break;
			end
		end
		DataCenter.redPointData:updateRedPointState(Lua_RedPointList.NewCollectionCabinet_Active, upStarRed or upLvRed);
		DataCenter.redPointData:updateRedPointState(Lua_RedPointList.NewCollectionCabinet_Chip_CanUpStar, upStarRed);
		-- DataCenter.packData:checkChipRedPoint();
	end
end

function Lua_CollectionCabinetData:updateCollectionCostInfo(bCheck)
	if type(bCheck) == "number" and (bCheck == 4 or bCheck == 5) then
		return;
	end

	local allIdsArr = DataCenter.collectionCabinetUtil.allCollectionIdsArr;
	local costId;
	self.allCostDic = {};
	self.activeCostDic = {};
	for i = 1, allIdsArr:getSize() do
		vo = self:getCollectionCabinetVo(allIdsArr[i]);
		if vo.isActive then
			if vo.upStarCostList then
				for i = 1, #vo.upStarCostList do
					costId = vo.upStarCostList[i][1];
					costNum = vo.upStarCostList[i][2];
					if self.allCostDic[costId] == nil or self.allCostDic[costId] > costNum then
						self.allCostDic[costId] = costNum;
					end
					if (type(bCheck) == "boolean" and bCheck) or (type(bCheck) == "number" and bCheck == 2) then
						if self.activeCostDic[costId] == nil or self.activeCostDic[costId] > costNum then
							self.activeCostDic[costId] = costNum;
						end
					end
				end
			end
			if vo.upLvCostList then
				for i = 1, #vo.upLvCostList do
					costId = vo.upLvCostList[i][1];
					costNum = vo.upLvCostList[i][2];
					if self.allCostDic[costId] == nil or self.allCostDic[costId] > costNum then
						self.allCostDic[costId] = costNum;
					end
					if (type(bCheck) == "boolean" and bCheck) or (type(bCheck) == "number" and bCheck == 2) then
						if self.activeCostDic[costId] == nil or self.activeCostDic[costId] > costNum then
							self.activeCostDic[costId] = costNum;
						end
					end
				end
			end
		else
			for i = 1, vo.activeCostIdArr:getSize() do
				costId = vo.activeCostIdArr[i];
				costNum = vo.activeCostItemDic[costId];
				if self.allCostDic[costId] == nil or self.allCostDic[costId] > costNum then
					self.allCostDic[costId] = costNum;
				end
			end
		end
	end
end










function Lua_CollectionCabinetData.Get:leftShareTimes()
	return self._leftShareTimes or 0;
end

function Lua_CollectionCabinetData.Get:shareTime()
	local tick = self._lastShareTick or 0;
	local time = DataCenter.roleData.serverTime - tick;
	return time;
end

-- 已激活藏品
function Lua_CollectionCabinetData.Get:collectionActiveArr()
	if self.isActiveIdSort then
		self.isActiveIdSort = false;
		self._collectionActiveIdsArr:sortf(self, self.sortFun);
	end
	return self._collectionActiveIdsArr;
end

function Lua_CollectionCabinetData:sortFun(a, b)
	local collectVo_a = self:getCollectionCabinetVo(a);
	local collectVo_b = self:getCollectionCabinetVo(b);
	if collectVo_a.color == collectVo_b.color then
		return collectVo_a.sort < collectVo_b.sort;
	else
		return collectVo_a.color > collectVo_b.color;
	end
end

function Lua_CollectionCabinetData:getColllectColorIdsByColor(color)
	if color == nil or self.colllectionColorIdsDic[color] == nil then
		return nil;
	end
	if self.isColorIdsSort then
		self.isColorIdsSort = false;
		for k,v in pairs(self.colllectionColorIdsDic) do
			v:sortf(self, self.sortColorFun);
		end
	end
	return self.colllectionColorIdsDic[color];
end

function Lua_CollectionCabinetData:sortColorFun(a, b)
	local collectVo_a = self:getCollectionCabinetVo(a);
	local collectVo_b = self:getCollectionCabinetVo(b);
	return collectVo_a.sort < collectVo_b.sort;
end

-- 当前藏品所需碎片数量
function Lua_CollectionCabinetData:getCurChipNumByCollectId(collectId)
	local collectVo = self:getCollectionCabinetVo(collectId);
	if collectVo.isMaxStar then
		return 0;
	end

	local num = 0;
	if collectVo.isActive then -- 升星所需
		local costListCfg = collectVo.upStarCostList;
		if costListCfg then
			for i = 1, #costListCfg do
				if costListCfg[i][1] == collectVo.chipId then
					num = num + costListCfg[i][2];
				end
			end
		end
	else -- 激活所需
		num = collectVo.activeCostCollectChipNum;
	end
	return num;
end




















---------------------------------------- 抽奖 ----------------------------------------
function Lua_CollectionCabinetData:fromServer50230001(msg)
	if msg == nil then
		return;
	end
	self._diamondTimes = msg.diamond_time; -- 剩余的钻石抽奖次数
	self._guaranteeTimes = msg.minimum_guarantee; -- 保底次数

	table.sort(msg.current_up, function(a, b) return tonumber(a.value) < tonumber(b.value) end);
	self.curPeriodId = tonumber(msg.current_up[1].key); -- 当期up池id
	self._curPeriodStartTime = tonumber(msg.current_up[1].value); -- 当期开始时间
	self._curPeriodEndTime = tonumber(msg.current_up[1].sub); -- 当期结束时间
	self._nextPeriodId = tonumber(msg.current_up[2].key); -- 下期up池id
	self._nextPeriodStartTime = tonumber(msg.current_up[2].value); -- 下期开始时间
	self._nexrPeriodEndTime = tonumber(msg.current_up[2].sub); -- 下期结束时间
end

function Lua_CollectionCabinetData:fromServer50230006(msg)
	if msg == nil then
		return;
	end
	self._diamondTimes = msg.diamond_time; -- 剩余的钻石抽奖次数
	self._guaranteeTimes = msg.minimum_guarantee; -- 保底次数

	table.sort(msg.current_up, function(a, b) return tonumber(a.value) < tonumber(b.value) end);
	self.curPeriodId = tonumber(msg.current_up[1].key); -- 当期up池id
	self._curPeriodStartTime = tonumber(msg.current_up[1].value); -- 当期开始时间
	self._curPeriodEndTime = tonumber(msg.current_up[1].sub); -- 当期结束时间
	self._nextPeriodId = tonumber(msg.current_up[2].key); -- 下期up池id
	self._nextPeriodStartTime = tonumber(msg.current_up[2].value); -- 下期开始时间
	self._nexrPeriodEndTime = tonumber(msg.current_up[2].sub); -- 下期结束时间

	self:updateCollectionDrawRed();
end

function Lua_CollectionCabinetData.Get:diamondTimes()
	return self._diamondTimes or 0;
end

function Lua_CollectionCabinetData.Get:guaranteeTimes()
	return self._guaranteeTimes or 0;
end

function Lua_CollectionCabinetData.Set:curPeriodId(val)
	if self._curPeriodId ~= val then
		self._curPeriodId = val;
		self:initShowRewardInfo();
	end
end

function Lua_CollectionCabinetData.Get:curPeriodId()
	return self._curPeriodId or 100001;
end

function Lua_CollectionCabinetData.Get:curPeriodStartTime()
	return self._curPeriodStartTime or 0;
end

function Lua_CollectionCabinetData.Get:curPeriodEndTime()
	return self._curPeriodEndTime or 0;
end

function Lua_CollectionCabinetData.Get:nextPeriodId()
	return self._nextPeriodId or 100001;
end

function Lua_CollectionCabinetData.Get:nextPeriodStartTime()
	return self._nextPeriodStartTime or 0;
end

function Lua_CollectionCabinetData.Get:nexrPeriodEndTime()
	return self._nexrPeriodEndTime or 0;
end

function Lua_CollectionCabinetData:getDrawRewardInfo(poolType)
	if self.curShowDrawRewardDic == nil then
		self:initShowRewardInfo();
	end
	return self.curShowDrawRewardDic[poolType];
end

function Lua_CollectionCabinetData:getShowDrawRewardIdx(poolType, poolId)
	if self.curShowDrawRewardIdxDic == nil then
		self:initShowRewardInfo();
	end
	return self.curShowDrawRewardIdxDic[poolType][poolId];
end

function Lua_CollectionCabinetData:initShowRewardInfo()
	if self.curShowDrawRewardDic == nil then
		self.curShowDrawRewardDic = {};
	end
	if self.curShowDrawRewardIdxDic == nil then
		self.curShowDrawRewardIdxDic = {};
	end
	local config = DataCenter.collectionCabinetUtil:getCollectionCabinetPeriodConfig(self.curPeriodId);
	local arr = self:getShowRewardIdList(config.low_chock); -- 低级
	arr:sort(nil, false);
	self.curShowDrawRewardDic[1] = arr;
	self.curShowDrawRewardIdxDic[1] = {};
	for i = 1, arr:getSize() do
		self.curShowDrawRewardIdxDic[1][arr[i]] = i;
	end

	arr = self:getShowRewardIdList(config.chock); -- 高级
	arr:sort(nil, false);
	self.curShowDrawRewardDic[2] = arr;
	self.curShowDrawRewardIdxDic[2] = {};
	for i = 1, arr:getSize() do
		self.curShowDrawRewardIdxDic[2][arr[i]] = i;
	end
end

function Lua_CollectionCabinetData:getShowRewardIdList(poolId)
	local mainList = DataCenter.collectionCabinetUtil:getCollectionCabinetSubPoolConfig(poolId);
	local subList;
	local arr = Array();
	for i = 1, #mainList do
		subList = DataCenter.collectionCabinetUtil:getCollectionCabinetSubPoolConfig(mainList[i][1]);
		for j = 1, #subList do
			arr:add(subList[j][1]);
		end
	end
	return arr;
end

function Lua_CollectionCabinetData:checkCollectionDrawRedPoint(dic)
	if not DataCenter.funcOpenData:isOpened(10218) then
		return;
	end
	local lowMoreCostList = DataCenter.collectionCabinetUtil.moreLowDrawCostList;
	local moreCostList = DataCenter.collectionCabinetUtil.moreDrawCostList;
	if dic[lowMoreCostList[1]] or dic[moreCostList[1]] then
		self:updateCollectionDrawRed();
	end
end

function Lua_CollectionCabinetData:updateCollectionDrawRed()
	local costList = DataCenter.collectionCabinetUtil.moreLowDrawCostList;
	local hasNum = DataCenter.packData:getItemCountById(costList[1]);
	DataCenter.redPointData:updateRedPointState(Lua_RedPointList.NewCollectionCabinet_Draw_LowMoreDraw, hasNum >= costList[2]);

	costList = DataCenter.collectionCabinetUtil.moreDrawCostList;
	hasNum = DataCenter.packData:getItemCountById(costList[1]);
	DataCenter.redPointData:updateRedPointState(Lua_RedPointList.NewCollectionCabinet_Draw_MoreDraw, hasNum >= costList[2]);
end