Lua_CollectionCabinetVo = class("Lua_CollectionCabinetVo", Lua_ToolVo);
function Lua_CollectionCabinetVo:ctor(id, config)
	self.id = nil;
	self:setData(id, config);
end

function Lua_CollectionCabinetVo:dispose()
	self._starLv = nil;
	self._curStarId = nil;
	self._level = nil;
	self._curLvId = nil;
	self._isActive = nil;
	self._shareTimes = nil;
	self._isCanDrawActReward = nil;
	self._getTick = nil;
	self._growProgress = nil;

	self._activeCostIdArr = nil; -- 激活时消耗的激活材料id列表
	self._activeCostItemDic = nil; -- 激活时消耗的激活材料道具数据
	self._activeCostCollectChipNum = nil; -- 激活时消耗的藏品碎片数量
	self._tvo = nil;
	self:resetStarInfo();
	self:resetLvInfo();

	self:superCall("dispose");
end

function Lua_CollectionCabinetVo:resetStarInfo()
	self._curStarId = nil;
	self._curStarConfig = nil;
	self._starEntryConfig = nil;
end

function Lua_CollectionCabinetVo:resetLvInfo()
	self._curLvId = nil;
	self._curLevelConfig = nil;
end

function Lua_CollectionCabinetVo:setData(id, config)
	id = tonumber(id);
	if self.id ~= id then
		self.id = id;
		self.config = config;
		if self.config == nil then
			self.config = Lua_CollectionCabinetConfigData.getConfig(id);
		end
		if self.config == nil and id ~= 0 then
			LogManager.LogError("no new_collection_cabinet-新藏品柜表 id = " .. id, false);
		end
	end
end

function Lua_CollectionCabinetVo:fromServer(msg)
	self.starLv = msg.star_level; -- 星级
	self.level = msg.level; -- 等级
	self._isActive = msg.is_active == 1; -- 是否激活 0未激活 1激活
	self._shareTimes = msg.share_time; -- 分享次数
	self._isCanDrawActReward = msg.is_draw_active_reward == 0; -- 是否领取激活奖励 0可以领取 1不可领取
	self._getTick = msg.get_tick; -- 藏品获取时间
	self._growProgress = msg.grow_progress; -- 成长进度
	self._collectForce = msg.collection_forces; -- 藏品战力
end

function Lua_CollectionCabinetVo.Get:isActive()
	return self._isActive;
end

function Lua_CollectionCabinetVo.Get:shareTimes()
	return self._shareTimes or 0;
end

function Lua_CollectionCabinetVo.Get:isCanDrawActReward()
	return self._isCanDrawActReward;
end

function Lua_CollectionCabinetVo.Get:getTick()
	return self._getTick or 0;
end

function Lua_CollectionCabinetVo.Get:growProgress()
	return self._growProgress or 0;
end

function Lua_CollectionCabinetVo.Get:collectForce()
	return self._collectForce or 0;
end

-- 藏品星级信息
function Lua_CollectionCabinetVo.Set:starLv(value) 
	if self._starLv ~= value then
		self:resetStarInfo();
	end
	self._starLv = value;
end

function Lua_CollectionCabinetVo.Get:starLv()
	if self._starLv == nil or self._starLv == 0 then
		self._starLv = self.initialStar;
	end
	return self._starLv;
end

function Lua_CollectionCabinetVo.Get:curStarId()
	if self._curStarId == nil then
		self._curStarId = DataCenter.collectionCabinetUtil:getCollectionCabinetStarId(self.id, self.starLv);
	end
	return self._curStarId;
end

-- 藏品等级信息
function Lua_CollectionCabinetVo.Set:level(value) 
	if self._level ~= value then
		self:resetLvInfo();
	end
	self._level = value;
end

function Lua_CollectionCabinetVo.Get:level()
	if self._level == nil or self._level == 0 then
		self._level = self.initialLevel;
	end
	return self._level;
end

function Lua_CollectionCabinetVo.Get:curLvId()
	if self._curLvId == nil then
		self._curLvId = DataCenter.collectionCabinetUtil:getCollectionCabinetLvId(self.levelNormalType, self.level);
	end
	return self._curLvId;
end

function Lua_CollectionCabinetVo.Get:chipRedpoint()
	if self.isActive then -- 已激活
		return self.isCanUpStar; -- 可以升星
	else
		return self.isCanActive; -- 可以激活
	end
end

function Lua_CollectionCabinetVo.Get:isCanActive()
	if self.isActive then
		return false;
	end

	local hasNum;
	local _isCanActive = true;
	for k,v in pairs(self.activeCostItemDic) do
		hasNum = DataCenter.packData:getItemCountById(k);
		if hasNum < v then
			_isCanActive = false;
			break;
		end
	end
	return _isCanActive;
end

function Lua_CollectionCabinetVo.Get:isCanUpStar()
	local _isCanUpStar;
	if not self.isActive or self.isMaxStar then
		_isCanUpStar = false;
	else
		_isCanUpStar = true;
		if self.upStarCostList and #self.upStarCostList > 0 then
			local hasNum;
			for i = 1, #self.upStarCostList do
				hasNum = DataCenter.packData:getItemCountById(self.upStarCostList[i][1]);
				if hasNum < self.upStarCostList[i][2] then
					_isCanUpStar = false;
					break;
				end
			end
		end
	end
	return _isCanUpStar;
end

function Lua_CollectionCabinetVo.Get:isCanUpLv()
	local _isCanUpLv;
	if not self.isActive or self.isCurMaxLv then
		_isCanUpLv = false;
	else
		_isCanUpLv = true;
		if self.upLvCostList and #self.upLvCostList > 0 then
			local hasNum;
			for i = 1, #self.upLvCostList do
				hasNum = DataCenter.packData:getItemCountById(self.upLvCostList[i][1]);
				if hasNum < self.upLvCostList[i][2] then
					_isCanUpLv = false;
					break;
				end
			end
		end
	end
	return _isCanUpLv;
end

function Lua_CollectionCabinetVo.Get:isMax()
	return self.isMaxStar and isCurMaxLv;
end

function Lua_CollectionCabinetVo.Get:isMaxStar()
	return self.starLv >= self.maxStar;
end

function Lua_CollectionCabinetVo.Get:isCurMaxLv()
	return self.level >= self.curMaxLv;
end

function Lua_CollectionCabinetVo.Get:isMaxLv()
	return self.level >= self.maxLevel;
end

function Lua_CollectionCabinetVo:initActiveMaterialData()
	if self._activeCostIdArr == nil then
		local tvo = getConfig("item", self.chipId);
		self._activeCostIdArr = Array();
		self._activeCostItemDic = {};
		if tvo ~= nil and tvo.compound then -- 激活时消耗的本体碎片
			local num = tvo.compound[2] or 0;
			self._activeCostCollectChipNum = num;
			self._activeCostIdArr:add(self.chipId);
			self._activeCostItemDic[self.chipId] = num;
		end
		-- if tvo ~= nil and tvo.consume_item then -- 激活时消耗的其他道具材料
		-- 	local obj;
		-- 	for i = 1, #tvo.consume_item do
		-- 		obj = tvo.consume_item[i];
		-- 		if self._activeCostIdArr:indexOf(obj[1]) == -1 then
		-- 			self._activeCostIdArr:add(obj[1]);
		-- 		end
		-- 		if self._activeCostItemDic[obj[1]] == nil then
		-- 			self._activeCostItemDic[obj[1]] = obj[2];
		-- 		else
		-- 			self._activeCostItemDic[obj[1]] = self._activeCostItemDic[obj[1]] + obj[2];
		-- 		end
		-- 	end
		-- end
	end
end

function Lua_CollectionCabinetVo.Get:activeCostIdArr()
	if self._activeCostIdArr == nil then
		self:initActiveMaterialData();
	end
	return self._activeCostIdArr;
end

function Lua_CollectionCabinetVo.Get:activeCostItemDic()
	if self._activeCostItemDic == nil then
		self:initActiveMaterialData();
	end
	return self._activeCostItemDic;
end

function Lua_CollectionCabinetVo.Get:activeCostCollectChipNum()
	if self._activeCostCollectChipNum == nil then
		self:initActiveMaterialData();
	end
	return self._activeCostCollectChipNum or 0;
end

function Lua_CollectionCabinetVo.Get:showScore()
	return self.score + self.curStarScore + self.curLvScore;
end

function Lua_CollectionCabinetVo.Get:tvo()
	if self._tvo == nil then
		self._tvo = Lua_ToolVo.createToolVo(self.itemId);
	end
	return self._tvo;
end










---------------------------------------- 藏品配置信息 ----------------------------------------
-- 藏品名称
function Lua_CollectionCabinetVo.Get:name()
	return self.config.name;
end

-- 显示顺序
function Lua_CollectionCabinetVo.Get:sort()
	return self.config.sort;
end

-- 本体道具id
function Lua_CollectionCabinetVo.Get:itemId()
	return self.config.item_id;
end

-- 初始星级
function Lua_CollectionCabinetVo.Get:initialStar()
	return self.config.initial_star;
end

-- 最大星级
function Lua_CollectionCabinetVo.Get:maxStar()
	return self.config.max_star;
end

-- 初始等级
function Lua_CollectionCabinetVo.Get:initialLevel()
	return self.config.initial_level;
end

-- 最大等级
function Lua_CollectionCabinetVo.Get:maxLevel()
	return self.config.max_level;
end

-- 最大等级
function Lua_CollectionCabinetVo.Get:levelNormalType()
	return self.config.level_normal_type;
end

-- 颜色
function Lua_CollectionCabinetVo.Get:color()
	return self.config.color;
end

-- 重复获得本体时转换道具
function Lua_CollectionCabinetVo.Get:transferTtem()
	return self.config.transfer_item;
end

-- 藏品碎片id
function Lua_CollectionCabinetVo.Get:chipId()
	return self.transferTtem[1] or 0;
end

-- 基础评分
function Lua_CollectionCabinetVo.Get:score()
	return self.config.score;
end

-- 引用图集
function Lua_CollectionCabinetVo.Get:atlas()
	return self.config.atlas;
end

-- 引用图标
function Lua_CollectionCabinetVo.Get:sprite()
	return self.config.sprite;
end

-- 图鉴
function Lua_CollectionCabinetVo.Get:atlasImg()
	return self.config.atlas_img;
end

-- 传记标题
function Lua_CollectionCabinetVo.Get:descTitle()
	return self.config.desc_title;
end

-- 传记内容
function Lua_CollectionCabinetVo.Get:descContent()
	return self.config.desc_content;
end










---------------------------------------- 藏品星级配置信息 ----------------------------------------
function Lua_CollectionCabinetVo.Get:curStarConfig()
	if self.curStarId == nil then
		return nil;
	end
	if self._curStarConfig == nil then
		self._curStarConfig = DataCenter.collectionCabinetUtil:getCollectionCabinetStarConfig(self.curStarId);
	end
	return self._curStarConfig;
end

-- 当前星级升星消耗
function Lua_CollectionCabinetVo.Get:upStarCostList()
	if self.curStarConfig == nil or self.curStarConfig.cost_item == nil then
		return nil;
	end
	return self.curStarConfig.cost_item;
end

-- 当前星级可以升的最大等级
function Lua_CollectionCabinetVo.Get:curMaxLv()
	if self.curStarConfig == nil or self.curStarConfig.max_level == nil then
		return 0;
	end
	return self.curStarConfig.max_level;
end

-- 当前星级对应评分
function Lua_CollectionCabinetVo.Get:curStarScore()
	if self.curStarConfig == nil or self.curStarConfig.score == nil then
		return 0;
	end
	return self.curStarConfig.score;
end

-- 当前星级对应属性
function Lua_CollectionCabinetVo.Get:starNormalProp()
	if self.curStarConfig == nil or self.curStarConfig.normal_prop == nil then
		return {};
	end
	return self.curStarConfig.normal_prop;
end

-- 当前星级对应词条id
function Lua_CollectionCabinetVo.Get:specialNatureId()
	if self.curStarConfig == nil or self.curStarConfig.special_nature_id == nil then
		return nil;
	end
	return self.curStarConfig.special_nature_id;
end










---------------------------------------- 当前星级词条配置信息 ----------------------------------------
function Lua_CollectionCabinetVo.Get:starEntryConfig()
	if self.specialNatureId == nil then
		return nil;
	end
	if self._starEntryConfig == nil then
		self._starEntryConfig = DataCenter.collectionCabinetUtil:getCollectionCabinetEntryConfig(self.specialNatureId);
	end
	return self._starEntryConfig;
end

-- 当前星级对应属性类型
function Lua_CollectionCabinetVo.Get:show_entry_type()
	if self.starEntryConfig == nil or self.starEntryConfig.show_entry_type == nil then
		return nil;
	end
	return self.starEntryConfig.show_entry_type;
end

-- 当前星级成长属性描述
function Lua_CollectionCabinetVo.Get:title_desc()
	if self.starEntryConfig == nil or self.starEntryConfig.title_desc == nil then
		return "";
	end
	return self.starEntryConfig.title_desc;
end

-- 当前星级弹窗属性描述
function Lua_CollectionCabinetVo.Get:popup_desc()
	if self.starEntryConfig == nil or self.starEntryConfig.popup_desc == nil then
		return "";
	end
	return self.starEntryConfig.popup_desc;
end

-- 当前星级对应支持阵营
function Lua_CollectionCabinetVo.Get:hero_camp()
	if self.starEntryConfig == nil or self.starEntryConfig.hero_camp == nil then
		return 0;
	end
	return self.starEntryConfig.hero_camp;
end

-- 当前星级单位成长属性
function Lua_CollectionCabinetVo.Get:prop()
	if self.starEntryConfig == nil or self.starEntryConfig.prop == nil then
		return {};
	end
	return self.starEntryConfig.prop;
end

-- 当前星级单位进度
function Lua_CollectionCabinetVo.Get:each()
	if self.starEntryConfig == nil or self.starEntryConfig.each == nil then
		return 0;
	end
	return self.starEntryConfig.each;
end

-- 当前星级对应成长条件
function Lua_CollectionCabinetVo.Get:grow_cond_id()
	if self.starEntryConfig == nil or self.starEntryConfig.grow_cond_id == nil then
		return nil;
	end
	return self.starEntryConfig.grow_cond_id;
end

-- 当前星级特殊属性描述
function Lua_CollectionCabinetVo.Get:other_desc()
	if self.starEntryConfig == nil or self.starEntryConfig.other_desc == nil then
		return "";
	end
	return self.starEntryConfig.other_desc;
end

-- 当前星级特殊属性 -- 策划要求：不显示
function Lua_CollectionCabinetVo.Get:other_prop()
	if self.starEntryConfig == nil or self.starEntryConfig.other_prop == nil then
		return {};
	end
	return self.starEntryConfig.other_prop;
end










---------------------------------------- 藏品等级配置信息 ----------------------------------------
function Lua_CollectionCabinetVo.Get:curLevelConfig()
	if self.curLvId == nil then
		return nil;
	end
	if self._curLevelConfig == nil then
		self._curLevelConfig = DataCenter.collectionCabinetUtil:getCollectionCabinetLvConfig(self.curLvId);
	end
	return self._curLevelConfig;
end

-- 当前等级升級消耗
function Lua_CollectionCabinetVo.Get:upLvCostList()
	if self.curLevelConfig == nil or self.curLevelConfig.cost_item == nil then
		return nil;
	end
	return self.curLevelConfig.cost_item;
end

-- 当前等级对应评分
function Lua_CollectionCabinetVo.Get:curLvScore()
	if self.curLevelConfig == nil or self.curLevelConfig.score == nil then
		return 0;
	end
	return self.curLevelConfig.score;
end

-- 当前等级对应属性
function Lua_CollectionCabinetVo.Get:lvNormalProp()
	if self.curLevelConfig == nil or self.curLevelConfig.normal_prop == nil then
		return {};
	end
	return self.curLevelConfig.normal_prop;
end

-- 当前等级对应属性描述
function Lua_CollectionCabinetVo.Get:levelDes()
	if self.curLevelConfig == nil or self.curLevelConfig.level_des == nil then
		return "";
	end
	return self.curLevelConfig.level_des;
end