Lua_ToolVo = class("Lua_ToolVo");
function Lua_ToolVo.createToolVo(id, count)
    if id == nil then
        return nil;
    end

    id = tonumber(id);
    if id <= 0 then
        return nil;
    end

    local config = Lua_StrangeBeastConfigData.getConfig(id);
    if config then
        return Lua_StrangeBeastVo.new(id, config);
    end
    
    config = getConfig("item", id);
    if config ~= nil then
        if config.main_type == Lua_PackDefine.EQUIP then
            return Lua_EquipVo.new(id, count);
        elseif config.main_type == Lua_PackDefine.RUNES then
            return Lua_RunesVo.new(id, count);
        -- elseif config.main_type == Lua_PackDefine.COLLECTION then
        --     return Lua_CollectionVo.new(id, count);
        elseif config.main_type == Lua_PackDefine.TACTIC then
            return Lua_HeroTacticVo.new(id, count, config);
        else
            return Lua_ToolVo.new(id, count, config);
        end
    end

    config = Lua_CollectionCabinetConfigData.getConfig(id);
    if config then
        return Lua_CollectionCabinetVo.new(id, config);
    end

    config = Lua_HeroConfigData.getConfig(id);
    if config ~= nil then
        return Lua_HeroVo.new(id, config);
    end

    LogManager.LogError("no item/hero config id : " .. id, false);
    return nil;
end

function Lua_ToolVo:ctor(id, count, config)
    self.id = nil;
    self:setData(id, count, config);
end

function Lua_ToolVo:setData(id, count, config)
    id = tonumber(id);
    if self.id ~= id then
        self.id = id;

        self.config = config;
        if self.config == nil then
            self.config = getConfig("item", id);
        end

        if self.config == nil then
            self.config = Lua_HeroConfigData.getConfig(id);
        end

        if self.config == nil then
            self.config = Lua_StrangeBeastConfigData.getConfig(id);
        end

        if self.config == nil then
            self.config = Lua_CollectionCabinetConfigData.getConfig(id);
        end
        
        if self.config == nil then
            LogManager.LogError("no config item/hero id " .. id, false);
        end
    end
    self.count = count;
end

function Lua_ToolVo:fromServer(msg)
    self.index = msg.index; -- 背包索引，从1开始
    self.unique_id = tonumber(msg.unique_id); -- 唯一id
    self._count = msg.amount; -- 道具id
    self.bind = msg.bind; -- 绑定：0 不绑定，1绑定
    self._expired_tick = tonumber(msg.expired_tick); -- 过期时间 0:永久时效
    self.show_time = msg.show_time; -- 展示倍数
    self.effect_type = msg.effect_type; -- 特效类型

    -- 图腾
    self.totem_info = msg.totem_info;
    if self.main_type == Lua_PackDefine.TOTEM and msg.totem_info then
        self._strengthenLv = msg.totem_info.level; -- 强化等级
        self._score = msg.totem_info.score; -- 评分
    end

    self:reset();
end

function Lua_ToolVo:reset()
    self._sortWeight = nil;
    self._chipSort = nil;
    self._totemSort = nil;
    self._iconEffUrl = nil;
end

function Lua_ToolVo.Set:count(value)
    self._count = value;
    self:reset();
end

function Lua_ToolVo.Get:count()
    return self._count or 0;
end

function Lua_ToolVo.Set:level(value)
    self._level = value;
end

function Lua_ToolVo.Get:level()
    return self._level == nil and 0 or self._level;
end

function Lua_ToolVo.Get:colorName()
    return Lua_ColorDefine.getColorStr(self.colorStr, self.name);
end

function Lua_ToolVo.Get:colorStr()
    return Lua_ColorDefine.getItemColorStr(self.color);
end

function Lua_ToolVo.Get:broadColorStr()
    return Lua_ColorDefine.getBroadcastColorStr(self.color);
end

function Lua_ToolVo.Get:broadcastName()
    return Lua_ColorDefine.getColorStr(self.broadColorStr, self.name);
end

-- 阵营图标
function Lua_ToolVo.Get:campSprite()
    return "xui_common_attribute_"..(29 + self.campType);
end

-- 是否是英雄碎片
function Lua_ToolVo.Get:isHeroChip() 
    if self.main_type == Lua_PackDefine.HERO_CHIP_RANDOM or self.main_type == Lua_PackDefine.HERO_CHIP_FIXED then
        return true;
    end
    return false;
end

-- 是否是英雄完整道具（英雄做成道具展示，例如英雄礼包）
function Lua_ToolVo.Get:isIntactHero() 
    if self.main_type == Lua_PackDefine.HERO then
        return true;
    end
    return false;
end

function Lua_ToolVo.Get:expired_tick()
    return self._expired_tick or 0
end

-- 是否过期
function Lua_ToolVo.Get:isExpired()
    if self.expired_tick == 0 or self.expired_tick > DataCenter.roleData.serverTime then
        return false;
    end
    return true;
end

function Lua_ToolVo.Set:isNew(value)
    self._isNew = value;
end

function Lua_ToolVo.Get:isNew()
    return self._isNew or 0;
end

-- 强化等级
function Lua_ToolVo.Get:strengthenLv()
    return self._strengthenLv or 0;
end

function Lua_ToolVo.Get:score()
    if self._score then
        return self._score;
    end
    return self.config.score or 0; -- 配置基础评分
end

-- 碎片合成id
function Lua_ToolVo.Get:synthesisId()
    if self.compound == nil then
        return 0;
    end
    return self.compound[1] or 0;
end

-- 碎片合成数量
function Lua_ToolVo.Get:synthesisNum()
    if self.compound == nil then
        return 0;
    end
    return self.compound[2] or 0;
end

-- 碎片上限数量
function Lua_ToolVo.Get:chipLimitNum()
    if self.pack_type == Lua_PackDefine.TYPE_CHIP then
        if self.artifactId > 0 then
            return DataCenter.artifactData:getChipLimitNum(self.artifactId);
        end
        if self.beastId > 0 then
            return DataCenter.strangeBeastData:getDebrisMergeNum(self.beastId);
        end
        if self.collectionCabinetId > 0 then
            -- return DataCenter.collectionCabinetData:getCurChipNumByCollectId(self.collectionCabinetId);
            return 0; -- 策划要求不显示
        end
        if self.main_type == Lua_PackDefine.TACTIC_CHIP then
            return DataCenter.heroTacticUtil:getChipLimitNum(self.synthesisId, self.synthesisNum);
        end
        if self.synthesisId > 0 and self.synthesisNum > 0 then
            return self.synthesisNum; -- 碎片合成
        end
    end
    return 0;
end

------------------------------ 配置数据 ------------------------------

function Lua_ToolVo.Get:main_type()
    return self.config.main_type or 0;
end

function Lua_ToolVo.Get:sub_type()
    return self.config.sub_type or 0;
end

function Lua_ToolVo.Get:pack_type()
    if self._pack_type == nil then
        return self.config.type;
    end
    return self._pack_type;
end

function Lua_ToolVo.Set:pack_type(value)
    self._pack_type = value;
end

function Lua_ToolVo.Get:name()
    return self.config.name;
end

function Lua_ToolVo.Get:color()
    if self._color == nil then
        if self.config.color == nil then
            return 1;
        end
        return self.config.color;
    end
    return self._color;
end

function Lua_ToolVo.Set:color(value)
    self._color = value;
end

-- 可叠加数量
function Lua_ToolVo.Get:overlap()
    return self.config.overlap or 1;
end

-- 物品限制次数
function Lua_ToolVo.Get:limit_times()
    return self.config.limit_times;
end

function Lua_ToolVo.Get:atlas()
    return self.config.atlas;
end

function Lua_ToolVo.Get:sprite()
    local sprite = self.config.sprite;
    if strIsNullOrEmpty(sprite) then
        sprite = tostring(self.id);
    end
    return sprite;
end

function Lua_ToolVo.Get:des()
    return self.config.des;
end

-- 出售价格
function Lua_ToolVo.Get:sys_price()
    return self.config.sys_price or 0;
end

-- 出售货币
function Lua_ToolVo.Get:money_id()
    return self.config.money_id or 0;
end

function Lua_ToolVo.Get:use_type()
    return self.config.use_type or 1;
end

-- 使用效果参数
function Lua_ToolVo.Get:effect_param()
    return self.config.effect_parameter;
end

-- 是否批量使用
function Lua_ToolVo.Get:isBatch()
    return self.config.isBatch or 0;
end

-- 作用效果
function Lua_ToolVo.Get:effect_des()
    return self.config.effect_des;
end

function Lua_ToolVo.Get:buttonList()
    return self.config.buttonList;
end

-- 是否显示查看来源
function Lua_ToolVo.Get:showPath()
    return self.config.showPath or 0;
end

-- 合成获得材料(获得道具，消耗数量)
function Lua_ToolVo.Get:compound()
    return self.config.compound;
end

-- 分解获得材料
function Lua_ToolVo.Get:decompose()
    return self.config.decompose;
end

-- 提供经验
function Lua_ToolVo.Get:provideExp()
    return self.config.provideExp or 0;
end

--强化石消耗需要的费用
function Lua_ToolVo.Get:consume_item()
    return self.config.consume_item;
end

-- 出售提示框
function Lua_ToolVo.Get:sell_tips()
    return self.config.sell_tips;
end

-- 获取途径
function Lua_ToolVo.Get:obtainPath()
    local list = self.config.obtain_path;
    if list ~= nil then
        local arr = Array();
        local config;
        for _, v in ipairs(list) do
            config = getConfig("obtain_path", v[1]);
            if config.type == 1 then
                for _, y in ipairs(config.use_item) do
                    local data = DataCenter.packData:getItemListById(y);
                    if data and data:getSize() > 0 then
                        arr:add(v);
                        break;
                    end
                end
            elseif config.type == 2 then
                local vo;
                if config.gift_type == 1 then
                    vo = DataCenter.dailyGiftData:getDailyGiftVo(config.gift_id);
                    if vo and vo.curLimitTimes > 0 then
                        arr:add(v);
                    end
                elseif config.gift_type == 2 then
                    vo = DataCenter.weeklyGiftData:getWeeklyGiftVo(config.gift_id);
                    if vo and vo.curLimitTimes > 0 then
                        arr:add(v);
                    end
                elseif config.gift_type == 3 then
                    vo = DataCenter.monthlyGiftData:getMonthlyGiftVo(config.gift_id);
                    if vo and vo.curLimitTimes > 0 then
                        arr:add(v);
                    end
                end
            end
        end
        return arr;
    end
    return nil;
end

-- 功能途径
function Lua_ToolVo.Get:funcPath()
    if self._funcPath == nil then
        local list = self.config.func_path;
        if list ~= nil then
            self._funcPath = Array();
            for _, v in ipairs(list) do
                if v[1] == 1024 then
                    -- 版署 屏蔽充值
                    if not Lua_PlatformUtil.IsBanShu() then
                        self._funcPath:add(v);
                    end
                else
                    self._funcPath:add(v);
                end
            end
        end
    end
    return self._funcPath;
end

-- 阵营类型
function Lua_ToolVo.Get:campType()
    return self.config.camp or 0;
end

-- 星
function Lua_ToolVo.Get:star()
    if self._star == nil then
        if self.config.star == nil then
            return 0;
        else
            return self.config.star;
        end
    end
    return self._star;
end

function Lua_ToolVo.Set:star(value)
    self._star = value;
end

function Lua_ToolVo.Get:quality()
    if self._quality == nil then
        self._quality = self.config.quality;
    end
    return self._quality;
end

function Lua_ToolVo.Set:quality(value)
    self._quality = value;
end

-- 展示星级数量
-- function Lua_ToolVo.Get:showStar()
--     local star = self.star;
--     if star ~= nil and star > 0 then
--         if star > 10 then
--             return star - 10;
--         elseif star <= 10 and star > 5 then
--             return star - 5;
--         elseif star <= 5 then
--             return star;
--         end
--     end
--     return 0;
-- end

function Lua_ToolVo.Get:iconEffUrl()
    if self._iconEffUrl == nil then
        self._iconEffUrl = LanUtil.StringProfile("/effects/uiEffects/", self.iconEff);
    end
    return self._iconEffUrl;
end

function Lua_ToolVo.Get:iconEff()
    if self.effect_type == 1 then -- 扭蛋机
        return "ui_ndj_daoju02";
    elseif self.effect_type == 2 then -- 若抽到已有异兽本体转化为碎片时,显示的特效类型ProtoItem：effect_type
        return "pet_item";
    end
    return self.config.icon_effect;
end

function Lua_ToolVo.Get:buy_money_id()
    return self.config.buy_money_id;
end

function Lua_ToolVo.Get:buy_price()
    return self.config.buy_price;
end

-- 客户端显示物品列表 礼包
function Lua_ToolVo.Get:client_goods_list()
    return self.config.client_goods_list;
end

-- 使用跳转id
function Lua_ToolVo.Get:func_path_id()
    return self.config.func_path_id;
end

function Lua_ToolVo.Get:clothes_id()
    return self.config.clothes_id or 0;
end

-- 战技id
function Lua_ToolVo.Get:talent_id()
    return self.config.talent_id;
end

-- 自选奖励数量
function Lua_ToolVo.Get:option_amount()
    return self.config.option_amount or 1;
end

-- 使用标识
function Lua_ToolVo.Get:use_logo()
    return self.config.use_logo or 0;
end

-- 一键分解标识
function Lua_ToolVo.Get:decompose_logo()
    return self.config.decompose_logo or 0;
end

-- 一键出售标识
function Lua_ToolVo.Get:sell_logo()
    return self.config.sell_logo or 0;
end

-- 排序
function Lua_ToolVo.Get:sort_weight()
    return self.config.sort_weight or 0;
end

-- 英雄id
function Lua_ToolVo.Get:hero_id()
    return self.config.hero_id or 0;
end

-- 使用单位消耗道具数量
function Lua_ToolVo.Get:eachAmount()
    return self.config.each_amount or 0;
end

-- 从道具表中获取英雄圆形图集和图片（非英雄直接使用atlas、sprite）
function Lua_ToolVo:getAvatarFromItemTable()
    if self.main_type == Lua_PackDefine.HERO then
        local config = Lua_HeroConfigData.getConfig(self.hero_id);
        if config then
            return config.avatar_atlas_1, config.avatar_sprite_1;
        end
    end
    return self.atlas, self.sprite;
end

function Lua_ToolVo.Get:showTimeText()
    return self.config.time_text
end

function Lua_ToolVo.Get:agingType()
    return self.config.aging_type or 0
end

-- 部位
function Lua_ToolVo.Get:part()
    return self.config.part or 1;
end

-- 作为强化材料时，提供的基础经验
function Lua_ToolVo.Get:addExp()
    return self.config.add_exp or 0;
end

function Lua_ToolVo.Get:iconAtlas()
    return self.config.icon_atlas or self.atlas;
end

function Lua_ToolVo.Get:iconSprite()
    return self.config.icon_sprite or self.sprite;
end

function Lua_ToolVo.Get:iconWidth()
    return self.config.icon_width or 0;
end

function Lua_ToolVo.Get:iconHeight()
    return self.config.icon_height or 0;
end

-- 孵化时间:异兽
function Lua_ToolVo.Get:hatch_time()
    return self.config.hatch_time or 0;
end

function Lua_ToolVo.Get:buy_limit()
    return self.config.buy_limit;
end

function Lua_ToolVo.Get:artifactId()
    return self.config.artifact_id or 0;
end

function Lua_ToolVo.Get:beastId()
    return self.config.beast_id or 0;
end

function Lua_ToolVo.Get:collectionCabinetId()
    return self.config.collection_cabinet_id or 0;
end






------------------------------ 藏宝柜 ------------------------------
-- 附加的词条列表
function Lua_ToolVo.Get:entry_list()
    return self.config.entry_list;
end

-- 藏品激活奖励
function Lua_ToolVo.Get:active_reward()
    return self.config.active_reward;
end

-- 任务描述
function Lua_ToolVo.Get:task_des()
    return self.config.task_des;
end

-- 藏品获取目标值
function Lua_ToolVo.Get:target_value()
    return self.config.target_value;
end

-- 藏品获取目标值
function Lua_ToolVo.Get:complete_path_id()
    return self.config.complete_path_id;
end






------------------------------ 钓鱼 ------------------------------
-- 简介
function Lua_ToolVo.Get:intro()
    return self.config.intro;
end

-- 产出地
function Lua_ToolVo.Get:fishery_output()
    return self.config.fishery_output;
end

-- 属性加成
function Lua_ToolVo.Get:add_prop()
    return self.config.add_prop;
end

-- 长度(cm)
function Lua_ToolVo.Get:length()
    return self.config.length or 0;
end

-- 最大长度(cm)
function Lua_ToolVo.Get:max_length()
    return self.config.max_length or 0;
end

-- 经验值
function Lua_ToolVo.Get:exp()
    return self.config.exp;
end










------------------------------ 图腾 ------------------------------
-- 套装激活所需数量 n件同类型不同位置
function Lua_ToolVo.Get:suitAmount()
    return self.config.suit_amount or 0;
end

-- 套装id
function Lua_ToolVo.Get:suitId()
    return self.config.suit_id or 0;
end

function Lua_ToolVo.Get:suitList()
    if self._suitList == nil then
        self._suitList = Array();
        local config;
        local suitId = self.suitId;
        while true do
            suitId = suitId - 1;
            config = DataCenter.totemUtil:getSuitConfig(suitId);
            if config == nil then
                break;
            else
                self._suitList:add(suitId);
            end
        end

        suitId = self.suitId;
        self._suitList:add(suitId);

        while true do
            suitId = suitId + 1;
            config = DataCenter.totemUtil:getSuitConfig(suitId);
            if config == nil then
                break;
            else
                self._suitList:add(suitId);
            end
        end

        self._suitList:sort(nil, false);
    end
    return self._suitList;
end










------------------------------ 红点处理 ------------------------------
-- 碎片是否可以合成
function Lua_ToolVo.Get:canSynthesis()
    if self.chipLimitNum > 0 and self.count >= self.chipLimitNum then
        if self.artifactId > 0 then
            local vo = DataCenter.artifactData:getArtifactVoById(self.artifactId);
            return vo.canUnlock or vo.canUpStar;
        end
        if self.beastId > 0 then
            local vo = DataCenter.strangeBeastData:getStrangeBeastVoById(self.beastId);
            return vo.show_redpoint;
        end
        if self.main_type == Lua_PackDefine.TACTIC_CHIP then
            local vo = DataCenter.packData:getItemVoById(self.synthesisId);
            if vo ~= nil then
                return vo.isUpstar;
            end
        end
        return true;
    end

    -- 策划要求不显示
    -- if self.collectionCabinetId > 0 then -- 藏品升星有材料升星和碎片升星
    --     local vo = DataCenter.collectionCabinetData:getCollectionCabinetVo(self.collectionCabinetId);
    --     return vo and vo.chipRedpoint;
    -- end
    return false;
end

-- 自选碎片礼包可使用红点
function Lua_ToolVo.Get:isShowGiftBagUseRed()
    if self.main_type == Lua_PackDefine.GIFT_BAG and self.sub_type == Lua_PackDefine.GIFT_BAG_SELECT then
        return self.count >= self.eachAmount;
    end
    return false;
end










------------------------------ 排序处理 ------------------------------
-- 排序权重
function Lua_ToolVo.Get:sortWeight()
    if self._sortWeight == nil then
        local weight = self.sort_weight * 10000 + self.count; -- 数量最大9999
        if self.use_logo == 1 then
            self._sortWeight = weight;
        else
            self._sortWeight = weight + 1000000000;
        end
    end
    return self._sortWeight;
end

-- 碎片排序
function Lua_ToolVo.Get:chipSort()
    if self.canSynthesis then
        return self.sort_weight;
    end
    return self.sort_weight + 10000000000;
end

-- 图腾排序
function Lua_ToolVo.Get:totemSort()
    if self._totemSort == nil then
        self._totemSort = (10 - self.star) * 10000000000 + (100 - self.strengthenLv) * 100000000 + (self.sort_weight * 100000) + (100000 - self.score);
    end
    return self._totemSort;
end