-- 异兽孵化巢
Lua_StrangeBeastHatchNestVo = class("Lua_StrangeBeastHatchNestVo");
function Lua_StrangeBeastHatchNestVo:ctor(id, config)
	self.id = id;
	self.config = config;
	if self.config == nil then
		self.config = getConfig("strange_beasts_hatch_nest", id);
	end

	if self.config == nil then
		LogManager.LogError("no strange_beasts_hatch_nest-孵化巢配置 id = " .. id, false);
	end
end

function Lua_StrangeBeastHatchNestVo:fromServer(serverData)
	if serverData == nil then
		return;
	end

	self.unlockState = serverData.state;
	self:updateNestInfo(serverData.egg_list);
end

function Lua_StrangeBeastHatchNestVo:dispose()
	self.unlockState = nil;
	if self._nestInfo ~= nil then
		for i = 1, #self._nestInfo do
			self._nestInfo[i] = nil;
		end
		self._nestInfo = nil;
	end
end

function Lua_StrangeBeastHatchNestVo:updateNestInfo(nestInfo)
	if self._nestInfo == nil then
		self._nestInfo = {};
	end

	local lastLen = #self._nestInfo;
	local curLen = nestInfo and #nestInfo or 0;
	for i = 1, curLen do
		local param = self._nestInfo[i];
		if param == nil then
			param = {eggId = tonumber(nestInfo[i].key) or 0, hatchTime = tonumber(nestInfo[i].value) or 0, hatchNum = tonumber(nestInfo[i].sub) or 0 };
			self._nestInfo[i] = param;
		else
			param.eggId = tonumber(nestInfo[i].key) or 0;
			param.hatchTime = tonumber(nestInfo[i].value) or 0;
			param.hatchNum = tonumber(nestInfo[i].sub) or 0;
		end
	end

	if lastLen > curLen then
		for i = curLen + 1, lastLen do
			self._nestInfo[i] = nil;
		end
	end
end

function Lua_StrangeBeastHatchNestVo.Get:bOpened()
	return self.unlockState == 1;
end

function Lua_StrangeBeastHatchNestVo:getNestInfoByIndex(index)
	if self._nestInfo == nil or index == nil then
		return;
	end

	return self._nestInfo[index];
end

function Lua_StrangeBeastHatchNestVo:getNestEggIdByIndex(index)
	local eggId = 0;
	local info = self:getNestInfoByIndex(index);
	if info then
		eggId = info.eggId;
	end

	return eggId;
end

function Lua_StrangeBeastHatchNestVo:getHatchTimeByIndex(index)
	local hatchTime = 0;
	local info = self:getNestInfoByIndex(index);
	if info then
		hatchTime = info.hatchTime;
	end

	return hatchTime;
end

function Lua_StrangeBeastHatchNestVo:getResidueTimeByIndex(index)
	local residueTime = 0;
	local info = self:getNestInfoByIndex(index);
	if info then
		residueTime = info.hatchTime;
		residueTime = residueTime - DataCenter.roleData.serverTime;
	end

	return residueTime;
end

function Lua_StrangeBeastHatchNestVo:checkReceiveByIndex(index)
	local bReceive = false;
	local eggId = self:getNestEggIdByIndex(index);
	local residueTime = self:getResidueTimeByIndex(index);
	if eggId > 0 and residueTime <= 0 then
		bReceive = true;
	end

	return bReceive;
end

function Lua_StrangeBeastHatchNestVo:getAccelateCostInfoByIndex(index)
	local itemId, num;
	local eggId = self:getNestEggIdByIndex(index);
	local residueTime = self:getResidueTimeByIndex(index);
	if eggId > 0 and residueTime > 0 then
		local cfg = self.quicken_cost;
		local param1 = cfg and cfg[2] or 0;
		local param2 = cfg and cfg[3] or 0;
		itemId = cfg and cfg[1] or 0;
		num = Mathf.Ceil(residueTime / param1 * param2);
	end

	return itemId, num;
end

function Lua_StrangeBeastHatchNestVo.Get:index()
	return self.id;
end

function Lua_StrangeBeastHatchNestVo.Get:nestInfo()
	if self._nestInfo == nil then
		self._nestInfo = {};
	end

	return self._nestInfo;
end

function Lua_StrangeBeastHatchNestVo.Get:can_hatch()
	if self.bOpened ~= true then
		return false;
	end

	if self._nestInfo ~= nil and #self._nestInfo <= 0 then
		return true;
	end

	return false;
end

function Lua_StrangeBeastHatchNestVo.Get:can_receive()
	if self.bOpened ~= true then
		return false;
	end

	if self._nestInfo == nil or (self._nestInfo ~= nil and #self._nestInfo <= 0) then
		return false;
	end

	local bReceive = self:checkReceiveByIndex(1);
	return bReceive;
end

function Lua_StrangeBeastHatchNestVo.Get:bHatching()
	if self.bOpened ~= true then
		return false;
	end

	if self._nestInfo ~= nil and #self._nestInfo > 0 then
		return true;
	end

	return false;
end















-----strange_beasts_hatch_nest-孵化巢配置----------
--孵化所需时间比例
function Lua_StrangeBeastHatchNestVo.Get:time_ratio()
	if self.config then
		return self.config.time_ratio;
	end
end

--加速消耗资源
function Lua_StrangeBeastHatchNestVo.Get:quicken_cost()
	if self.config then
		return self.config.quicken_cost;
	end
end

--解锁描述
function Lua_StrangeBeastHatchNestVo.Get:unlock_desc()
	if self.config then
		return self.config.unlock_desc;
	end
end