-- 任务
checkRequire("Lua_TaskActInfoVo");
checkRequire("Lua_TaskInfoVo");
checkRequire("Lua_ActiveWarVo");
Lua_TaskData = class("Lua_TaskData");
Lua_TaskData.dailyTask = 1; -- 每日任务
Lua_TaskData.weeklyTask = 2; -- 周常任务
Lua_TaskData.achievementTaskTask = 3; -- 成就任务
Lua_TaskData.activeWarTask = 4; -- 活跃战令
-- 冒险之旅
Lua_TaskData.AdventureJourney_Task = 1; -- 冒险之旅任务
function Lua_TaskData:ctor()
	-- 任务
	self.needSortDic = {};
	self.activePointVoDic = {};
	self.singelTaskVoDic = {};
	self.recordDataArr = Array();
	self.lastReceivedTime = 0; -- 上次领取时间
	self.receiveDeltaTime = 0.5; -- 领取间隔时间

	self:dispose();
end

function Lua_TaskData:dispose()
	-- 任务
	if self.activePointVoDic ~= nil then
		for k,v in pairs(self.activePointVoDic) do
			v:dispose();
			v = nil;
		end
	end
	if self.singelTaskVoDic ~= nil then
		for k,v in pairs(self.singelTaskVoDic) do
			v:dispose();
			v = nil;
		end
	end
	if self.recordDataArr ~= nil then
		self.recordDataArr:dispose();
		self.recordDataArr:clear();
	end
	self.taskPointDic = nil;

	-- 活跃战令
	self._primaryLockState = nil;
	self._seniorLockState = nil;
	self.roundId = nil; -- 活跃战令轮次id	
	self.activeWarReward = nil;
	self._activeWarShowIdx = nil;

	-- 冒险之旅
	self.curStageId = 1;
	self.curRewardState = 0;
	self.artifactState = 0;
	self.artifactId = nil;
	self.taskRedState = false;
	self.showTaskArr = Array();
	self.openState = false;
	self.selectIdx = 1;
end

-------------------- 通知-日常任务奖励状态信息 --------------------
function Lua_TaskData:fromServer80178001(msg)
	if msg == nil or not DataCenter.funcOpenData:isOpened(10194) then
		return;
	end

	local obj = msg.task;
	self.needSortDic[obj.main_type] = true;
	self:updateTaskListInfo(obj.task_list, true);
	self:showPopupData();
	self:showTaskRedDot(obj.main_type, self:getRewardStateByMainType(obj.main_type)); -- 单个任务类型红点显示

	-- 窗口打开时刷新页面
	if Lua_WindowUtil.isWindowShowing("Lua_TaskBaseWindow") then
		Lua_MsgUtil.callFunc("SELECT_DAILYTASK_PANEL");
	end
end

-- 先冒险弹窗后日常弹窗
function Lua_TaskData:showPopupData()
	if self.showTaskArr:getSize() > 0 then
		if not Lua_WindowUtil.isWindowShowing("Lua_ExploreTourPopupWindow") then
			Lua_WindowUtil.openWin("Lua_ExploreTourPopupWindow", Lua_WinParam.ACTION_OPEN);
		end
	else
		if self.recordDataArr:getSize() > 0 and not Lua_WindowUtil.isWindowShowing("Lua_TaskRewardTipWindow") then
			local popupId = self.recordDataArr:shift();
			local vo = self:getSingleTaskVoById(popupId);
			if vo == nil then
				if getConfig("task", popupId) == nil then
					LogManager.LogError("no task config id：" .. popupId, false);
				else
					LogManager.LogError("task config id is dispose：" .. popupId, false);
				end
				return;
			end
			Lua_WindowUtil.openWin("Lua_TaskRewardTipWindow", Lua_WinParam.ACTION_OPEN, popupId);
		end
	end
end










-------------------- 日常、周常、成就信息 --------------------
function Lua_TaskData:fromServer50178002(msg)
	if msg == nil then
		return;
	end
	if self.taskPointDic == nil then
		self.taskPointDic = {};
	end

	local obj;
	for i = 1, #msg.task_list do
		obj = msg.task_list[i];
		self.needSortDic[obj.main_type] = true;
		self.taskPointDic[obj.main_type] = obj.point; -- 活跃度
		self:updataActivePointInfo(obj);
		self:updateTaskListInfo(obj.task_list);
	end
	self:updateTaskRedPoint();
end

function Lua_TaskData:updataActivePointInfo(taskInfo)
	if taskInfo.main_type ~= Lua_TaskData.achievementTaskTask then -- 更新活跃度奖励是否已领取
		local dic = {};
		for i = 1, #taskInfo.draw_point_list do
			dic[taskInfo.draw_point_list[i]] = 1;
		end

		local arr = DataCenter.taskUtil.activePointIdsDic[taskInfo.main_type];
		local vo;
		for i = 1, arr:getSize() do
			vo = self:getActivePointVoByActiveId(arr[i]);
			if vo ~= nil then
				vo:updateActivePointInfo(taskInfo.point, dic[arr[i]] == 1);
			end
		end
	end
end

function Lua_TaskData:updateTaskListInfo(taskList, isPopUp)
	local obj, vo;
	for i = 1, #taskList do
		obj = taskList[i];
		vo = self:getSingleTaskVoById(obj.id);
		if vo ~= nil then
			vo:updateTaskVoInfo(obj.progress, obj.flag, obj.index);

			-- isPopUp：是否弹窗，弹窗需要记录该任务的id 
			-- flag：任务状态 0未完成 1完成 2已领取
			-- is_inform：是否已弹出过弹窗 0没有，1已弹出过
			if isPopUp and obj.flag == 1 and obj.is_inform == 0 then
				self.recordDataArr:add(vo.id);
			end
		end
	end
end

function Lua_TaskData:updateTaskRedPoint()
	for k,_ in pairs(DataCenter.taskUtil.taskIdsDic) do
		self:showTaskRedDot(k, self:getRewardStateByMainType(k));
	end
end

-- 根据类型（日常、周常、成就任务）获取该任务界面是否有奖励未领取
function Lua_TaskData:getRewardStateByMainType(mainType)
	-- 活跃度奖励
	local hasReward = false;
	if mainType ~= Lua_TaskData.achievementTaskTask then
		hasReward = self:getActivePointRewardStateByMainType(mainType);
		if hasReward then
			return hasReward;
		end
	end

	-- 任务列表奖励
	local idArr = self:getTaskIdsByMainType(mainType);
	local vo;
	for i = 1, idArr:getSize() do
		vo = self:getSingleTaskVoById(idArr[i]);
		if vo.curReceiveState == 1 then
			hasReward = true;
			break;
		end
	end
	return hasReward;
end

-- 根据类型（日常、周常），获取界面是否有活跃度奖励可领取
function Lua_TaskData:getActivePointRewardStateByMainType(mainType)
	local vo;
	local hasReward = false;
	local activePointIdsArr = DataCenter.taskUtil.activePointIdsDic[mainType]
	for i = 1, activePointIdsArr:getSize() do
		vo = self:getActivePointVoByActiveId(activePointIdsArr[i]);
		if vo.rewardState == 1 then
			hasReward = true;
			break;
		end
	end
	return hasReward;
end

-- 根据类型（日常、周常、成就），更新红点
function Lua_TaskData:showTaskRedDot(mainType, redPointState)
	if mainType == Lua_TaskData.dailyTask then
		DataCenter.redPointData:updateRedPointState(Lua_RedPointList.Task_DailyTab, redPointState);
	elseif mainType == Lua_TaskData.weeklyTask then
		DataCenter.redPointData:updateRedPointState(Lua_RedPointList.Task_WeeklyTab, redPointState);
	else
		DataCenter.redPointData:updateRedPointState(Lua_RedPointList.Task_AchievementTab, redPointState);
	end
end

-- 根据id获取活跃度奖励的具体信息（日常、周常）
function Lua_TaskData:getActivePointVoByActiveId(activeId)
	local vo = self.activePointVoDic[activeId];
	if vo == nil then
		local config = getConfig("task_progress", activeId);
		if config then
			vo = Lua_TaskActInfoVo.new(activeId, config);
			self.activePointVoDic[activeId] = vo;
		end
	end
	return vo;
end

-- 根据id获取任务的具体信息（日常、周常、成就）
function Lua_TaskData:getSingleTaskVoById(taskId)
	local vo = self.singelTaskVoDic[taskId];
	if vo == nil then
		local config = getConfig("task", taskId);
		if config then
			vo = Lua_TaskInfoVo.new(taskId, config);
			self.singelTaskVoDic[taskId] = vo;
		end
	end
	return vo;
end

-- 根据类型（日常、周常、成就）获取各自的任务列表
function Lua_TaskData:getTaskIdsByMainType(mainType)
	local arr = DataCenter.taskUtil.taskIdsDic[mainType];
	if self.needSortDic[mainType] then
		self.needSortDic[mainType] = false;
		arr:sortf(self, self.sortTaskFun);
	end
	return arr;
end

-- 可领取 > 未完成 > 未开启 > 已领取
function Lua_TaskData:sortTaskFun(a, b)
	local vo_a = self:getSingleTaskVoById(a);
	local vo_b = self:getSingleTaskVoById(b);
	if vo_a.isOpen and vo_b.isOpen then
		if vo_a.isOpen then
			if vo_a.receiveStage == vo_b.receiveStage then
				return vo_a.sortIndex < vo_b.sortIndex;
			else
				local aWeight = (vo_a.receiveStage + 1) % 3;
				local bWeight = (vo_b.receiveStage + 1) % 3;
				return aWeight > bWeight;
			end
		else
			return vo_a.sortIndex < vo_b.sortIndex;
		end
	else
		if vo_a.isOpen then
			return vo_a.receiveStage ~= 2;
		else
			return vo_b.receiveStage == 2;
		end
	end
	return true;
end

-- 点击主图标按钮时，默认打开有奖励的界面
function Lua_TaskData:getDefaultSelectIdx()
	local defaultSelectIdx = Lua_TaskData.dailyTask;
	local activeWarOpenState = DataCenter.funcOpenData:isOpened(DataCenter.taskUtil.activeWarOpenId, false);
	for k,_ in pairs(DataCenter.taskUtil.taskIdsDic) do
		if k == Lua_TaskData.achievementTaskTask and activeWarOpenState and self.activeWarReward then -- 判断活跃战令是否有奖励，活跃战令排序在成就前面
			defaultSelectIdx = Lua_TaskData.activeWarTask;
			break;
		end
		local hasReward = self:getRewardStateByMainType(k);
		if hasReward then
			defaultSelectIdx = k;
			break;
		end
	end
	return defaultSelectIdx;
end

-- 根据类型（日常、周常）获取活跃度
function Lua_TaskData:getTaskPointByMainType(mainType)
	if mainType == nil or self.taskPointDic == nil or self.taskPointDic[mainType] == nil then
		return nil;
	end
	return self.taskPointDic[mainType];
end










-------------------- 日常任务-活跃战令 --------------------
Lua_TaskData.primaryWarStyle = 1; -- 初级战令
Lua_TaskData.seniorWarStyle = 2; -- 高级战令
function Lua_TaskData:fromServer50178009(msg)
	if msg == nil then
		return;
	end

	self.activeWarPoint = msg.activity; -- 活跃度
	self.activeWarTime = msg.round_time; -- 轮次刷新时间戳
	self._primaryLockState = msg.lock_state == 1; -- 初级奖励解锁状态 0否 1是
	self._seniorLockState = msg.stage_lock_state == 1; -- 高级奖励解锁状态 0否 1是

	self:updateAcitveWarInfo(msg.cur_round); -- 当前轮次
	self:updateAcitveWarShowInfo(msg.reward_state, msg.first_login == 0); -- reward_state：奖励状态，first_login：是否首次登录 0否 1是
end

function Lua_TaskData:updateAcitveWarInfo(roundId)
	if roundId and self.roundId ~= roundId then
		self.roundId = roundId;

		if self.activeWarDic ~= nil then
			local len = self._acitveWarArr:getSize();
			while (len > 0) do
				local key = self._acitveWarArr:remove(len);
				self.activeWarDic[key]:dispose();
				self.activeWarDic[key] = nil;
				len = len - 1;
			end
		else
			self.activeWarDic = {};
			self._acitveWarArr = Array();
		end

		local config = getConfig("active_war_order");
		local vo;
		for k, v in pairs(config) do
			vo = Lua_ActiveWarVo.new(k, v);
			if type(vo.id) == "number" and vo.round == roundId then		
				self._acitveWarArr:add(k);
				self.activeWarDic[k] = vo;
			end
		end
		self._acitveWarArr:sort(nil, false);
	end
end

function Lua_TaskData:updateAcitveWarShowInfo(list, isFirstIn)
	-- table.sort( list, function(a, b) return a.id < b.id end );
	local vo;
	self.activeWarReward = false;
	self._activeWarShowIdx = nil;
	for i = 1, #list do
		vo = self:getActiveWarVoById(list[i].id);
		if vo ~= nil then
			vo:updateActiveWarVoInfo(list[i]);
			if (self._activeWarShowIdx == nil or self._activeWarShowIdx > list[i].id) and not self.activeWarReward then
				if ((vo.primaryRewardState == 1 and self.primaryLockState) or (vo.seniorRewardState == 1 and self.seniorLockState)) then
					self.activeWarReward = true;
					self._activeWarShowIdx = list[i].id;
				end
				if vo.primaryRewardState == 0 then
					self._activeWarShowIdx = list[i].id;
				end
			end
		end
	end
	DataCenter.redPointData:updateRedPointState(Lua_RedPointList.Task_ActiveWarTab, self.activeWarReward or isFirstIn);
end

function Lua_TaskData:getActiveWarVoById(id)
	if id == nil or self.activeWarDic == nil or self.activeWarDic[id] == nil then
		return nil;
	end
	return self.activeWarDic[id];
end

function Lua_TaskData.Get:activeWarShowIdx()
	return self._activeWarShowIdx or 1;
end

function Lua_TaskData.Get:acitveWarArr()
	return self._acitveWarArr;
end

function Lua_TaskData.Get:primaryLockState()
	return self._primaryLockState;
end

function Lua_TaskData.Get:seniorLockState()
	return self._seniorLockState;
end










-------------------- 冒险之旅 --------------------

-- 请求主线任务信息返回
function Lua_TaskData:fromServer50178005(msg)
	self.curStageId = msg.cur_stage; -- 当前阶段
	self.curRewardState = msg.stage_reward_state; -- 0 不可领 1：可领取 2：已领取
	self.artifactState = msg.is_unlock_artifact; -- 1已解锁 0没有神器或者未解锁
	self.artifactId = DataCenter.taskUtil:getExploreTourStageInfo(self.curStageId, "unlock_artifact"); -- 神器id
	self:updateTaskInfo(msg.stage_task_info);
	self:checOpenState();
	Lua_MsgUtil.callFunc("UPDATE_EXPLORER_TOUR_INFO", self.curStageId);
end

-- 请求领取主线任务奖励返回
function Lua_TaskData:fromServer50178006(msg)
	self.curStageId = msg.stage_id; -- 当前阶段
	self.curRewardState = msg.stage_reward_state; -- 0 不可领 1：可领取 2：已领取
	self:updateTaskInfo(msg.task_info);
	self:checOpenState();
	Lua_MsgUtil.callFunc("UPDATE_EXPLORER_TOUR_INFO", self.curStageId);
end

-- 请求解锁主线神器返回
function Lua_TaskData:fromServer50178007(msg)
	self.curStageId = msg.cur_stage_id; -- 当前阶段
	self.curRewardState = msg.stage_reward_state; -- 0 不可领 1：可领取 2：已领取
	self.artifactState = msg.is_unlock_artifact; -- 1已解锁 0没有神器或者未解锁
	self.artifactId = DataCenter.taskUtil:getExploreTourStageInfo(self.curStageId, "unlock_artifact");--神器id
	self:updateTaskInfo(msg.task_info);
	self:checOpenState();

	local lastStep = self.curStageId;
	if self.artifactState == 0 then
		lastStep = lastStep - 1;
	end

	local lastArtifactId = DataCenter.taskUtil:getExploreTourStageInfo(lastStep, "unlock_artifact");
	if lastArtifactId ~= nil then
		Lua_MsgUtil.callFunc("UNLOCK_EXPLORERTOUR_ARTIFACT", lastArtifactId);
	end
	Lua_MsgUtil.callFunc("UPDATE_EXPLORER_TOUR_INFO", self.curStageId);
end

-- 通知主线任务完成
function Lua_TaskData:fromServer80178008(msg)
	if DataCenter.funcOpenData:isOpened(10028) == false then
		return;
	end

	local vo;
	local isShowTask = false;
	for k, v in ipairs(msg.task_list) do
		vo = DataCenter.taskUtil:getExploreTourTaskVo(v.key);
		if vo ~= nil then
			local progress = tonumber(v.value);
			if vo.task_id == 58 and progress ~= 0 then
				vo.taskProgress = Lua_MainLineCheckpointConfigData.getConfig(progress).number;
			else
				vo.taskProgress = tonumber(progress);--任务进度
			end

			if vo.taskProgress >= vo.process then
				isShowTask = true;
				if vo.rewardState == 0 then
					vo.rewardState = 1;
				end
				self.showTaskArr:add(vo);

				-- self:updateTotalTaskData(Lua_TaskData.AdventureJourney_Task, vo.id, vo.conditionId, vo.rewardState);
			end
		end
	end

	Lua_MsgUtil.callFunc("UPDATE_EXPLORER_TOUR_INFO", self.curStageId);
	if isShowTask == false then	
		return;
	end

	self.taskRedState = true;
	self:checOpenState();
	self:showPopupData();
end

-- 更新阶段任务信息
function Lua_TaskData:updateTaskInfo(msg)
	local vo;
	self.taskRedState = false;
	for k, v in ipairs(msg) do
		vo = DataCenter.taskUtil:getExploreTourTaskVo(v.key);
		if vo ~= nil then
			vo:fromServer(v);
			if vo.rewardState == 1 then
				self.taskRedState = true;
			end

			-- self:updateTotalTaskData(Lua_TaskData.AdventureJourney_Task, vo.id, vo.conditionId, vo.rewardState);
		end
	end
end

-- 获取任务弹窗第一个数据
function Lua_TaskData.Get:getPopupTaskVo()
	if self.showTaskArr:getSize() > 0 then
		return self.showTaskArr:shift();
	end
	return nil;
end

-- 获取各阶段的进度值
function Lua_TaskData:getStagePrograss(stage)
	if self.curStageId == nil or self.curStageId < stage then
		return 0, 1;
	elseif self.curStageId > stage then
		return 1, 1;
	else
		local curPrograss = 0;
		local totalPrograss = 0;
		local arr = DataCenter.taskUtil:getExploreTourTaskArr(stage);
		for i = 1, arr:getSize() do
			curPrograss = curPrograss + arr[i].taskProgress;
			totalPrograss = totalPrograss + arr[i].process;
		end
		return curPrograss, totalPrograss;
	end
end

-- 检查主界面显示
function Lua_TaskData:checOpenState()
	local state = DataCenter.funcOpenData:isOpened(10028) and self.isExploreFinish == false;
	if state ~= self.openState then
		self.openState = state;
		Lua_MsgUtil.callFunc("UPDATE_TASK_STATE");
	end

	local artifactReward = self.curRewardState == 2 and self.artifactId ~= nil and self.artifactState == 0;
	DataCenter.redPointData:updateRedPointState(Lua_RedPointList.ExploreTour, self.taskRedState or self.curRewardState == 1 or artifactReward);
end

-- 是否完成所有任务且领完奖励
function Lua_TaskData.Get:isExploreFinish()
	if self.curStageId >= DataCenter.taskUtil:getConstConfig("max_stage", 1) and self.curRewardState == 2 then
		return self.artifactId == nil or self.artifactState == 1;
	end
	return false;
end


---------------------------------------- 外部接口 ----------------------------------------

-- 根据任务id获取冒险之旅任务状态 0：不可领 1：可领取 2：已领取
function Lua_TaskData:getExploreTaskState(taskId)
	local vo = DataCenter.taskUtil:getExploreTourTaskVo(taskId);
	if vo ~= nil then
		if vo.stage > self.curStageId then
			return 0;
		elseif vo.stage < self.curStageId then
			return 2;
		else
			return vo.rewardState;
		end
	end
	return 0;
end

-- 获取冒险之旅当前阶段状态 0：阶段奖励不可领 1：阶段奖励可领取 2：可解锁神器 3：阶段已过 4：阶段未到
function Lua_TaskData:getExploreStageState(stageId)
	if stageId < self.curStageId then
		return 3;
	elseif stageId > self.curStageId then
		return 4;
	else
		if self.curRewardState < 2 then
			return self.curRewardState;
		else
			if self.artifactId ~= nil and self.artifactState == 0 then
				return 2;
			end
		end
		return 0;
	end
end













---------暂时存放引导id数据,测试----------------------
Lua_TaskData.UN_ENABLE_RECEIVE = 0; -- 不可领取
Lua_TaskData.ENABLE_RECEIVE_REWARD = 1; -- 可领取
Lua_TaskData.COMPLETE_TASK = 2; -- 已完成

function Lua_TaskData:resetGuideIdData()
	if self.taskIdDir ~= nil then
		self.taskIdDir = nil;
	end
end

function Lua_TaskData:updateTotalTaskData(taskType, curId, conditionId, state)
	-- print("============任务类型", taskType, "配置表id", curId, "总配置表id", conditionId, "任务状态", state, "===========");
	if self.taskIdDir == nil then
		self.taskIdDir = {};
	end

	if curId == nil and conditionId == nil then
		return;
	end

	-- 登录或初始化
	if self.taskIdDir[taskType] == nil then
		self.taskIdDir[taskType] = {};
	end

	if self.taskIdDir[taskType][curId] == nil then
		if state == Lua_TaskData.ENABLE_RECEIVE_REWARD then
			-- print("初始化", "任务类型", taskType, "配置表id", curId, "总配置表id", conditionId, "任务状态", state)
		elseif state == Lua_TaskData.COMPLETE_TASK then
			-- print("初始化", "任务类型", taskType, "配置表id", curId, "总配置表id", conditionId, "任务状态", state)
		end
			
		self.taskIdDir[taskType][curId] = {conditionId = conditionId, state = state};
	-- 任务状态变更
	else
		if self:getBCompleteTask(taskType, curId, conditionId, state) then
			-- print("任务变更，完成", "任务类型", taskType, "配置表id", curId, "总配置表id", conditionId, "任务状态", state)
		end

		if self:getEnableReceive(taskType, curId, conditionId, state) then
			-- print("任务变更，可领取", "任务类型", taskType, "配置表id", curId, "总配置表id", conditionId, "任务状态", state)
		end
		self.taskIdDir[taskType][curId].state = state;
		self.taskIdDir[taskType][curId].conditionId = conditionId;
	end
end

-- 任务状态是否转变为<已完成>
function Lua_TaskData:getBCompleteTask(taskType, curId, conditionId, state)
	local info = self.taskIdDir[taskType][curId];
	if info == nil then
		return;
	end
	local enableComplete = info.conditionId ~= conditionId;
	if not enableComplete then
		enableComplete = info.state == Lua_TaskData.ENABLE_RECEIVE_REWARD and state == Lua_TaskData.COMPLETE_TASK;
	end
	return enableComplete;
end

-- 任务状态是否转变为<可领取>
function Lua_TaskData:getEnableReceive(taskType, curId, conditionId, state)
	local info = self.taskIdDir[taskType][curId];
	if info == nil then
		return;
	end
	local enableReceive = info.conditionId ~= conditionId and state == Lua_TaskData.ENABLE_RECEIVE_REWARD;
	if not enableReceive then
		enableReceive = info.state == Lua_TaskData.UN_ENABLE_RECEIVE and state == Lua_TaskData.ENABLE_RECEIVE_REWARD;
	end
	return enableReceive;
end