checkRequire("Lua_WildStageFloorVo");
checkRequire("Lua_WildMineRegionVo");
Lua_WildSecretData = class("Lua_WildSecretData")
function Lua_WildSecretData:ctor()
	self:superCall("ctor");
	self:dispose();
	self.stageMsg = {};
	self.selfLodeDic = {};
end

function Lua_WildSecretData:dispose()
	if self.stageMsg then
		table.clear(self.stageMsg);
	end
	if self._regionArr and self._regionDic then
		local id;
		for i = 1, self._regionArr:getSize() do
			id = self._regionArr[i];
			if self._regionDic[id] then
				self._regionDic[id]:fromServer(nil);
			end
		end
	end

	self._stage_settle_tick = nil;
	self._stage_id = nil;
	self._is_finish = nil;
	self._profit_per_hour = nil;
	self._daily_reward_stage_id = nil;
	self._stage_num = nil;
	self._stageRewardState = nil;

	self._mine_settle_tick = nil;
	self._left_grab_times = nil;
	self._total_profit = nil;
	self._mine_info = nil;
	self._occupy_num = nil;
	self._unlock_num = nil;

	self.reportList = nil;
end

function Lua_WildSecretData:getStageFightSceneId(stageId)
	local floor = getConfig("wild_secret_stage", stageId, "floor") or 1;
	return getConfig("wild_secret_layer", floor, "fight_scene") or 0;
end

function Lua_WildSecretData:getLodeFightSceneId(stageId)
	local region = getConfig("wild_secret_mine_site", stageId, "region") or 1;
	return getConfig("wild_secret_mine_region", region, "fight_scene") or 0;
end

function Lua_WildSecretData.Get:stageFloorArr()
	if self._floorArr == nil then
		self._floorArr = Array();
		local config = getConfig("wild_secret_layer");
		for k, _ in pairs(config) do
			self._floorArr:add(k);
		end
		-- quickSort(self._floorArr, nil, false);
	end
	return self._floorArr;
end

function Lua_WildSecretData:getStageFloorVo(id)
	if self._floorDic == nil then
		self._floorDic = {};
	end
	local vo = self._floorDic[id];
	if vo == nil then
		local config = getConfig("wild_secret_layer", id);
		if config then
			vo = Lua_WildStageFloorVo.new(id, config);
			self._floorDic[id] = vo;
		end
	end
	return vo;
end

function Lua_WildSecretData:fromServer50218002(msg)
	local floor = getConfig("wild_secret_stage", msg.stage_id, "floor") or 1;
	if floor ~= self.curFloor then
		Lua_WindowUtil.openWin("Lua_WildStageWindow", Lua_WinParam.ACTION_CLOSE);
	end

	self._stage_settle_tick = msg.settle_tick; -- 结算时间
	self._stage_id = msg.stage_id; -- 当前关卡id
	self._is_finish = msg.is_finish; -- 是否通关当天关卡 1：通关 0：未通关
	self._profit_per_hour = msg.profit_per_hour; -- 当天关卡的每小时收益
	self._daily_reward_stage_id = msg.daily_reward_stage_id; -- 0：没有每日奖励 对应的关卡id
	self._stage_num = msg.stage_num; -- 当天通关数量

	table.clear(self.stageMsg);
	local info;
	for i = 1, #msg.stage_info do
		info = msg.stage_info[i];
		self.stageMsg[info.stage] = info;
	end

	self:checkStageRedpoint();
end

function Lua_WildSecretData:getFloorServerInfo(floorId)
	return self.stageMsg[floorId];
end

function Lua_WildSecretData.Get:stageSettleTick()
	return self._stage_settle_tick or 0;
end

function Lua_WildSecretData.Get:curStageId()
	return self._stage_id or 1001;
end

function Lua_WildSecretData.Get:curStageCfg()
	return DataCenter.wildSecretUtil:getStageConfig(self.curStageId);
end

function Lua_WildSecretData.Get:isFinish()
	return self._is_finish == 1;
end

function Lua_WildSecretData.Get:curIdx()
	local index = getConfig("wild_secret_stage", self.curStageId, "index") or 1;
	if self.isFinish then
		return index;
	end
	return index - 1;
end

-- 当前层数
function Lua_WildSecretData.Get:curFloor()
	return getConfig("wild_secret_stage", self.curStageId, "floor") or 1;
end

function Lua_WildSecretData.Get:dailyRewardStageId()
	return self._daily_reward_stage_id or 0;
end

function Lua_WildSecretData.Get:stageNum()
	return self._stage_num or 0;
end

function Lua_WildSecretData.Get:isFightLimit()
	return self.stageNum >= DataCenter.wildSecretUtil.dailyStageLimit;
end

function Lua_WildSecretData.Get:profitPerHour()
	return self._profit_per_hour or 0;
end

function Lua_WildSecretData.Get:isFightSkip()
	return self.curStageId >= DataCenter.wildSecretUtil.fightSkipLimit;
end

-- 检测关卡是否解锁
function Lua_WildSecretData:checkStageIsUnlock(stageId)
	if stageId == self.curStageId then
		return self.isFinish;
	end
	return self.curStageId > stageId;
end

function Lua_WildSecretData:fromServer50218003(msg)
	DataCenter.wildSecretUtil.bStageRewardSort = true;
	local state = false;
	self._stageRewardState = {};
	local info;
	for i = 1, #msg.reward_info do
		info = msg.reward_info[i];
		if info.value == 1 then
			state = true;
		end
		self._stageRewardState[info.key] = info.value;
	end
	DataCenter.redPointData:updateRedPointState(Lua_RedPointList.WildSecret_Stage_Reward, state);
end

-- 获取关卡奖励状态
function Lua_WildSecretData:getStageRewardState(id)
	if self._stageRewardState == nil then
		return 0;
	end
	return self._stageRewardState[id] or 0;
end

function Lua_WildSecretData:fromServer50218005(msg)
	self._daily_reward_stage_id = msg.daily_reward_stage_id;
	self:checkStageRedpoint();
end





function Lua_WildSecretData.Get:mineRegionArr()
	if self._regionArr == nil then
		self._regionArr = Array();
		local config = getConfig("wild_secret_mine_region");
		for k, _ in pairs(config) do
			self._regionArr:add(k);
		end
		quickSort(self._regionArr, nil, false);
	end
	return self._regionArr;
end

function Lua_WildSecretData:getMineRegionVo(id)
	if self._regionDic == nil then
		self._regionDic = {};
	end
	local vo = self._regionDic[id];
	if vo == nil then
		local config = getConfig("wild_secret_mine_region", id);
		if config then
			vo = Lua_WildMineRegionVo.new(id, config);
			self._regionDic[id] = vo;
		end
	end
	return vo;
end

function Lua_WildSecretData:fromServer50218001(msg)
	self._total_profit = msg.total_profit; -- 总收益
	self._mine_info = msg.mine_info; -- 矿点占领信息
	self._occupy_num = msg.occupy_num; -- 占领矿点个数
	self._unlock_num = msg.unlock_num; -- 解锁矿点个数
end

function Lua_WildSecretData:fromServer50218009(msg)
	table.clear(self.selfLodeDic);
	local info;
	local state = false;
	for i = 1, #msg.mite_site_list do
		info = msg.mite_site_list[i];
		if info.left_output_tick > 0 and info.mine_site_id == 0 then
			state = true;
		end
		self.selfLodeDic[info.num] = info.mine_site_id;
	end
	DataCenter.redPointData:updateRedPointState(Lua_RedPointList.WildSecret_Lode_Occupy, state);
end

function Lua_WildSecretData:getSelfLodeIdByNum(num)
	return self.selfLodeDic[num] or 0;
end

function Lua_WildSecretData:fromServer50218021(msg)
	self._mine_settle_tick = msg.settle_tick; -- 结算时间
	self._left_grab_times = msg.left_grab_times; -- 剩余抢夺次数
	self._total_profit = msg.total_profit; -- 总收益
	self._occupy_num = msg.occupy_num; -- 占领矿点个数
	self._unlock_num = msg.unlock_num; -- 解锁矿点个数

	local info;
	local vo;
	for i = 1, #msg.region_info do
		info = msg.region_info[i];
		vo = self:getMineRegionVo(info.region);
		if vo then
			vo:fromServer(info);
		end
	end

	DataCenter.redPointData:updateRedPointState(Lua_RedPointList.WildSecret_Lode_Plunder, self.leftGrabTimes > 0);
end

function Lua_WildSecretData.Get:mineSettleTick()
	return self._mine_settle_tick or 0;
end

function Lua_WildSecretData.Get:leftGrabTimes()
	return self._left_grab_times or 0;
end

function Lua_WildSecretData.Get:totalProfit()
	return self._total_profit or 0;
end

function Lua_WildSecretData.Get:mineInfo()
	return self._mine_info;
end

function Lua_WildSecretData.Get:occupyNum()
	return self._occupy_num or 0;
end

function Lua_WildSecretData.Get:unlockNum()
	return self._unlock_num or 0;
end

function Lua_WildSecretData.Get:isCanOccupy()
	return self.occupyNum < self.unlockNum; 
end

function Lua_WildSecretData:fromServer80218014(msg)
	self.reportList = msg.report_list;
end









function Lua_WildSecretData:checkStageRedpoint()
	DataCenter.redPointData:updateRedPointState(Lua_RedPointList.WildSecret_Stage_DailyReward, self.dailyRewardStageId > 0);
end