checkRequire("Lua_TimeSpaceRaceItem");
Lua_TimeSpaceRacePlayer = class("Lua_TimeSpaceRacePlayer")
function Lua_TimeSpaceRacePlayer:setPlayer(player)
	self.isActive = true;
	self.bLoadCompleted = false;
	self._player = player;
end

function Lua_TimeSpaceRacePlayer.Get:player()
	return self._player;
end

function Lua_TimeSpaceRacePlayer:initData()
	self.data = self.player.data;
	
	self.uniqueId = tonumber(tostring(self.data.roleId)); -- 玩家唯一id
	self.objId = tonumber(tostring(self.data.objId)); -- objId
	self.sceneId = self.data.sceneId; -- 场景
	self.fightRecordId = tonumber(tostring(self.data.fightRecordId)); -- 战斗id
	self.formationType = self.data.formationType; -- 攻守方
	self.formationPos = self.data.formationPos; -- 阵型位置
	DataCenter.thingData:addThingPlayer(self, self.uniqueId, self.objId, self.sceneId, self.fightRecordId, self.formationType, self.formationPos);

	if self.headContainer == nil then
		self.headContainer = UICreate.createGameObject(GlobalVar.uiHeadContainer);
		self.headContainer:AddComponent(typeof(UnityEngine.RectTransform));
		self.headContainer.name = self.uniqueId.."_headContainer";
	end
	if self.infoItem == nil then
		self.infoItem = Lua_TimeSpaceRaceItem.new(self.headContainer);
	end
	self:updateData();
	self:refreshActive(true);
end

function Lua_TimeSpaceRacePlayer:loadBodyComplete()
	self.bLoadCompleted = true;
	self:updateThingInfo();
end

function Lua_TimeSpaceRacePlayer:updateThingInfo()
	if self.player == nil or self.player.IsUI or not self.data.isShowName then
		self:refreshActive(false);
		return;
	end

	if self.player.viewState then
		if self.headContainer == nil then
			self.headContainer = UICreate.createGameObject(GlobalVar.uiHeadContainer);
			self.headContainer:AddComponent(typeof(UnityEngine.RectTransform));
			self.headContainer.name = self.uniqueId.."_headContainer";
		end
		if self.infoItem == nil then
			self.infoItem = Lua_TimeSpaceRaceItem.new(self.headContainer);
		end
		self:updateData();
		self:refreshActive(self.player.bActive);
	else
		self:refreshActive(false);
	end
end

function Lua_TimeSpaceRacePlayer:updateData()
	if self.infoItem == nil then
		return;
	end

	local info = DataCenter.timeSpaceRaceData:getEnemyInfo(self.objId);
	if info == nil then
		self.player:removeEffect(Lua_ThingDefine.PLAYER_EFFECT, self.hangerId);
		self.hangerId = nil;
		self.player:setActive(false);
		self:refreshActive(false);
		return;
	end
	
	local hangerId;
	if info.type == 2 or info.type == 3 then
		hangerId = "xui_skzb_widget_25";
	else
		hangerId = (info.rank > 3) and "xui_skzb_widget_14" or "xui_skzb_widget_13";
	end
	if hangerId ~= self.hangerId then
		self.player:removeEffect(Lua_ThingDefine.PLAYER_EFFECT, self.hangerId);
		self.player:addHangerEffect(Lua_ThingDefine.PLAYER_EFFECT, hangerId);
		self.hangerId = hangerId;
	end
	self.infoItem:setData(info);
end

function Lua_TimeSpaceRacePlayer:refreshHeadPos()
	if not self.isActive or self.headContainer == nil or self.infoItem == nil or self.data == nil then
		return;
	end
	LuaUtil.setUIObjPos(self.headContainer, self.infoItem.content, self.data.px * 0.01, self.data.py * 0.01, self.data.pz * 0.01, 0, 0, 0);
end

function Lua_TimeSpaceRacePlayer:refreshActive(value)
	-- print("refreshActive", self.uniqueId, self.isActive, value, getTraceback())
	-- if self.isActive == value then
	-- 	return;
	-- end
	self.isActive = value;

	if self.headContainer then
		self.headContainer:SetActive(value);
	end
	if value == true then
		self:refreshHeadPos();
	end
end

function Lua_TimeSpaceRacePlayer:getLuaData()

end

function Lua_TimeSpaceRacePlayer:showClickInfo(clickStr)
	local info = DataCenter.timeSpaceRaceData:getEnemyInfo(self.objId);
	if info == nil or info.type ~= 0 then
		OperateAlert.showLangValue(770028);
	else
		Lua_ProxyManager.getProxy("Lua_RoleProxy"):sOtherRoleId(self.objId, info.show_role.server_flag);
	end
end

function Lua_TimeSpaceRacePlayer:clearRes()
	self.bLoadCompleted = false;
	if self.infoItem then
		self.infoItem:dispose();
		self.infoItem = nil;
	end
	if self.headContainer then
		UnityEngine.GameObject.Destroy(self.headContainer);
		self.headContainer = nil;
	end
end

function Lua_TimeSpaceRacePlayer:dispose()
	self.isDispose = true;
	if not self.player.IsUI then
		DataCenter.thingData:removeThingPlayer(self, self.uniqueId, self.objId, self.sceneId, self.fightRecordId, self.formationType, self.formationPos);
	end
	self:clearRes();
	self._player = nil;
end